@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-06-17 22:25:31 jcs>
+/* Time-stamp: <2006-05-01 15:04:54 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: charset.c,v 1.23 2005/06/17 13:30:02 jcsjcs Exp $
+|  $Id: charset.c 955 2007-01-16 10:15:27Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -41,7 +41,7 @@
    set with each call of charset_to_utf8(). You can get a copy of its
    value by calling charset_get_auto().
    This variable will only be reset by calling charset_reset_auto(). */
-static const gchar *auto_charset = NULL;
+static gchar *auto_charset = NULL;
 
 typedef struct {
 	gchar *descr;
@@ -154,7 +154,7 @@ void charset_init_combo (GtkCombo *combo)
     static GList *charsets = NULL; /* list with choices -- takes a while to
 				     * initialize, so we only do it once */
     
-    current_charset = prefs_get_charset ();
+    current_charset = prefs_get_string("charset");
     if ((current_charset == NULL) || (strlen (current_charset) == 0))
     {
 	description = g_strdup (_("System Charset"));
@@ -206,6 +206,7 @@ void charset_init_combo (GtkCombo *combo)
     /* set standard entry */
     gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), description);
     g_free (description);
+    g_free(current_charset);
 }
 
 
@@ -264,49 +265,66 @@ gchar *charset_to_description (gchar *charset)
 
 /* code for automatic detection of Japanese char-subset donated by
    Hiroshi Kawashima */
-static const gchar *charset_check_k_code (const guchar *p)
+static const gchar *charset_check_k_code (const gchar *p2)
 {
-    while (p && *p && (*p != '\n'))
-    {
-	if ((p[0] == 0x1b) &&
-	    p[1] && (p[1] == 0x24) &&
-	    p[2] && (p[2] == 0x42))	    return ("ISO-2022-JP");
-	if ((p[0] == 0x1b) &&
-	    p[1] && (p[1] == 0x24) &&
-	    p[2] && (p[2] == 0x40))	    return ("ISO-2022-JP");
-	if (((p[0] >= 0x81) && (p[0] <= 0x9f)) ||
-	    (p[0] >= 0xe0)) 	            return ("Shift_JIS");
-	if ((p[0] >= 0xa1) && (p[0] <= 0xfe) && p[1] &&
-	    (((p[1] >= 0x21) && (p[1] <= 0x7e)) ||
-	     ((p[1] >= 0xa1) && (p[1] <= 0xfe))))
-	                                    return ("EUC-JP");
-	++p;
+    static const char* charsets[] = {
+	    "UTF-8",
+	    "EUC-JP",
+	    "CP932",
+	    "ISO-2022-JP",
+	    NULL
+    };
+    int i;
+    gchar *ret;
+    gssize len;
+
+    if (p2 == NULL) return NULL;
+ 
+    len = strlen ((gchar*)p2);
+    for (i=0; charsets[i]; i++) {
+      ret = g_convert ((gchar*)p2,            /* string to convert */
+		       len,                  /* length of string  */
+		       "UTF-8",              /* to_codeset        */
+		       charsets[i],          /* from_codeset      */
+		       NULL,                 /* *bytes_read       */
+		       NULL,                 /* *bytes_written    */
+		       NULL);                /* GError **error    */
+      if (ret != NULL) {
+        g_free(ret);
+        return charsets[i];
+      }
     }
     return (NULL);
 }
 
-/* same as check_k_code, but defaults to "EUC-JP" if no match is found */
+/* same as check_k_code, but defaults to "UTF-8" if no match is found */
 static const gchar *charset_check_k_code_with_default (const guchar *p)
 {
     const gchar *result=NULL;
 
     if (p)       result = charset_check_k_code (p);
-    if (!result) result = "EUC-JP";
+    if (!result) result = "UTF-8";
     return result;
 }
 
-
 /* return the charset actually used for the "auto detection"
  * feature. So far only Japanese Auto Detecion is implemented */
-static const gchar *charset_check_auto (const gchar *str)
+static gchar *charset_check_auto (const gchar *str)
 {
-    const gchar *charset;
+    gchar *charset;
+    gchar *result;
 
     if (str == NULL) return NULL; /* sanity */
-    charset = prefs_get_charset ();
+
+    charset = prefs_get_string("charset");
+
     if (charset && (strcmp (charset, GTKPOD_JAPAN_AUTOMATIC) == 0))
-	return (charset_check_k_code (str));
-    return NULL;
+	result = g_strdup(charset_check_k_code (str));
+    else
+	result = NULL;
+
+    g_free(charset);
+    return result;
 }
 
 /* See description at the definition of gchar *auto_charset; for
@@ -328,23 +346,31 @@ void charset_reset_auto (void)
 /* Must free the returned string yourself */
 gchar *charset_to_utf8 (const gchar *str)
 {
-    const gchar *charset;
+    gchar *charset;  /* From prefs */
+    const gchar *locale_charset; /* Used if prefs doesn't have a charset */
+    gchar *result;
 
     if (str == NULL) return NULL;  /* sanity */
     charset = charset_check_auto (str);
     if (charset)
     {
-	auto_charset = charset;
+	g_free(auto_charset);
+	auto_charset = g_strdup(charset);
     }
     else
     {
-	charset = prefs_get_charset ();
+	charset = prefs_get_string("charset");
 	if (!charset || !strlen (charset))
 	{    /* use standard locale charset */
-	    g_get_charset (&charset);
+	    g_free(charset);
+	    g_get_charset (&locale_charset);
+	    charset = g_strdup(locale_charset);
 	}
     }
-    return charset_to_charset ((gchar *)charset, "UTF-8", str);
+    
+    result = charset_to_charset (charset, "UTF-8", str);
+    g_free(charset);
+    return result;
 }
 
 
@@ -354,15 +380,23 @@ gchar *charset_to_utf8 (const gchar *str)
 /* Must free the returned string yourself */
 gchar *charset_from_utf8 (const gchar *str)
 {
-    const gchar *charset;
+    gchar *charset;
+    const gchar *locale_charset;
+    gchar *result;
 
     if (str == NULL) return NULL;  /* sanity */
-    charset = prefs_get_charset ();
+    charset = prefs_get_string("charset");
     if (!charset || !strlen (charset))
-    {    /* use standard locale charset */
-	g_get_charset (&charset);
+    {   
+       /* use standard locale charset */
+	g_free(charset);
+	g_get_charset (&locale_charset);
+	charset = g_strdup(locale_charset);
     }
-    return charset_to_charset ("UTF-8", (gchar *)charset, str);
+
+    result = charset_to_charset ("UTF-8", charset, str);
+    g_free(charset);
+    return result;
 }
 
 /* Convert "str" from utf8 to the charset specified in @s->charset. If
@@ -371,7 +405,9 @@ gchar *charset_from_utf8 (const gchar *str)
 /* Must free the returned string yourself */
 gchar *charset_track_charset_from_utf8 (Track *s, const gchar *str)
 {
-    const gchar *charset = NULL;
+    gchar *charset;
+    const gchar *locale_charset;
+    gchar *result;
     ExtraTrackData *etd;
 
     g_return_val_if_fail (s, NULL);
@@ -382,13 +418,20 @@ gchar *charset_track_charset_from_utf8 (Track *s, const gchar *str)
     etd = s->userdata;
 
     if (etd->charset && strlen (etd->charset))
-	   charset = etd->charset;
-    else   charset = prefs_get_charset ();
+	   charset = g_strdup(etd->charset);
+    else   
+	charset = prefs_get_string("charset");
+   
     if (!charset || !strlen (charset))
     {    /* use standard locale charset */
-	g_get_charset (&charset);
+	g_free(charset);
+	g_get_charset (&locale_charset);
+	charset = g_strdup(locale_charset);
     }
-    return charset_to_charset ("UTF-8", (gchar *)charset, str);
+
+    result = charset_to_charset ("UTF-8", charset, str);
+    g_free(charset);
+    return result;
 }
 
 /* Convert "str" from "from_charset" to "to_charset", trying to skip
