@@ -1,8 +1,9 @@
-/* Time-stamp: <2004-02-04 21:19:22 JST jcs>
+/* Time-stamp: <2005-06-17 22:25:31 jcs>
 |
-|  Copyright (C) 2002 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: charset.h,v 1.13 2004/02/04 12:23:52 jcsjcs Exp $
+|  $Id: charset.h 589 2005-06-17 13:30:05Z jcsjcs $
 */
 
 #ifndef __CHARSET_H__
@@ -35,7 +36,7 @@
 
 #include <gtk/gtk.h>
 #include <stdio.h>
-#include "track.h"
+#include "itdb.h"
 
 #define GTKPOD_JAPAN_AUTOMATIC "gtkpod-japan-automatic"
 
