@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-09-21 23:31:59 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: details.c,v 1.16 2006/09/21 15:03:13 jcsjcs Exp $
+|  $Id: details.c 1755 2007-11-05 01:16:19Z phantom_sf $
 */
 
 /* This file provides functions for the details window */
@@ -34,9 +33,13 @@
 #include "details.h"
 #include "fileselection.h"
 #include "misc.h"
+#include "fetchcover.h"
+#include "display_coverart.h"
 #include "misc_track.h"
 #include "prefs.h"
+#include "info.h"
 #include <string.h>
+#include <glib/gstdio.h>
 
 /*
 void details_close (void);
@@ -48,30 +51,55 @@ void details_remove_track (Track *track);
 /* List with all detail windows */
 static GList *details = NULL;
 
-struct _Detail
-{
-    GladeXML *xml;      /* XML info                           */
-    GtkWidget *window;  /* pointer to details window          */
-    iTunesDB *itdb;     /* pointer to the original itdb       */
-    GList *orig_tracks; /* tracks displayed in details window */
-    GList *tracks;      /* tracks displayed in details window */
-    Track *track;       /* currently displayed track          */
-    gboolean changed;   /* at lease one track was changed     */
-
-};
-
-typedef struct _Detail Detail;
-
-
 /* string constants for preferences */
 static const gchar *DETAILS_WINDOW_DEFX="details_window_defx";
 static const gchar *DETAILS_WINDOW_DEFY="details_window_defy";
 static const gchar *DETAILS_WINDOW_NOTEBOOK_PAGE="details_window_notebook_page";
 
+/* enum types */
+typedef enum
+{
+    DETAILS_MEDIATYPE_AUDIO_VIDEO = 0,
+    DETAILS_MEDIATYPE_AUDIO,
+    DETAILS_MEDIATYPE_MOVIE,
+    DETAILS_MEDIATYPE_PODCAST,
+    DETAILS_MEDIATYPE_VIDEO_PODCAST,
+    DETAILS_MEDIATYPE_AUDIOBOOK,
+    DETAILS_MEDIATYPE_MUSICVIDEO,
+    DETAILS_MEDIATYPE_TVSHOW,
+    DETAILS_MEDIATYPE_MUSICVIDEO_TVSHOW
+} DETAILS_MEDIATYPE;
+
+typedef struct
+{
+    guint32 id;
+    const gchar *str;
+} ComboEntry;
+
+/* strings for mediatype combobox */
+static const ComboEntry mediatype_comboentries[] =
+{
+    { 0,                                               N_("Audio/Video") },
+    { ITDB_MEDIATYPE_AUDIO,                            N_("Audio") },
+    { ITDB_MEDIATYPE_MOVIE,                            N_("Video") },
+    { ITDB_MEDIATYPE_PODCAST,                          N_("Podcast") },
+    { ITDB_MEDIATYPE_PODCAST|ITDB_MEDIATYPE_MOVIE,     N_("Video Podcast") },
+    { ITDB_MEDIATYPE_AUDIOBOOK,                        N_("Audiobook") },
+    { ITDB_MEDIATYPE_MUSICVIDEO,                       N_("Music Video") },
+    { ITDB_MEDIATYPE_TVSHOW,                           N_("TV Show") },
+    { ITDB_MEDIATYPE_TVSHOW|ITDB_MEDIATYPE_MUSICVIDEO, N_("TV Show & Music Video") },
+    { 0,                                               NULL }
+};
+
+/* Detail image drag types for dnd */
+GtkTargetEntry cover_image_drag_types [] = {
+		{ "image/jpeg", 0, DND_IMAGE_JPEG },
+		{ "text/uri-list", 0, DND_TEXT_URI_LIST },
+		{ "text/plain", 0, DND_TEXT_PLAIN },
+		{ "STRING", 0, DND_TEXT_PLAIN }
+};
 
 /* Declarations */
-static void details_update_buttons (Detail *detail);
-static void details_update_thumbnail (Detail *detail);
 static void details_set_track (Detail *detail, Track *track);
 static void details_free (Detail *detail);
 static void details_get_item (Detail *detail, T_item item,
@@ -80,8 +108,9 @@ static void details_get_changes (Detail *detail);
 static gboolean details_copy_artwork (Track *frtrack, Track *totrack);
 static void details_undo_track (Detail *detail, Track *track);
 static void details_update_headline (Detail *detail);
-
-
+static gboolean dnd_details_art_drag_drop(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, gpointer user_data);
+static void dnd_details_art_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x, gint y, GtkSelectionData *data, guint info, guint time, gpointer user_data);
+static gboolean dnd_details_art_drag_motion (GtkWidget *widget, GdkDragContext *dc, gint x, gint y, guint time, gpointer user_data);
 
 /* Store the window size */
 static void details_store_window_state (Detail *detail)
@@ -104,7 +133,7 @@ static void details_store_window_state (Detail *detail)
 
 
 /* Query the state of the writethrough checkbox */
-static gboolean details_writethrough (Detail *detail)
+gboolean details_writethrough (Detail *detail)
 {
     GtkWidget *w;
 
@@ -171,6 +200,22 @@ static void details_checkbutton_toggled (GtkCheckButton *button,
 }
 
 
+static void details_combobox_changed (GtkComboBox *combobox,
+					 Detail *detail)
+{
+    T_item item;
+
+    g_return_if_fail (combobox);
+
+    item = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (combobox),
+					       "details_item"));
+    
+    g_return_if_fail ((item > 0) && (item < T_ITEM_NUM));
+
+    details_get_item (detail, item, FALSE);
+}
+
+
 static void details_writethrough_toggled (GtkCheckButton *button,
 					  Detail *detail)
 {
@@ -266,6 +311,7 @@ static void details_button_set_artwork_clicked (GtkButton *button,
 		g_return_if_fail (etr);
 		gp_track_set_thumbnails (tr, filename);
 		etr->tchanged = TRUE;
+		etr->tartwork_changed = TRUE;
 	    }
 	}
 	else
@@ -274,6 +320,7 @@ static void details_button_set_artwork_clicked (GtkButton *button,
 	    g_return_if_fail (etr);
 	    gp_track_set_thumbnails (detail->track, filename);
 	    etr->tchanged = TRUE;
+	    etr->tartwork_changed = TRUE;
 	}
 	detail->changed = TRUE;
 	details_update_thumbnail (detail);
@@ -324,6 +371,7 @@ static void details_button_apply_clicked (GtkButton *button,
 {
     GList *gl, *gl_orig;
     gboolean changed = FALSE;
+    GList *changed_tracks = NULL;
 
     g_return_if_fail (detail);
 
@@ -356,8 +404,18 @@ static void details_button_apply_clicked (GtkButton *button,
 
 	    if (tr_changed)
 	    {
+		tr_orig->time_modified = time (NULL);
 		pm_track_changed (tr_orig);
 	    }
+
+	    if (prefs_get_int("id3_write"))
+	    {
+		/* add tracks to a list because write_tags_to_file()
+		   can remove newly created duplicates which is not a
+		   good idea from within a for() loop over tracks */
+		changed_tracks = g_list_prepend (changed_tracks, tr_orig);
+	    }
+
 	    changed |= tr_changed;
 	    etr->tchanged = FALSE;
 	}
@@ -366,7 +424,24 @@ static void details_button_apply_clicked (GtkButton *button,
     detail->changed = FALSE;
 
     if (changed)
+    {
 	data_changed (detail->itdb);
+    }
+    
+    if (prefs_get_int("id3_write"))
+    {
+	if (changed_tracks)
+	{
+	    for (gl=changed_tracks; gl; gl=gl->next)
+	    {
+		Track *tr = gl->data;
+		write_tags_to_file (tr);
+		/* display possible duplicates that have been removed */
+	    }
+	    gp_duplicate_remove (NULL, NULL);
+	}
+    }
+    g_list_free (changed_tracks);
 
     details_update_headline (detail);
 
@@ -461,14 +536,14 @@ static void details_delete_event (GtkWidget *widget,
 }
 
 
-/****** Copy artwork data if filaname has changed ****** */
+/****** Copy artwork data if filename has changed ****** */
 static gboolean details_copy_artwork (Track *frtrack, Track *totrack)
 {
-    gboolean changed = FALSE;
-    ExtraTrackData *fretr, *toetr;
+		gboolean changed = FALSE;
+  	ExtraTrackData *fretr, *toetr;
 
-    g_return_val_if_fail (frtrack, FALSE);
-    g_return_val_if_fail (totrack, FALSE);
+  	g_return_val_if_fail (frtrack, FALSE);
+  	g_return_val_if_fail (totrack, FALSE);
 
     fretr = frtrack->userdata;
     toetr = totrack->userdata;
@@ -478,26 +553,29 @@ static gboolean details_copy_artwork (Track *frtrack, Track *totrack)
 
     g_return_val_if_fail (fretr->thumb_path_locale, FALSE);
     g_return_val_if_fail (toetr->thumb_path_locale, FALSE);
-
-    if (strcmp (fretr->thumb_path_locale, toetr->thumb_path_locale) != 0)
-    {
-	itdb_artwork_free (totrack->artwork);
-	totrack->artwork = itdb_artwork_duplicate (frtrack->artwork);
-	totrack->artwork_size = frtrack->artwork_size;
-	totrack->artwork_count = frtrack->artwork_count;
-	totrack->has_artwork = frtrack->has_artwork;
-	g_free (toetr->thumb_path_locale);
-	g_free (toetr->thumb_path_utf8);
-	toetr->thumb_path_locale = g_strdup (fretr->thumb_path_locale);
-	toetr->thumb_path_utf8 = g_strdup (fretr->thumb_path_utf8);
-	changed = TRUE;
-    }
+	
+	if (strcmp (fretr->thumb_path_locale, toetr->thumb_path_locale) != 0
+			|| fretr->tartwork_changed == TRUE)
+  {
+		itdb_artwork_free (totrack->artwork);
+		totrack->artwork = itdb_artwork_duplicate (frtrack->artwork);
+		totrack->artwork_size = frtrack->artwork_size;
+		totrack->artwork_count = frtrack->artwork_count;
+		totrack->has_artwork = frtrack->has_artwork;
+		g_free (toetr->thumb_path_locale);
+		g_free (toetr->thumb_path_utf8);
+		toetr->thumb_path_locale = g_strdup (fretr->thumb_path_locale);
+		toetr->thumb_path_utf8 = g_strdup (fretr->thumb_path_utf8);
+		toetr->tartwork_changed = TRUE;
+		changed = TRUE;
+	}
     /* make sure artwork gets removed, even if both thumb_paths were
        unset ("") */
     if (!frtrack->artwork->thumbnails)
     {
 	changed |= gp_track_remove_thumbnails (totrack);
     }
+        
     return changed;
 }
 
@@ -549,6 +627,55 @@ static void details_get_changes (Detail *detail)
 }
 
 
+/****** comboentries helper functions ******/
+
+/* Get index from ID (returns -1 if ID could not be found) */
+static gint comboentry_index_from_id (const ComboEntry centries[],
+				      guint32 id)
+{
+    gint i;
+
+    g_return_val_if_fail (centries, -1);
+
+    for (i=0; centries[i].str; ++i)
+    {
+	if (centries[i].id == id)  return i;
+    }
+    return -1;
+}
+
+
+/* initialize a combobox with the corresponding entry strings */
+static void details_setup_combobox (GtkWidget *cb,
+				    const ComboEntry centries[])
+{
+    const ComboEntry *ce = centries;
+    GtkCellRenderer *cell;
+    GtkListStore *store;
+
+    g_return_if_fail (cb);
+    g_return_if_fail (centries);
+
+    /* clear any renderers that may have been set */
+    gtk_cell_layout_clear (GTK_CELL_LAYOUT (cb));
+    /* set new model */
+    store = gtk_list_store_new (1, G_TYPE_STRING);
+    gtk_combo_box_set_model (GTK_COMBO_BOX (cb), GTK_TREE_MODEL (store));
+    g_object_unref (store);
+
+    cell = gtk_cell_renderer_text_new ();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (cb), cell, TRUE);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (cb), cell,
+				    "text", 0,
+				    NULL);
+
+    while (ce->str != NULL)
+    {
+	gtk_combo_box_append_text (GTK_COMBO_BOX (cb), _(ce->str));
+	++ce;
+    }
+}
+
 
 /****** Setup of widgets ******/
 static void details_setup_widget (Detail *detail, T_item item)
@@ -571,6 +698,7 @@ static void details_setup_widget (Detail *detail, T_item item)
 	w = gtkpod_xml_get_widget (detail->xml, buf);
 	gtk_button_set_label (GTK_BUTTON (w),
 			      gettext (get_t_string (item)));
+	g_free (buf);
 	break;
     default:
 	buf = g_strdup_printf ("details_label_%d", item);
@@ -616,6 +744,18 @@ static void details_setup_widget (Detail *detail, T_item item)
     case T_TIME_PLAYED:
     case T_TIME_MODIFIED:
     case T_TIME_RELEASED:
+    case T_TV_SHOW:
+    case T_TV_EPISODE:
+    case T_TV_NETWORK:
+    case T_SEASON_NR:
+    case T_EPISODE_NR:
+    case T_ALBUMARTIST:
+    case T_SORT_ARTIST:
+    case T_SORT_TITLE:
+    case T_SORT_ALBUM:
+    case T_SORT_ALBUMARTIST:
+    case T_SORT_COMPOSER:
+    case T_SORT_TVSHOW:
 	buf = g_strdup_printf ("details_entry_%d", item);
 	w = gtkpod_xml_get_widget (detail->xml, buf);
 	g_signal_connect (w, "activate",
@@ -630,6 +770,7 @@ static void details_setup_widget (Detail *detail, T_item item)
     case T_CHECKED:
     case T_REMEMBER_PLAYBACK_POSITION:
     case T_SKIP_WHEN_SHUFFLING:
+    case T_GAPLESS_TRACK_FLAG:
 	buf = g_strdup_printf ("details_checkbutton_%d", item);
 	w = gtkpod_xml_get_widget (detail->xml, buf);
 	g_signal_connect (w, "toggled",
@@ -646,6 +787,14 @@ static void details_setup_widget (Detail *detail, T_item item)
 			  G_CALLBACK (details_text_changed),
 			  detail);
 	break;
+    case T_MEDIA_TYPE:
+	buf = g_strdup_printf ("details_combobox_%d", item);
+	w = gtkpod_xml_get_widget (detail->xml, buf);
+	details_setup_combobox (w, mediatype_comboentries);
+	g_signal_connect (w, "changed",
+			  G_CALLBACK (details_combobox_changed),
+			  detail);
+	break;
     case T_ALL:
     case T_ITEM_NUM:
 	/* cannot happen because of assertion above */
@@ -667,7 +816,7 @@ static void details_set_item (Detail *detail, Track *track, T_item item)
     GtkTextBuffer *tb;
     GtkWidget *w = NULL;
     gchar *text;
-    gchar *entry, *checkbutton, *textview;
+    gchar *entry, *checkbutton, *textview, *combobox;
 
     g_return_if_fail (detail);
     g_return_if_fail ((item > 0) && (item < T_ITEM_NUM));
@@ -675,12 +824,19 @@ static void details_set_item (Detail *detail, Track *track, T_item item)
     entry = g_strdup_printf ("details_entry_%d", item);
     checkbutton = g_strdup_printf ("details_checkbutton_%d", item);
     textview = g_strdup_printf ("details_textview_%d", item);
+    combobox = g_strdup_printf ("details_combobox_%d", item);
 
     if (track != NULL)
     {
 	track->itdb = detail->itdb;
 	text = track_get_text (track, item);
 	track->itdb = NULL;
+	if ((item == T_THUMB_PATH) && (!detail->artwork_ok))
+	{
+	    gchar *new_text = g_strdup_printf (_("%s (image data corrupted or unreadable)"), text);
+	    g_free (text);
+	    text = new_text;
+	}
     }
     else
     {
@@ -721,6 +877,18 @@ static void details_set_item (Detail *detail, Track *track, T_item item)
     case T_TIME_PLAYED:
     case T_TIME_MODIFIED:
     case T_TIME_RELEASED:
+    case T_TV_SHOW:
+    case T_TV_EPISODE:
+    case T_TV_NETWORK:
+    case T_SEASON_NR:
+    case T_EPISODE_NR:
+    case T_ALBUMARTIST:
+    case T_SORT_ARTIST:
+    case T_SORT_TITLE:
+    case T_SORT_ALBUM:
+    case T_SORT_ALBUMARTIST:
+    case T_SORT_COMPOSER:
+    case T_SORT_TVSHOW:
 	w = gtkpod_xml_get_widget (detail->xml, entry);
 	g_signal_handlers_block_by_func (w, details_text_changed, detail);
 	gtk_entry_set_text (GTK_ENTRY (w), text);
@@ -792,6 +960,34 @@ static void details_set_item (Detail *detail, Track *track, T_item item)
 					      FALSE);
 	}
 	break;
+    case T_MEDIA_TYPE:
+	if ((w = gtkpod_xml_get_widget (detail->xml, combobox)))
+	{
+	    gint index = -1;
+	    if (track)
+	    {
+		index = comboentry_index_from_id (mediatype_comboentries,
+						  track->mediatype);
+		if (index == -1)
+		{
+		    gtkpod_warning (_("Please report unknown mediatype %x\n"),
+				    track->mediatype);
+		}
+	    }
+	    gtk_combo_box_set_active (GTK_COMBO_BOX (w), index);
+	}
+	break;
+    case T_GAPLESS_TRACK_FLAG:
+	if ((w = gtkpod_xml_get_widget (detail->xml, checkbutton)))
+	{
+	    if (track)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+					      track->gapless_track_flag);
+	    else
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+					      FALSE);
+	}
+	break;
     case T_ALL:
     case T_ITEM_NUM:
 	/* cannot happen because of assertion above */
@@ -801,6 +997,7 @@ static void details_set_item (Detail *detail, Track *track, T_item item)
     g_free (entry);
     g_free (checkbutton);
     g_free (textview);
+    g_free (combobox);
     g_free (text);
 }
 
@@ -814,7 +1011,7 @@ static void details_get_item (Detail *detail, T_item item,
 			      gboolean assumechanged)
 {
     GtkWidget *w = NULL;
-    gchar *entry, *checkbutton, *textview;
+    gchar *entry, *checkbutton, *textview, *combobox;
     gboolean changed = FALSE;
     ExtraTrackData *etr;
     Track *track;
@@ -830,6 +1027,7 @@ static void details_get_item (Detail *detail, T_item item,
     entry = g_strdup_printf ("details_entry_%d", item);
     checkbutton = g_strdup_printf ("details_checkbutton_%d", item);
     textview = g_strdup_printf ("details_textview_%d", item);
+    combobox = g_strdup_printf ("details_combobox_%d", item);
 
     switch (item)
     {
@@ -861,6 +1059,18 @@ static void details_get_item (Detail *detail, T_item item,
     case T_STARTTIME:
     case T_STOPTIME:
     case T_SOUNDCHECK:
+    case T_TV_SHOW:
+    case T_TV_EPISODE:
+    case T_TV_NETWORK:
+    case T_SEASON_NR:
+    case T_EPISODE_NR:
+    case T_ALBUMARTIST:
+    case T_SORT_ARTIST:
+    case T_SORT_TITLE:
+    case T_SORT_ALBUM:
+    case T_SORT_ALBUMARTIST:
+    case T_SORT_COMPOSER:
+    case T_SORT_TVSHOW:
 	if ((w = gtkpod_xml_get_widget (detail->xml, entry)))
 	{
 	    const gchar *text;
@@ -976,11 +1186,29 @@ static void details_get_item (Detail *detail, T_item item,
 	    }
 	}
 	break;
+    case T_MEDIA_TYPE:
+	if ((w = gtkpod_xml_get_widget (detail->xml, combobox)))
+	{
+	    gint active;
+	    active = gtk_combo_box_get_active (GTK_COMBO_BOX (w));
+	    if (active != -1)
+	    {
+		guint32 new_mediatype = mediatype_comboentries[active].id;
+
+		if (track->mediatype != new_mediatype)
+		{
+		    track->mediatype = new_mediatype;
+		    changed = TRUE;
+		}
+	    }
+	}
+	break;
     case T_TRANSFERRED:
     case T_PC_PATH:
     case T_IPOD_PATH:
     case T_IPOD_ID:
     case T_THUMB_PATH:
+    case T_GAPLESS_TRACK_FLAG:
 	/* These are read-only only */
 	break;
 	break;
@@ -1020,13 +1248,14 @@ static void details_get_item (Detail *detail, T_item item,
     g_free (entry);
     g_free (checkbutton);
     g_free (textview);
+    g_free (combobox);
 
     details_update_buttons (detail);
 }
 
 
 /* Render the Apply button insensitive as long as no changes were done */
-static void details_update_buttons (Detail *detail)
+void details_update_buttons (Detail *detail)
 {
     GtkWidget *w;
     gchar *buf;
@@ -1123,7 +1352,7 @@ static void details_update_buttons (Detail *detail)
 }
 
 /* Update the displayed thumbnail */
-static void details_update_thumbnail (Detail *detail)
+void details_update_thumbnail (Detail *detail)
 {
     Thumb *thumb;
     GtkImage *img;
@@ -1137,6 +1366,7 @@ static void details_update_thumbnail (Detail *detail)
 
     if (detail->track)
     {
+	detail->artwork_ok = TRUE;
 	/* Get large cover */
 	thumb = itdb_artwork_get_thumb_by_type (detail->track->artwork,
 						ITDB_THUMB_COVER_LARGE);
@@ -1150,6 +1380,12 @@ static void details_update_thumbnail (Detail *detail)
 		gtk_image_set_from_pixbuf (img, pixbuf);
 		gdk_pixbuf_unref (pixbuf);
 	    }
+	    else
+	    {
+		gtk_image_set_from_stock (img, GTK_STOCK_DIALOG_WARNING,
+					  GTK_ICON_SIZE_DIALOG);
+		detail->artwork_ok = FALSE;
+	    }
 	}
 	details_set_item (detail,  detail->track, T_THUMB_PATH);
     }
@@ -1185,7 +1421,6 @@ static void details_update_headline (Detail *detail)
     g_free (buf);
 }
 
-
 /* Set the display to @track */
 static void details_set_track (Detail *detail, Track *track)
 {
@@ -1195,6 +1430,9 @@ static void details_set_track (Detail *detail, Track *track)
 
     detail->track = track;
 
+    /* Set thumbnail */
+    details_update_thumbnail (detail);
+
     for (item=1; item<T_ITEM_NUM; ++item)
     {
 	details_set_item (detail, track, item);
@@ -1202,9 +1440,6 @@ static void details_set_track (Detail *detail, Track *track)
 
     details_update_headline (detail);
 
-    /* Set thumbnail */
-    details_update_thumbnail (detail);
-
     details_update_buttons (detail);
 }
 
@@ -1250,6 +1485,7 @@ static void details_set_tracks (Detail *detail, GList *tracks)
 	etr_dup = tr_dup->userdata;
 	g_return_if_fail (etr_dup);
 	etr_dup->tchanged = FALSE;
+	etr_dup->tartwork_changed = FALSE;
 	detail->tracks = g_list_prepend (detail->tracks, tr_dup);
     }
 
@@ -1300,7 +1536,7 @@ static void details_free (Detail *detail)
 {
     g_return_if_fail (detail);
 
-    /* FIXME: how do we free the detail->xml? */
+    g_object_unref (detail->xml);
 
     if (detail->window)
     {
@@ -1418,17 +1654,39 @@ void details_edit (GList *selected_tracks)
 		      G_CALLBACK (details_writethrough_toggled),
 		      detail);
 
-
-
     g_signal_connect (detail->window, "delete_event",
 		      G_CALLBACK (details_delete_event), detail);
 
+    /* enable drag and drop for coverart window */
+    GtkImage *img;
+    img = GTK_IMAGE (gtkpod_xml_get_widget (detail->xml,
+    					    "details_image_thumbnail"));
+    
+    gtk_drag_dest_set (
+    		GTK_WIDGET(img), 
+    		0, 
+    		cover_image_drag_types, 
+    		TGNR (cover_image_drag_types), 
+    		GDK_ACTION_COPY|GDK_ACTION_MOVE);
+
+    g_signal_connect ((gpointer) img, "drag-drop",
+    		G_CALLBACK (dnd_details_art_drag_drop), 
+    		NULL);
+        
+    g_signal_connect ((gpointer) img, "drag-data-received",
+    		G_CALLBACK (dnd_details_art_drag_data_received), 
+    		NULL);
+
+    g_signal_connect ((gpointer) img, "drag-motion",
+    		G_CALLBACK (dnd_details_art_drag_motion), 
+    		NULL);
+    
     details_set_tracks (detail, selected_tracks);
 
     /* set notebook page */
     w = gtkpod_xml_get_widget (detail->xml, "details_notebook");
     page = prefs_get_int (DETAILS_WINDOW_NOTEBOOK_PAGE);
-    if ((page >= 0) && (page < 3))
+    if ((page >= 0) && (page <= 4))
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (w), page);
 
     /* set default size */
@@ -1467,3 +1725,231 @@ void details_remove_track (Track *track)
 	details_remove_track_intern (detail, track);
     }
 }
+
+/* Returns the detail struct currently selected in details */
+Detail *details_get_selected_detail ()
+{
+        return details->data;
+}
+
+static gboolean dnd_details_art_drag_drop(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, gpointer user_data)
+{
+	GdkAtom target;
+	target = gtk_drag_dest_find_target (widget, drag_context, NULL);
+	
+	if (target != GDK_NONE)
+	{
+		gtk_drag_get_data (widget, drag_context, target, time);
+		return TRUE;	
+	}
+	
+	return FALSE;
+}
+
+static gboolean dnd_details_art_drag_motion (GtkWidget *widget,
+				GdkDragContext *dc,
+				gint x,
+				gint y,
+				guint time,
+				gpointer user_data)
+{
+	GdkAtom target;
+	iTunesDB *itdb;
+	ExtraiTunesDBData *eitdb;
+	
+	itdb = gp_get_selected_itdb ();
+	/* no drop is possible if no playlist/repository is selected */
+	if (itdb == NULL)
+	{
+		gdk_drag_status (dc, 0, time);
+		return FALSE;
+	}
+	
+	eitdb = itdb->userdata;
+	g_return_val_if_fail (eitdb, FALSE);
+	/* no drop is possible if no repository is loaded */
+	if (!eitdb->itdb_imported)
+	{
+		gdk_drag_status (dc, 0, time);
+		return FALSE;
+	}
+	    
+	target = gtk_drag_dest_find_target (widget, dc, NULL);
+	/* no drop possible if no valid target can be found */
+	if (target == GDK_NONE)
+	{
+		gdk_drag_status (dc, 0, time);
+		return FALSE;
+	}
+	    
+	gdk_drag_status (dc, dc->suggested_action, time);
+
+  return TRUE;
+}
+
+static void dnd_details_art_drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x, gint y, GtkSelectionData *data, guint info, guint time, gpointer user_data)
+{
+	g_return_if_fail (widget);
+	g_return_if_fail (dc);
+	g_return_if_fail (data);
+	g_return_if_fail (data->data);
+	g_return_if_fail (data->length > 0);
+
+#if DEBUG
+		printf ("data length = %d\n", data->length);
+		printf ("data->data = %s\n", data->data);
+#endif
+			
+	Detail *detail;
+	GList *tracks;
+	gchar *url = NULL;
+	Fetch_Cover *fcover;
+	gchar *filename = NULL;
+	gboolean image_status = FALSE;
+	gchar *image_error = NULL;
+	/* For use with DND_IMAGE_JPEG */
+	GdkPixbuf *pixbuf;
+	GError *error = NULL;
+	
+	/* Find the selected detail item for the coverart image */
+	detail = details_get_selected_detail ();
+	tracks = detail->tracks;
+	
+	switch (info)
+	{
+		case DND_IMAGE_JPEG:			
+#if DEBUG
+			printf ("Using DND_IMAGE_JPEG\n");
+#endif
+			pixbuf = gtk_selection_data_get_pixbuf (data);
+			if (pixbuf != NULL)
+			{
+				/* initialise the url string with a safe value as not used if already have image */
+				url = "local image";
+				/* Initialise a fetchcover object */
+				fcover = fetchcover_new (url, tracks);
+				fcover->parent_window =  GTK_WINDOW(detail->window);
+				coverart_block_change (TRUE);
+								
+				/* find the filename with which to save the pixbuf to */
+				if (fetchcover_select_filename (fcover))
+				{
+					filename = g_build_filename(fcover->dir, fcover->filename, NULL);
+					if (! gdk_pixbuf_save (pixbuf, filename, "jpeg", &error, NULL))
+					{
+						/* Save failed for some reason */
+						fcover->err_msg = g_strdup (error->message);
+						g_error_free (error);
+					}
+					else
+					{
+						/* Image successfully saved */
+						image_status = TRUE;
+					}
+				}
+				/* record any errors and free the fetchcover */
+				if (fcover->err_msg != NULL)
+					image_error = g_strdup(fcover->err_msg);
+								
+				free_fetchcover (fcover);
+				gdk_pixbuf_unref (pixbuf);
+				coverart_block_change (FALSE);
+			}
+			else
+			{
+				/* despite the data being of type image/jpeg, the pixbuf is NULL */
+				image_error = "jpeg data flavour was used but the data did not contain a GdkPixbuf object";
+			}
+			break;
+		case DND_TEXT_PLAIN:
+#if DEBUG
+				printf ("Defaulting to using DND_TEXT_PLAIN\n");
+#endif
+							
+#ifdef HAVE_CURL
+				/* initialise the url string with the data from the dnd */
+				url = g_strdup ((gchar *) data->data);
+				/* Initialise a fetchcover object */
+				fcover = fetchcover_new (url, tracks);
+				/* assign details window as the parent window so the file exists dialog is
+				 * properly centred and visible if a file has to be overwritten
+				 */
+				fcover->parent_window =  GTK_WINDOW(detail->window);
+				coverart_block_change (TRUE);
+						
+				if (fetchcover_net_retrieve_image (fcover))
+				{
+#if DEBUG
+					printf ("Successfully retrieved\n");
+					printf ("Url of fetch cover: %s\n", fcover->url->str);
+					printf ("filename of fetch cover: %s\n", fcover->filename);
+#endif
+							
+					filename = g_build_filename(fcover->dir, fcover->filename, NULL);
+					image_status = TRUE;
+				}
+								
+				/* record any errors and free the fetchcover */
+				if (fcover->err_msg != NULL)
+					image_error = g_strdup(fcover->err_msg);
+												
+				free_fetchcover (fcover);
+				coverart_block_change (FALSE);
+#else
+			image_error = "Item had to be downloaded but gtkpod was not compiled with curl.";
+			image_status = FALSE;
+#endif
+	}
+				
+	if (!image_status || filename == NULL)
+	{
+		gtkpod_warning (_("Error occurred dropping an image onto the details window: %s\n"), image_error);
+		
+		if (image_error)
+			g_free (image_error);
+		if (filename)
+			g_free (filename);
+				
+		gtk_drag_finish (dc, FALSE, FALSE, time);
+		return;
+	}
+	
+	if (details_writethrough(detail))
+	{
+		GList *list;
+		for (list = detail->tracks; list; list = list->next)
+		{
+			ExtraTrackData *etd;
+			Track *track = list->data;
+					    
+			if (!track)
+				break;
+
+			etd = track->userdata;
+			gp_track_set_thumbnails(track, filename);
+			etd->tchanged = TRUE;
+			etd->tartwork_changed = TRUE;
+		}
+	}
+	else
+	{
+		ExtraTrackData *etd = detail->track->userdata;
+		if (etd)
+		{
+			gp_track_set_thumbnails(detail->track, filename);
+			etd->tchanged = TRUE;
+			etd->tartwork_changed = TRUE;
+		}
+	}
+	detail->changed = TRUE;
+	details_update_thumbnail(detail);
+	details_update_buttons(detail);
+
+	if (image_error)
+		g_free (image_error);
+		
+	g_free (filename);	
+	gtkpod_statusbar_message (_("Successfully set new coverart for selected tracks"));
+	gtk_drag_finish (dc, FALSE, FALSE, time);
+	return;
+}
