@@ -1,4 +1,4 @@
-/* Time-stamp: <2006-06-25 00:22:56 jcs>
+/* Time-stamp: <2007-12-13 01:02:57 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display.c,v 1.149 2006/06/24 15:39:22 jcsjcs Exp $
+|  $Id: display.c 1825 2007-12-13 13:28:20Z dforsi $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,7 @@
 #include "display_private.h"
 #include "info.h"
 #include "ipod_init.h"
+#include "file_convert.h"
 #include "misc.h"
 #include "misc_track.h"
 #include "prefs.h"
@@ -44,17 +45,14 @@
 #include "tools.h"
 #include <stdlib.h>
 #include <string.h>
+#include <glib/gprintf.h>
 
 
 GtkWidget *gtkpod_window = NULL;
 
-/* used for stopping of display refresh */
-gint stop_add = SORT_TAB_MAX;
-
 /* Create the listviews etc */
 void display_create (void)
 {
-    GtkWidget *stop_button;
     gint defx, defy;
     GtkTooltips *main_tooltips;
 
@@ -78,11 +76,13 @@ void display_create (void)
 
     tm_create_treeview ();
     st_create_tabs ();
-    st_set_default_sizes ();
     pm_create_treeview ();
-    /* Hide the "stop_button" */
-    stop_button = gtkpod_xml_get_widget (main_window_xml, "stop_button");
-    if (stop_button) gtk_widget_hide (stop_button);
+
+    /* initialize sorting */
+    tm_sort (prefs_get_int("tm_sortcol"), prefs_get_int("tm_sort"));
+
+    st_set_default_sizes ();
+
     /* Hide/Show the toolbar */
     display_show_hide_toolbar ();
     /* Hide/Show tooltips */
@@ -96,6 +96,10 @@ void display_create (void)
     gtkpod_statusbar_init ();
     gtkpod_tracks_statusbar_init ();
     gtkpod_space_statusbar_init ();
+
+    /* Initialize the coverart display */
+    coverart_init_display ();
+
     /* set the menu item for the info window correctly */
     /* CAREFUL: must be done after calling ..._space_statusbar_init() */
     display_set_info_window_menu ();
@@ -135,7 +139,7 @@ void display_reset (gint inst)
 
     pm_set_selected_playlist (cur_pl);
     /* add playlists back to model (without selecting) */
-    pm_add_all_playlists ();
+    pm_add_all_itdbs ();
 }
 
 
@@ -203,6 +207,7 @@ void display_adjust_menus (void)
 	gtk_widget_set_sensitive (edit3, FALSE);
 	gtk_widget_set_sensitive (edit4, FALSE);
 	gtk_widget_set_sensitive (edit5, FALSE);
+	gtk_widget_set_sensitive (espl, FALSE);
     }
     else
     {
@@ -432,17 +437,108 @@ void display_update_default_sizes (void)
     /* x,y size of main window */
     if (gtkpod_window)
     {
-			gtk_window_get_size (GTK_WINDOW (gtkpod_window), &x, &y);
-			prefs_set_int("size_gtkpod.x", x);
-			prefs_set_int("size_gtkpod.y", y);
+	gtk_window_get_size (GTK_WINDOW (gtkpod_window), &x, &y);
+	prefs_set_int("size_gtkpod.x", x);
+	prefs_set_int("size_gtkpod.y", y);
     }
     tm_update_default_sizes ();
     st_update_default_sizes ();
     prefs_window_update_default_sizes ();
     info_update_default_sizes ();
     details_update_default_sizes ();
+    file_convert_update_default_sizes ();
 }
 
+/**
+ * display_image_dialog
+ * 
+ * @GdkPixbuf: image
+ * 
+ * function to load a transient dialog displaying the provided image at either
+ * it maximum size or the size of the screen (whichever is smallest).
+ * 
+ */
+void display_image_dialog (GdkPixbuf *image)
+{
+	g_return_if_fail (image);
+	
+	GladeXML *preview_xml;
+	GtkWidget *dialog;
+	GtkWidget *canvasbox;
+	GtkWidget *res_label;
+	GdkPixbuf *scaled = NULL;
+	gchar *text;
+		
+	preview_xml = glade_xml_new (xml_file, "coverart_preview_dialog", NULL);
+		
+	dialog = gtkpod_xml_get_widget (preview_xml, "coverart_preview_dialog");
+	canvasbox = gtkpod_xml_get_widget (preview_xml, "coverart_preview_dialog_vbox");
+	res_label = gtkpod_xml_get_widget (preview_xml, "coverart_preview_dialog_res_lbl");
+	g_return_if_fail (dialog);
+	g_return_if_fail (canvasbox);
+	g_return_if_fail (res_label);
+		
+	/* Set the dialog parent */
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gtkpod_window));
+			
+	gint pixheight = gdk_pixbuf_get_height (image);
+	gint pixwidth = gdk_pixbuf_get_width (image);
+		
+	/* Set the resolution in the label */
+	gchar *resvalues = (gchar *) g_malloc (sizeof(gint) + (sizeof(gchar) * 3) + sizeof(gint));
+	g_sprintf (resvalues, "%d x %d", pixwidth, pixheight);
+	text = g_markup_printf_escaped (_("<b>Image Dimensions: %s</b>"), resvalues);
+	gtk_label_set_markup (GTK_LABEL (res_label), text);
+	g_free (text);
+	
+	gint scrheight = gdk_screen_height() - 100;
+	gint scrwidth = gdk_screen_width() - 100;
+		
+	gdouble ratio = (gdouble) pixwidth / (gdouble) pixheight;
+	if (pixwidth > scrwidth)
+	{
+		pixwidth = scrwidth;
+		pixheight = pixwidth / ratio;
+	}
+		
+	if (pixheight > scrheight)
+	{
+		pixheight = scrheight;
+		pixwidth = pixheight * ratio;
+	}
+		
+	GnomeCanvas *canvas;
+	canvas = GNOME_CANVAS (gnome_canvas_new());
+	gtk_widget_set_size_request ( GTK_WIDGET(canvas),
+			pixwidth,
+			pixheight);
+	
+	gnome_canvas_set_scroll_region (	canvas,
+																														0.0, 0.0, 
+																														pixwidth,
+																														pixheight);
+	GnomeCanvasItem *canvasitem;											
+	canvasitem = gnome_canvas_item_new(	gnome_canvas_root(canvas),
+																																			GNOME_TYPE_CANVAS_PIXBUF, NULL);
+		
+	scaled = gdk_pixbuf_scale_simple (image, pixwidth, pixheight, GDK_INTERP_NEAREST);
+		
+	/* Apply the image to the canvas */
+	gnome_canvas_item_set (	canvasitem,
+																							"pixbuf", scaled);
+
+	gtk_box_pack_start_defaults (GTK_BOX(canvasbox), GTK_WIDGET (canvas));
+		
+	/* Display the dialog and block everything else until the
+	 * dialog is closed.
+	 */
+	gtk_widget_show_all (dialog);
+	gtk_dialog_run (GTK_DIALOG(dialog));
+			
+	/* Destroy the dialog as no longer required */
+	gdk_pixbuf_unref (scaled);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
 
 
 /* Utility function: returns a copy of the tracks currently
@@ -500,253 +596,10 @@ GList *display_get_selected_members (gint inst)
 
 /* ------------------------------------------------------------
 
-           Functions for stopping display update
+           Functions for treeview autoscroll (during DND)
 
    ------------------------------------------------------------ */
 
-enum {
-    BR_BLOCK,
-    BR_RELEASE,
-    BR_ADD,
-    BR_CALL,
-};
-
-
-#if DEBUG_CB_INIT
-static gchar *act_to_str (gint action)
-{
-    switch (action)
-    {
-    case BR_BLOCK: return ("BLOCK");
-    case BR_RELEASE: return ("RELEASE");
-    case BR_ADD: return ("ADD");
-    case BR_CALL: return ("CALL");
-    }
-    return "\"\"";
-}
-#endif
-
-/* called by block_selection() and release_selection */
-static void block_release_selection (gint inst, gint action,
-				     br_callback brc,
-				     gpointer user_data1,
-				     gpointer user_data2)
-{
-    static gint count_st[SORT_TAB_MAX];
-    static gint count_pl = 0;
-    static GtkWidget *stop_button = NULL;
-    /* instance that has a pending callback */
-    static gint level = SORT_TAB_MAX; /* no level -> no registered callback */
-    static br_callback r_brc;
-    static gpointer r_user_data1;
-    static gpointer r_user_data2;
-    static guint timeout_id = 0;
-    gint i;
-
-#if DEBUG_CB_INIT
-    printf ("block_release_selection: inst: %d, action: %s, callback: %p\n  user1: %p, user2: %d\n", inst, act_to_str (action), brc, user_data1, (guint)user_data2);
-    printf ("  enter: level: %d, count_pl: %d, cst0: %d, cst1: %d\n",
-	    level, count_pl, count_st[0], count_st[1]);
-#endif
-
-    /* lookup stop_button */
-    if (stop_button == NULL)
-    {
-	stop_button = gtkpod_xml_get_widget (main_window_xml, "stop_button");
-	if (stop_button == NULL)
-	    g_warning ("Programming error: stop_button not found\n");
-    }
-
-    switch (action)
-    {
-    case BR_BLOCK:
-	if (count_pl == 0)
-	{
-	    block_widgets ();
-	    if (stop_button) gtk_widget_show (stop_button);
-	}
-	++count_pl;
-	for (i=0; (i<=inst) && (i<SORT_TAB_MAX); ++i)
-	{
-	    ++count_st[i];
-	}
-	break;
-    case BR_RELEASE:
-	for (i=0; (i<=inst) && (i<SORT_TAB_MAX); ++i)
-	{
-	    --count_st[i];
-	    if ((count_st[i] == 0) && (stop_add == i))
-		stop_add = SORT_TAB_MAX;
-	}
-	--count_pl;
-	if (count_pl == 0)
-	{
-	    if (stop_button) gtk_widget_hide (stop_button);
-	    stop_add = SORT_TAB_MAX;
-	    release_widgets ();
-	}
-	/* check if we have to call a callback */
-	if (level < SORT_TAB_MAX)
-	{
-	    /* don't call it directly -- let's first return to the
-	       calling function */
-/* 	    if (timeout_id == 0) */
-/* 	    { */
-/* 		timeout_id =  */
-/* 		    gtk_idle_add_priority (G_PRIORITY_HIGH_IDLE, */
-/* 					   selection_callback_timeout, */
-/* 					   NULL); */
-/* 	    } */
-	    /* remove timeout function just to be sure */
-	    if (timeout_id)
-	    {
-		gtk_timeout_remove (timeout_id);
-		timeout_id = 0;
-	    }
-	    if (((level == -1) && (count_pl == 0)) ||
-		((level >= 0) && (count_st[level] == 0)))
-	    {
-		level = SORT_TAB_MAX;
-		r_brc (r_user_data1, r_user_data2);
-	    }
-	}
-	break;
-    case BR_ADD:
-/*	printf("adding: inst: %d, level: %d, brc: %p, data1: %p, data2: %p\n",
-	inst, level, brc, user_data1, user_data2);*/
-	if (((inst == -1) && (count_pl == 0)) ||
-	    ((inst >= 0) && (count_st[inst] == 0)))
-	{ /* OK, we could just call the desired function because there
-	    is nothing to be stopped. However, due to a bug or a
-            inevitability of gtk+, if we interrupt the selection
-	    process with a gtk_main_iteration() call the button
-	    release event will be "discarded". Therefore we register
-	    the call here and have it activated with a timeout
-	    function. */
-	    /* We overwrite an older callback of the same level or
-	     * higher  */
-	    if (inst <= level)
-	    {
-		level = inst;
-		r_brc = brc;
-		r_user_data1 = user_data1;
-		r_user_data2 = user_data2;
-		if (timeout_id == 0)
-		{
-		    timeout_id =
-			gtk_idle_add_priority (G_PRIORITY_HIGH_IDLE,
-					       selection_callback_timeout,
-					       NULL);
-		}
-	    }
-	}
-	else
-	{
-	    if (inst <= level)
-	    {   /* Once the functions have stopped, down to the
-		specified level/instance, the desired function is
-		called (about 15 lines up: r_brc (...)) */
-		/* We need to emit a stop_add signal */
-		stop_add = inst;
-		/* and safe the callback data */
-		level = inst;
-		r_brc = brc;
-		r_user_data1 = user_data1;
-		r_user_data2 = user_data2;
-		/* don't let a timeout snatch away this callback
-		 * before all functions have stopped */
-		if (timeout_id)
-		{
-		    gtk_timeout_remove (timeout_id);
-		    timeout_id = 0;
-		}
-	    }
-	}
-	break;
-    case BR_CALL:
-	if (timeout_id)
-	{
-	    gtk_timeout_remove (timeout_id);
-	    timeout_id = 0;
-	}
-	if (level == SORT_TAB_MAX) break;  /* hmm... what happened to
-					      our callback? */
-	if (((level == -1) && (count_pl == 0)) ||
-	    ((level >= 0) && (count_st[level] == 0)))
-	{ /* Let's call the callback function */
-		level = SORT_TAB_MAX;
-		r_brc (r_user_data1, r_user_data2);
-	}
-	else
-	{ /* This is strange and should not happen -- let's forget
-	   * about the callback */
-	    level = SORT_TAB_MAX;
-	}
-	break;
-    default:
-	g_warning ("Programming error: unknown BR_...: %d\n", action);
-	break;
-    }
-#if DEBUG_CB_INIT
-    printf ("   exit: level: %d, count_pl: %d, cst0: %d, cst1: %d\n",
-	    level, count_pl, count_st[0], count_st[1]);
-#endif
-}
-
-/* Will block the possibility to select another playlist / sort tab
-   page / tab entry. Will also block the widgets and activate the
-   "stop button" that can be pressed to stop the update
-   process. "inst" is the sort tab instance up to which the selections
-   should be blocked. "-1" corresponds to the playlist view */
-void block_selection (gint inst)
-{
-    block_release_selection (inst, BR_BLOCK, NULL, NULL, NULL);
-}
-
-/* Makes selection possible again */
-void release_selection (gint inst)
-{
-    block_release_selection (inst, BR_RELEASE, NULL, NULL, NULL);
-}
-
-
-/* Stops the display updates down to instance "inst". "-1" is the
- * playlist view */
-void display_stop_update (gint inst)
-{
-    stop_add = inst;
-}
-
-/* registers @brc to be called as soon as all functions down to
-   instance @inst have been stopped */
-void add_selection_callback (gint inst, br_callback brc,
-				    gpointer user_data1, gpointer user_data2)
-{
-    block_release_selection (inst, BR_ADD, brc, user_data1, user_data2);
-}
-
-/* Called as a high priority timeout to initiate the callback of @brc
-   in the last function */
-gboolean selection_callback_timeout (gpointer data)
-{
-    block_release_selection (0, BR_CALL, NULL, NULL, NULL);
-    return FALSE;
-}
-
-
-
-/* Put all treeviews into the unsorted state (@enable=FALSE) or back
-   to the sorted state (@enable=TRUE) */
-/* Indicates that we begin adding playlists from iTunesDB.
-   This is a good time to stop sorting display. */
-void display_enable_disable_view_sort (gboolean enable)
-{
-    st_enable_disable_view_sort (0, enable);
-/*    tm_enable_disable_view_sort (enable);*/
-}
-
-
-
 static void _remove_scroll_row_timeout (GtkWidget *widget)
 {
     g_return_if_fail (widget);
@@ -1242,13 +1095,6 @@ on_check_ipod_files_activate           (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_stop_button_clicked                 (GtkButton       *button,
-					gpointer         user_data)
-{
-    display_stop_update (-1);
-}
-
 void
 on_update_playlist_activate (GtkMenuItem     *menuitem,
 			     gpointer         user_data)
@@ -1879,6 +1725,22 @@ on_info_window1_activate               (GtkMenuItem     *menuitem,
     else info_close_window ();
 }
 
+void
+on_conversion_log1_activate               (GtkMenuItem     *menuitem,
+					   gpointer         user_data)
+{
+    gboolean state;
+
+    state = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem));
+
+    if (state != prefs_get_int (FILE_CONVERT_DISPLAY_LOG))
+    {
+	prefs_set_int (FILE_CONVERT_DISPLAY_LOG, state);
+	file_convert_prefs_changed ();
+    }
+}
+
+
 void
 on_sync_all_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
