@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-09-21 23:12:28 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display.h,v 1.116 2006/09/21 15:03:13 jcsjcs Exp $
+|  $Id: display.h 1765 2007-11-08 20:02:42Z phantom_sf $
 */
 
 #ifndef __DISPLAY_H__
@@ -38,6 +37,7 @@
 #include <glade/glade.h>
 #include "itdb.h"
 #include "display_itdb.h"
+#include "display_coverart.h"
 
 /* Main XML glade file */
 GladeXML *main_window_xml;
@@ -87,12 +87,6 @@ enum {
 /* max. number of stars */
 #define RATING_MAX 5
 
-/* time between display refreshs in ms */
-#define REFRESH_MS 200
-/* initial count number between display refreshs -- will be
-   re-calculated to match the time interval specified above */
-#define REFRESH_INIT_COUNT 5
-
 /* struct for each entry in sort tab */
 typedef struct {
   gchar *name;
@@ -105,8 +99,8 @@ typedef struct {
 typedef struct {
     gchar *int_str;   /* copy of string specified in the sort tab */
     gboolean valid;   /* is current string valid? */
-    guint32 lower;    /* MAC timestamp for lower limit */
-    guint32 upper;    /* MAC timestamp for upper limit ("-1": no limit) */
+    time_t lower;     /* timestamp for lower limit */
+    time_t upper;     /* timestamp for upper limit ("-1": no limit) */
     GtkWidget *entry; /* pointer to GtkEntry in sort tab */
     GtkWidget *active;/* pointer to toggle button */
 } TimeInfo;
@@ -144,12 +138,6 @@ typedef enum  {
 } ST_item;
 
 /* Column numbers in track model */
-/* Note: the toggle buttons for tag_autoset and display_col
- * in the prefs_window are
- * named after the numbers (Title: tag_autoset0, Artist: tag_autoset1
- * etc.). Since the labels to the buttons are set in prefs_window.c
- * when creating the window, you only need to name the buttons in the
- * intended order using glade-2. There is no need to label them. */
 /* Note: add corresponding entries to T_item and TM_to_T() as well 
  * (below and in misc_conversion.c). 
  * IMPORTANT: Do not change the order -- always add new entries at the
@@ -190,6 +178,19 @@ typedef enum  {
   TM_COLUMN_SUBTITLE,
   TM_COLUMN_TIME_RELEASED,
   TM_COLUMN_THUMB_PATH,
+  TM_COLUMN_MEDIA_TYPE,       /* 35 */
+  TM_COLUMN_TV_SHOW,
+  TM_COLUMN_TV_EPISODE,
+  TM_COLUMN_TV_NETWORK,
+  TM_COLUMN_SEASON_NR,
+  TM_COLUMN_EPISODE_NR,       /* 40 */
+  TM_COLUMN_ALBUMARTIST,
+  TM_COLUMN_SORT_ARTIST,
+  TM_COLUMN_SORT_TITLE,
+  TM_COLUMN_SORT_ALBUM,
+  TM_COLUMN_SORT_ALBUMARTIST, /* 45 */
+  TM_COLUMN_SORT_COMPOSER,
+  TM_COLUMN_SORT_TVSHOW,
   TM_NUM_COLUMNS
 } TM_item;
 
@@ -242,6 +243,20 @@ typedef enum {
     T_REMEMBER_PLAYBACK_POSITION,
     T_SKIP_WHEN_SHUFFLING,
     T_THUMB_PATH,   /* 40 */
+    T_MEDIA_TYPE,
+    T_TV_SHOW,
+    T_TV_EPISODE,
+    T_TV_NETWORK,
+    T_SEASON_NR,    /* 45 */
+    T_EPISODE_NR,
+    T_ALBUMARTIST,
+    T_SORT_ARTIST,
+    T_SORT_TITLE,
+    T_SORT_ALBUM,   /* 50 */
+    T_SORT_ALBUMARTIST,
+    T_SORT_COMPOSER,
+    T_SORT_TVSHOW,
+    T_GAPLESS_TRACK_FLAG,
     T_ITEM_NUM,
 } T_item;
 
@@ -251,7 +266,8 @@ typedef enum {
 
 /* "Column numbers" in playlist model */
 enum  {
-  PM_COLUMN_PLAYLIST = 0,
+  PM_COLUMN_ITDB = 0,
+  PM_COLUMN_PLAYLIST,
   PM_NUM_COLUMNS
 };
 
@@ -261,7 +277,8 @@ enum {
     DND_GTKPOD_TM_PATHLIST,
     DND_GTKPOD_PLAYLISTLIST,
     DND_TEXT_URI_LIST,
-    DND_TEXT_PLAIN
+    DND_TEXT_PLAIN,
+    DND_IMAGE_JPEG
 };
 
 /* used for the ST_CAT_SPECIAL user_data (see st_create_special and
@@ -276,12 +293,12 @@ void display_show_hide_toolbar (void);
 void display_reset (gint inst);
 GList *display_get_selection (guint32 inst);
 GList *display_get_selected_members (gint inst);
-void display_enable_disable_view_sort (gboolean enable);
 void display_remove_autoscroll_row_timeout (GtkWidget *widget);
 void display_install_autoscroll_row_timeout (GtkWidget *widget);
+void display_image_dialog (GdkPixbuf *image);
 
-Playlist* pm_get_selected_playlist(void);
-GtkTreePath *pm_get_path (Playlist *pl);
+Playlist* pm_get_selected_playlist (void);
+gint pm_get_position_for_itdb (iTunesDB *itdb);
 void pm_remove_playlist (Playlist *playlist, gboolean select);
 void pm_add_playlist (Playlist *playlist, gint position);
 void pm_add_itdb (iTunesDB *itdb, gint pos);
@@ -289,7 +306,7 @@ void pm_select_playlist (Playlist *playlist);
 void pm_unselect_playlist (Playlist *playlist);
 void pm_remove_track (Playlist *playlist, Track *track);
 void pm_add_track (Playlist *playlist, Track *track, gboolean display);
-void pm_name_changed (Playlist *playlist);
+void pm_itdb_name_changed (iTunesDB *itdb);
 void pm_track_changed (Track *track);
 void pm_sort (GtkSortType order);
 void pm_stop_editing (gboolean cancel);
@@ -299,7 +316,7 @@ void pm_set_renderer_text (GtkCellRenderer *renderer,
 			   Playlist *playlist);
 
 void st_stop_editing (gint inst, gboolean cancel);
-void st_page_selected (GtkNotebook *notebook, guint page);
+gboolean st_set_selection (Itdb_Track *track);
 void st_redisplay (guint32 inst);
 void st_sort (GtkSortType order);
 void st_remove_entry (TabEntry *entry, guint32 inst);
@@ -313,11 +330,8 @@ void cal_open_calendar (gint inst, T_item item);
 void sp_go (guint32 inst);
 void sp_conditions_changed (guint32 inst);
 
-TimeInfo *sp_update_date_interval_from_string (guint32 inst,
-					       T_item item,
-					       gboolean force_update);
 void tm_add_track_to_track_model (Track *track, GtkTreeIter *into_iter);
-guint tm_get_nr_of_tracks(void);
+gint tm_get_nr_of_tracks(void);
 void tm_rows_reordered(void);
 gboolean tm_add_filelist (gchar *data, GtkTreePath *path,
 			  GtkTreeViewDropPosition pos);
@@ -337,7 +351,6 @@ void display_update_default_sizes (void);
 void display_set_default_sizes (void);
 void display_show_hide_tooltips (void);
 void display_set_info_window_menu (void);
-void display_stop_update (gint inst);
 
 void spl_edit (Playlist *spl);
 void spl_edit_new (iTunesDB *itdb, gchar *name, gint32 pos);
