@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-09-21 23:56:52 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_itdb.c,v 1.58 2006/09/21 15:03:13 jcsjcs Exp $
+|  $Id: display_itdb.c 1723 2007-10-08 03:45:55Z tiffman $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -38,8 +37,9 @@
 #include "details.h"
 #include "display_itdb.h"
 #include "display.h"
-#include "md5.h"
+#include "sha1.h"
 #include "file.h"
+#include "file_convert.h"
 #include "misc.h"
 #include "misc_track.h"
 #include "info.h"
@@ -63,7 +63,8 @@ void gp_itdb_extra_destroy (ExtraiTunesDBData *eitdb)
 {
     if (eitdb)
     {
-	md5_free_eitdb (eitdb);
+	sha1_free_eitdb (eitdb);
+	gp_itdb_pc_path_hash_destroy (eitdb);
 	g_free (eitdb);
     }
 }
@@ -106,7 +107,7 @@ void gp_track_extra_destroy (ExtraTrackData *etrack)
 	g_free (etrack->year_str);
 	g_free (etrack->pc_path_utf8);
 	g_free (etrack->hostname);
-	g_free (etrack->md5_hash);
+	g_free (etrack->sha1_hash);
 	g_free (etrack->charset);
 	g_free (etrack);
     }
@@ -124,11 +125,14 @@ ExtraTrackData *gp_track_extra_duplicate (ExtraTrackData *etr)
 	etr_dup->year_str = g_strdup (etr->year_str);
 	etr_dup->pc_path_locale = g_strdup (etr->pc_path_locale);
 	etr_dup->pc_path_utf8 = g_strdup (etr->pc_path_utf8);
+	etr_dup->converted_file = g_strdup (etr->converted_file);
 	etr_dup->thumb_path_locale = g_strdup (etr->thumb_path_locale);
 	etr_dup->thumb_path_utf8 = g_strdup (etr->thumb_path_utf8);
 	etr_dup->hostname = g_strdup (etr->hostname);
-	etr_dup->md5_hash = g_strdup (etr->md5_hash);
+	etr_dup->sha1_hash = g_strdup (etr->sha1_hash);
 	etr_dup->charset = g_strdup (etr->charset);
+	/* clear the pc_path_hashed flag */
+	etr_dup->pc_path_hashed = FALSE;
     }
     return etr_dup;
 }
@@ -150,6 +154,10 @@ void gp_itdb_free (iTunesDB *itdb)
     {
 	space_set_ipod_itdb (NULL);
     }
+
+    /* cancel all pending conversions */
+    file_convert_cancel_itdb (itdb);
+
     itdb_free (itdb);
 }
 
@@ -169,12 +177,13 @@ void gp_itdb_add_extra (iTunesDB *itdb)
 	    (ItdbUserDataDuplicateFunc)gp_itdb_extra_duplicate;
 	eitdb->data_changed = FALSE;
 	eitdb->itdb_imported = FALSE;
+	gp_itdb_pc_path_hash_init (eitdb);
     }
 }
 
 
 /* Validate a complete @itdb (including tracks and playlists),
- * i.e. add the Extra*Data and validate the track entries */
+ * i.e. add the Extra*Data */
 void gp_itdb_add_extra_full (iTunesDB *itdb)
 {
     GList *gl;
@@ -262,7 +271,7 @@ void gp_track_add_extra (Track *track)
 Track *gp_track_add (iTunesDB *itdb, Track *track)
 {
     Track *result=NULL;
-    Track *oldtrack = md5_track_exists_insert (itdb, track);
+    Track *oldtrack = sha1_track_exists_insert (itdb, track);
 
     if(oldtrack)
     {
@@ -274,12 +283,16 @@ Track *gp_track_add (iTunesDB *itdb, Track *track)
     {
 	/* Make sure all strings are initialised -- that way we don't
 	   have to worry about it when we are handling the strings */
-	/* exception: md5_hash, hostname, charset: these may be NULL. */
+	/* exception: sha1_hash, hostname, charset: these may be NULL. */
 	gp_track_validate_entries (track);
 	itdb_track_add (itdb, track, -1);
+	/* add to filename hash */
+	gp_itdb_pc_path_hash_add_track (track);
+	/* add to background conversion if necessary */
+	file_convert_add_track (track);
 	result = track;
+	data_changed (itdb);
     }
-    data_changed (itdb);
     return result;
 }
 
@@ -290,10 +303,12 @@ Track *gp_track_add (iTunesDB *itdb, Track *track)
    playlist -- see gp_playlist_remove_track for details */
 void gp_track_remove (Track *track)
 {
-    /* currently only the details window may be accessing the tracks */
-    details_remove_track (track);
-    /* remove from database */
-    itdb_track_remove (track);
+    /* call gp_track_unlink() and itdb_track_free() instead of
+       itdb_track_remove() so we don't have to maintain two remove
+       functions separately. If something needs to be done before
+       removing the track do it in gp_track_unlink */
+    gp_track_unlink (track);
+    itdb_track_free (track);
 }
 
 
@@ -303,8 +318,14 @@ void gp_track_remove (Track *track)
    playlist -- see gp_playlist_remove_track for details */
 void gp_track_unlink (Track *track)
 {
-    /* currently only the details window may be accessing the tracks */
+    /* the details window may be accessing the tracks */
     details_remove_track (track);
+    /* cancel pending conversions */
+    file_convert_cancel_track (track);
+    /* remove from SHA1 hash */
+    sha1_track_remove (track);
+    /* remove from pc_path_hash */
+    gp_itdb_pc_path_hash_remove_track (track);
     /* remove from database */
     itdb_track_unlink (track);
 }
@@ -441,7 +462,6 @@ void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
     Playlist *old_pl, *mpl;
     GList *old_link;
     gchar *old_pl_name = NULL;
-    GtkTreePath *path;
     gint pos = -1; /* default: add to the end */
 
     g_return_if_fail (old_itdb);
@@ -464,20 +484,11 @@ void gp_replace_itdb (iTunesDB *old_itdb, iTunesDB *new_itdb)
     }
 
     /* get position of @old_itdb */
-    mpl = itdb_playlist_mpl (old_itdb);
-    g_return_if_fail (mpl);
-    path = pm_get_path (mpl);
-    if (path)
-    {
-	gint *indices = gtk_tree_path_get_indices (path);
-	if (indices)
-	    pos = indices[0];
-	gtk_tree_path_free (path);
-    }
+    pos = pm_get_position_for_itdb (old_itdb);
 
     /* remove @old_itdb (all playlists are removed if the MPL is
        removed and add @new_itdb at its place */
-    pm_remove_playlist (mpl, FALSE);
+    pm_remove_playlist (itdb_playlist_mpl (old_itdb), FALSE);
 
     /* replace old_itdb with new_itdb */
     new_eitdb->itdbs_head = itdbs_head;
@@ -542,7 +553,7 @@ Playlist *gp_playlist_by_name_or_add (iTunesDB *itdb, gchar *pl_name,
     g_return_val_if_fail (pl_name, pl);
     pl = itdb_playlist_by_name (itdb, pl_name);
     if (pl)
-    {   /* check if the it's the same type (spl or normal) */
+    {   /* check if it's the same type (spl or normal) */
 	if (pl->is_spl == spl) return pl;
     }
     /* Create a new playlist */
@@ -597,7 +608,7 @@ guint gp_playlist_remove_by_name (iTunesDB *itdb, gchar *pl_name)
    No action is taken if "track" is not in the playlist.
    If "plitem" == NULL, remove from master playlist.
    If the track is removed from the MPL, it's also removed
-   from memory completely (i.e. from the tracklist and md5 hash).
+   from memory completely (i.e. from the tracklist and sha1 hash).
    Depending on @deleteaction, the track is either marked for deletion
    on the ipod/hard disk or just removed from the database
  */
@@ -617,10 +628,8 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
     case DELETE_ACTION_IPOD:
     case DELETE_ACTION_LOCAL:
     case DELETE_ACTION_DATABASE:
-	/* remove from MPL in these cases (unless we are removing
-	   podcasts) */
-	if (!(plitem && itdb_playlist_is_podcasts (plitem)))
-	    plitem = NULL;
+	/* remove from MPL in these cases */
+	plitem = NULL;
 	break;
     case DELETE_ACTION_PLAYLIST:
 	/* cannot remove from MPL */
@@ -641,6 +650,8 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
     /* remove track from playlist */
     itdb_playlist_remove_track (plitem, track);
 
+#if 0
+    /* podcasts are no longer treated differently from other playlists */
     /* if we removed a podcasts, remove it from memory as well, unless
        it's present in the MPL (this happens if this podcast was on
        the iPod as podcast as well as standard track) */
@@ -662,6 +673,7 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 	    gp_track_set_flags_default (track);
 	}
     }
+#endif
 
     if (itdb_playlist_is_mpl (plitem))
     { /* if it's the MPL, we remove the track permanently */
@@ -685,7 +697,6 @@ void gp_playlist_remove_track (Playlist *plitem, Track *track,
 
     if (remove_track)
     {
-	md5_track_remove (track);
 	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
 	{
 	    switch (deleteaction)
@@ -766,7 +777,7 @@ void gp_playlist_add_track (Playlist *pl, Track *track, gboolean display)
 
 /* Make sure all strings are initialised -- that way we don't
    have to worry about it when we are handling the strings.
-   exception: md5_hash, hostname and charset: these may be NULL. */
+   exception: sha1_hash, hostname and charset: these may be NULL. */
 void gp_track_validate_entries (Track *track)
 {
     ExtraTrackData *etr;
@@ -789,6 +800,16 @@ void gp_track_validate_entries (Track *track)
     if (!track->podcastrss)      track->podcastrss = g_strdup ("");
     if (!track->subtitle)        track->subtitle = g_strdup ("");
     if (!track->ipod_path)       track->ipod_path = g_strdup ("");
+    if (!track->tvshow)          track->tvshow = g_strdup("");
+    if (!track->tvepisode)       track->tvepisode = g_strdup("");
+    if (!track->tvnetwork)       track->tvnetwork = g_strdup("");
+    if (!track->albumartist)     track->albumartist = g_strdup ("");
+    if (!track->sort_artist)     track->sort_artist = g_strdup ("");
+    if (!track->sort_title)      track->sort_title = g_strdup ("");
+    if (!track->sort_album)      track->sort_album = g_strdup ("");
+    if (!track->sort_albumartist)track->sort_albumartist = g_strdup ("");
+    if (!track->sort_composer)   track->sort_composer = g_strdup ("");
+    if (!track->sort_tvshow)     track->sort_tvshow = g_strdup ("");
     if (!etr->pc_path_utf8)      etr->pc_path_utf8 = g_strdup ("");
     if (!etr->pc_path_locale)    etr->pc_path_locale = g_strdup ("");
     if (!etr->thumb_path_utf8)   etr->thumb_path_utf8 = g_strdup ("");
@@ -826,32 +847,37 @@ void init_data (GtkWidget *window)
     {
 	/* databases have not been set up previously -- take care of
 	   this */
-	gchar *mountpoint, *filename;
-
-	/* iPod database */
-	mountpoint = prefs_get_string ("initial_mountpoint");
-	filename = g_build_filename (cfgdir, "iTunesDB", NULL);
-	prefs_set_int ("itdb_0_type", GP_ITDB_TYPE_IPOD);
-	prefs_set_string ("itdb_0_name", _("iPod"));
-	prefs_set_string ("itdb_0_filename", filename);
-	prefs_set_string ("itdb_0_mountpoint", mountpoint);
-	g_free (mountpoint);
-	g_free (filename);
+#ifndef HAVE_GNOME_VFS
+	gchar *mountpoint;
+#endif
+	gchar *filename;
 
 	/* Local database */
 	filename = g_build_filename (cfgdir, "local_0.itdb", NULL);
-	prefs_set_int ("itdb_1_type", GP_ITDB_TYPE_LOCAL);
-	prefs_set_string ("itdb_1_name", _("Local"));
-	prefs_set_string ("itdb_1_filename", filename);
+	prefs_set_int ("itdb_0_type", GP_ITDB_TYPE_LOCAL);
+	prefs_set_string ("itdb_0_name", _("Local"));
+	prefs_set_string ("itdb_0_filename", filename);
 	g_free (filename);
 
 	/* Podcasts database */
 	filename = g_build_filename (cfgdir, "podcasts.itdb", NULL);
-	prefs_set_int ("itdb_2_type",
+	prefs_set_int ("itdb_1_type",
 		       GP_ITDB_TYPE_PODCASTS|GP_ITDB_TYPE_LOCAL);
-	prefs_set_string ("itdb_2_name", _("Podcasts"));
-	prefs_set_string ("itdb_2_filename", filename);
+	prefs_set_string ("itdb_1_name", _("Podcasts"));
+	prefs_set_string ("itdb_1_filename", filename);
 	g_free (filename);
+
+#ifndef HAVE_GNOME_VFS
+	/* iPod database -- only set up if autodetection is not active */
+	mountpoint = prefs_get_string ("initial_mountpoint");
+	filename = g_build_filename (cfgdir, "iTunesDB", NULL);
+	prefs_set_int ("itdb_2_type", GP_ITDB_TYPE_IPOD);
+	prefs_set_string ("itdb_2_name", _("iPod"));
+	prefs_set_string ("itdb_2_filename", filename);
+	prefs_set_string ("itdb_2_mountpoint", mountpoint);
+	g_free (mountpoint);
+	g_free (filename);
+#endif
     }
 
 
@@ -972,8 +998,16 @@ iTunesDB *setup_itdb_n (gint i)
 	    itdb_playlist_set_mpl (pl);
 	    itdb_playlist_add (itdb, pl, -1);
 
-	    eitdb->data_changed = FALSE;
-	    eitdb->itdb_imported = FALSE;
+           if ((type & GP_ITDB_TYPE_PODCASTS) || (type & GP_ITDB_TYPE_LOCAL))
+             {
+               eitdb->data_changed = TRUE;
+               eitdb->itdb_imported = TRUE;
+             }
+           else
+             {
+               eitdb->data_changed = FALSE;
+               eitdb->itdb_imported = FALSE;
+             }
 	}
 	else
 	{
@@ -1016,15 +1050,15 @@ iTunesDB *setup_itdb_n (gint i)
 }
 
 
-/* Increase playcount of filename <file> by <num>. If md5 is activated,
-   use md5 to find the track. Otherwise use the filename. If @md5 is
+/* Increase playcount of filename <file> by <num>. If sha1 is activated,
+   use sha1 to find the track. Otherwise use the filename. If @sha1 is
    set, this value is used directly to look up the track in the
    database (instead of calculating it from the file).
 
    Return value:
    TRUE: OK (playcount increased in GP_ITDB_TYPE_IPOD)
    FALSE: file could not be found in the GP_ITDB_TYPE_IPOD */
-gboolean gp_increase_playcount (gchar *md5, gchar *file, gint num)
+gboolean gp_increase_playcount (gchar *sha1, gchar *file, gint num)
 {
     gboolean result = FALSE;
     Track *track = NULL;
@@ -1037,8 +1071,8 @@ gboolean gp_increase_playcount (gchar *md5, gchar *file, gint num)
 	iTunesDB *itdb = gl->data;
 	g_return_val_if_fail (itdb, FALSE);
 
-	if (md5) track = md5_md5_exists (itdb, md5);
-	else     track = md5_file_exists (itdb, file, TRUE);
+	if (sha1) track = sha1_sha1_exists (itdb, sha1);
+	else     track = sha1_file_exists (itdb, file, TRUE);
 	if (!track)	  track = gp_track_by_filename (itdb, file);
 	if (track)
 	{
