@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-09-18 15:27:12 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_itdb.h,v 1.35 2006/09/18 07:10:44 jcsjcs Exp $
+|  $Id: display_itdb.h 1750 2007-11-01 21:57:01Z phantom_sf $
 */
 
 #ifndef __DISPLAY_ITDB_H__
@@ -36,6 +35,7 @@
 
 #include <gtk/gtk.h>
 #include "itdb.h"
+#include "file_convert.h"
 
 struct itdbs_head
 {
@@ -44,16 +44,19 @@ struct itdbs_head
 
 typedef struct
 {
-    struct itdbs_head *itdbs_head; /* pointer to the master itdbs_head */
-    GHashTable *md5hash;           /* md5 hash information             */
+    struct itdbs_head *itdbs_head; /* pointer to the master itdbs_head     */
+    GHashTable *sha1hash;          /* sha1 hash information                */
+    GHashTable *pc_path_hash;      /* hash with local filenames            */
     GList *pending_deletion;       /* tracks marked for removal from
-				      media                            */
+				      media                                */
     gchar *offline_filename;       /* filename for offline database
-				      (only for GP_ITDP_TYPE_IPOD)     */
-    gboolean offline;              /* offline mode?                    */
-    gboolean data_changed;         /* data changed since import?       */
-    gboolean itdb_imported;        /* has in iTunesDB been imported?   */
-    gboolean ipod_ejected;         /* if iPod was ejected              */
+				      (only for GP_ITDP_TYPE_IPOD)         */
+    gboolean offline;              /* offline mode?                        */
+    gboolean data_changed;         /* data changed since import?           */
+    gboolean photo_data_changed;	/* photo data changed since import?		*/
+    gboolean itdb_imported;        /* has in iTunesDB been imported?       */
+    gboolean ipod_ejected;         /* if iPod was ejected                  */
+    PhotoDB *photodb;            /* Photo DB reference used if the ipod supports photos */
 } ExtraiTunesDBData;
 
 typedef struct
@@ -63,26 +66,36 @@ typedef struct
 
 typedef struct
 {
-  gint32  oldsize;        /* used when updating tracks: size on iPod */
-  gchar   *year_str;      /* year as string -- always identical to year */
-  gchar   *pc_path_locale;/* path on PC (local encoding)             */
-  gchar   *pc_path_utf8;  /* PC filename in utf8 encoding            */
-  time_t  mtime;          /* modification date of PC file            */
-  gchar   *thumb_path_locale; /* same for thumbnail                  */
-  gchar   *thumb_path_utf8;   /* same for thumbnail                  */
-  gchar   *hostname;      /* name of host this file has been imported on*/
-  gchar   *md5_hash;      /* md5 hash of file (or NULL)              */
-  gchar   *charset;       /* charset used for ID3 tags               */
-  gint32  sortindex;      /* used for stable sorting (current order) */
-  gboolean tchanged;      /* temporary use, e.g. in detail.c         */
+  gchar   *year_str;        /* year as string -- always identical to year  */
+  gchar   *pc_path_locale;  /* path on PC (local encoding)                 */
+  gchar   *pc_path_utf8;    /* PC filename in utf8 encoding                */
+  time_t  mtime;            /* modification date of PC file                */
+  gboolean pc_path_hashed;  /* for programming error detection (see
+			       gp_itdb_local_path_hash_add_track()         */
+  gchar   *converted_file;  /* if converted file exists: name in utf8      */
+  gint32  orig_filesize;    /* size of original file (if converted)        */
+  FileConvertStatus conversion_status; /* current status of conversion     */
+  gchar   *thumb_path_locale;/* same for thumbnail                         */
+  gchar   *thumb_path_utf8;  /* same for thumbnail                         */
+  gchar   *hostname;        /* name of host this file has been imported on */
+  gchar   *sha1_hash;       /* sha1 hash of file (or NULL)                 */
+  gchar   *charset;         /* charset used for ID3 tags                   */
+  gint32  sortindex;        /* used for stable sorting (current order)     */
+  gboolean tchanged;        /* temporary use, e.g. in detail.c             */
+  gboolean tartwork_changed;			/* temporary use for artwork, eg. in detail.c          */
+  guint64 local_itdb_id;    /* when using DND from local to iPod:
+			       original itdb                               */
+  guint64 local_track_dbid; /* when using DND from local to iPod:
+			       original track                              */
 } ExtraTrackData;
 
 /* types for iTunesDB */
 typedef enum
 {
     GP_ITDB_TYPE_LOCAL = 1<<0,    /* local browsing, normal music */
-    GP_ITDB_TYPE_IPOD  = 1<<1,    /* iPod */
-    GP_ITDB_TYPE_PODCASTS = 1<<2, /* local browsing, podcasts */
+    GP_ITDB_TYPE_IPOD  = 1<<1,    /* iPod                         */
+    GP_ITDB_TYPE_PODCASTS = 1<<2, /* local browsing, podcasts     */
+    GP_ITDB_TYPE_AUTOMATIC = 1<<3,/* repository was automounted   */
 } GpItdbType;
 
 /* Delete actions */
@@ -146,7 +159,7 @@ void gp_playlist_add_track (Playlist *pl, Track *track, gboolean display);
 
 void gp_playlist_add_extra (Playlist *pl);
 
-gboolean gp_increase_playcount (gchar *md5, gchar *file, gint num);
+gboolean gp_increase_playcount (gchar *sha1, gchar *file, gint num);
 iTunesDB *gp_get_selected_itdb (void);
 iTunesDB *gp_get_ipod_itdb (void);
 iTunesDB *gp_get_podcast_itdb (void);
