@@ -23,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_tracks.c 1589 2007-06-25 14:23:59Z jcsjcs $
+|  $Id: display_tracks.c 1815 2007-12-12 15:36:40Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -838,6 +838,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
      {
      case TM_COLUMN_TITLE:
      case TM_COLUMN_ALBUM:
+     case TM_COLUMN_ALBUMARTIST:
      case TM_COLUMN_ARTIST:
      case TM_COLUMN_GENRE:
      case TM_COLUMN_COMPOSER:
@@ -870,6 +871,12 @@ tm_cell_edited (GtkCellRendererText *renderer,
      case TM_COLUMN_TV_NETWORK:
      case TM_COLUMN_SEASON_NR:
      case TM_COLUMN_EPISODE_NR:
+     case TM_COLUMN_SORT_TITLE:
+     case TM_COLUMN_SORT_ALBUM:
+     case TM_COLUMN_SORT_ALBUMARTIST:
+     case TM_COLUMN_SORT_COMPOSER:
+     case TM_COLUMN_SORT_TVSHOW:
+     case TM_COLUMN_SORT_ARTIST:
 	 changed = track_set_text (track, new_text, TM_to_T (column));
 	 if (changed && (column == TM_COLUMN_TRACKLEN))
 	 {  /* be on the safe side and reset starttime, stoptime and
@@ -903,9 +910,16 @@ tm_cell_edited (GtkCellRendererText *renderer,
 	     break;
 	 }
 	 break;
-     default:
-        g_warning ("Programming error: tm_cell_edited: unknown track cell (%d) edited\n", column);
-        break;
+     case TM_COLUMN_IPOD_ID:
+     case TM_COLUMN_PC_PATH:
+     case TM_COLUMN_TRANSFERRED:
+     case TM_COLUMN_SIZE:
+     case TM_COLUMN_IPOD_PATH:
+     case TM_COLUMN_COMPILATION:
+     case TM_COLUMN_THUMB_PATH:
+     case TM_NUM_COLUMNS:
+	 /* These are not editable text fields */
+	 break;
      }
 /*      printf ("  changed: %d\n", changed); */
      if (changed)
@@ -1057,6 +1071,39 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
 
 
 
+/* This function is analogous to tm_cell_data_func(), but is only used
+   for the title column to distinguish between the text and the toggle
+   button there. The other toggle buttons (e.g. compilation) can
+   easily be handled in the original tm_cell_data_func() */
+static void tm_cell_data_func_toggle (GtkTreeViewColumn *tree_column,
+				      GtkCellRenderer   *renderer,
+				      GtkTreeModel      *model,
+				      GtkTreeIter       *iter,
+				      gpointer           data)
+{
+  Track *track;
+  TM_item column;
+
+  column = (TM_item)g_object_get_data (G_OBJECT (renderer), "column");
+  gtk_tree_model_get (model, iter, READOUT_COL, &track, -1);
+
+  /* printf ("tm_cell_data_func_toggle() entered\n"); */
+
+  switch (column)
+  {
+  case TM_COLUMN_TITLE:
+      g_object_set (G_OBJECT (renderer),
+		    "active", !track->checked,
+		    "activatable", TRUE, NULL);
+      break;
+  default:
+      g_warning ("Programming error: unknown column in tm_cell_data_func_toggle: %d\n", column);
+      break;
+  }
+}
+
+
+
 /* Called when a toggle cell is being changed. Stores new data to the
    track list. */
 static void
@@ -1077,7 +1124,7 @@ tm_cell_toggled (GtkCellRendererToggle *renderer,
   selection = gtk_tree_view_get_selection(track_treeview);
   row_list = gtk_tree_selection_get_selected_rows(selection, &model);
 
-/*  printf("tm_cell_toggled: column: %d, arg1: %p\n", column, arg1); */
+  /* printf("tm_cell_toggled: column: %d, arg1: %p\n", column, arg1); */
 
   sel_rows_num = g_list_length (row_list);
 
@@ -1117,9 +1164,55 @@ tm_cell_toggled (GtkCellRendererToggle *renderer,
 	 if (!active) track->compilation = 1;
 	 else        track->compilation = 0;
         break;
-     default:
-        g_warning ("Programming error: tm_cell_toggled: unknown track cell (%d) edited\n", column);
-        break;
+     case TM_COLUMN_ARTIST:
+     case TM_COLUMN_ALBUM:
+     case TM_COLUMN_GENRE:
+     case TM_COLUMN_COMPOSER:
+     case TM_COLUMN_TRACK_NR:
+     case TM_COLUMN_IPOD_ID:
+     case TM_COLUMN_PC_PATH:
+     case TM_COLUMN_TRANSFERRED:
+     case TM_COLUMN_SIZE:
+     case TM_COLUMN_TRACKLEN:
+     case TM_COLUMN_BITRATE:
+     case TM_COLUMN_PLAYCOUNT:
+     case TM_COLUMN_RATING:
+     case TM_COLUMN_TIME_PLAYED:
+     case TM_COLUMN_TIME_MODIFIED:
+     case TM_COLUMN_VOLUME:
+     case TM_COLUMN_YEAR:
+     case TM_COLUMN_CD_NR:
+     case TM_COLUMN_TIME_ADDED:
+     case TM_COLUMN_IPOD_PATH:
+     case TM_COLUMN_SOUNDCHECK:
+     case TM_COLUMN_SAMPLERATE:
+     case TM_COLUMN_BPM:
+     case TM_COLUMN_FILETYPE:
+     case TM_COLUMN_GROUPING:
+     case TM_COLUMN_COMMENT:
+     case TM_COLUMN_CATEGORY:
+     case TM_COLUMN_DESCRIPTION:
+     case TM_COLUMN_PODCASTURL:
+     case TM_COLUMN_PODCASTRSS:
+     case TM_COLUMN_SUBTITLE:
+     case TM_COLUMN_TIME_RELEASED:
+     case TM_COLUMN_THUMB_PATH:
+     case TM_COLUMN_MEDIA_TYPE:
+     case TM_COLUMN_TV_SHOW:
+     case TM_COLUMN_TV_EPISODE:
+     case TM_COLUMN_TV_NETWORK:
+     case TM_COLUMN_SEASON_NR:
+     case TM_COLUMN_EPISODE_NR:
+     case TM_COLUMN_ALBUMARTIST:
+     case TM_COLUMN_SORT_ARTIST:
+     case TM_COLUMN_SORT_TITLE:
+     case TM_COLUMN_SORT_ALBUM:
+     case TM_COLUMN_SORT_ALBUMARTIST:
+     case TM_COLUMN_SORT_COMPOSER:
+     case TM_COLUMN_SORT_TVSHOW:
+     case TM_NUM_COLUMNS:
+	 /* these are not toggle buttons */
+	 break;
      }
 /*      printf ("  changed: %d\n", changed); */
      if (changed)
@@ -1147,35 +1240,6 @@ tm_cell_toggled (GtkCellRendererToggle *renderer,
 
 
 
-/* The track data is stored in a separate list (static GList *tracks)
-   and only pointers to the corresponding Track structure are placed
-   into the model.
-   This function reads the data for the given cell from the list and
-   passes it to the renderer. */
-static void tm_cell_data_func_toggle (GtkTreeViewColumn *tree_column,
-				      GtkCellRenderer   *renderer,
-				      GtkTreeModel      *model,
-				      GtkTreeIter       *iter,
-				      gpointer           data)
-{
-  Track *track;
-  TM_item column;
-
-  column = (TM_item)g_object_get_data (G_OBJECT (renderer), "column");
-  gtk_tree_model_get (model, iter, READOUT_COL, &track, -1);
-
-  switch (column)
-  {
-  case TM_COLUMN_TITLE:
-      g_object_set (G_OBJECT (renderer),
-		    "active", !track->checked,
-		    "activatable", TRUE, NULL);
-      break;
-  default:
-      g_warning ("Programming error: unknown column in tm_cell_data_func_toggle: %d\n", column);
-      break;
-  }
-}
 
 /**
  * tm_get_nr_of_tracks - get the number of tracks displayed
@@ -1451,6 +1515,7 @@ static gint tm_data_compare (Track *track1, Track *track2,
   {
   case TM_COLUMN_TITLE:
   case TM_COLUMN_ALBUM:
+  case TM_COLUMN_ALBUMARTIST:
   case TM_COLUMN_GENRE:
   case TM_COLUMN_COMPOSER:
   case TM_COLUMN_COMMENT:
@@ -1465,6 +1530,12 @@ static gint tm_data_compare (Track *track1, Track *track2,
   case TM_COLUMN_TV_SHOW:
   case TM_COLUMN_TV_EPISODE:
   case TM_COLUMN_TV_NETWORK:
+  case TM_COLUMN_SORT_TITLE:
+  case TM_COLUMN_SORT_ALBUM:
+  case TM_COLUMN_SORT_ARTIST:
+  case TM_COLUMN_SORT_ALBUMARTIST:
+  case TM_COLUMN_SORT_COMPOSER:
+  case TM_COLUMN_SORT_TVSHOW:
       /* string_compare_func is set to either compare_string_fuzzy or
 	 compare_string in tm_sort_column_changed() which is called
 	 once before the comparing begins. */
@@ -1562,8 +1633,7 @@ static gint tm_data_compare (Track *track1, Track *track2,
   case TM_COLUMN_MEDIA_TYPE:
       cmp = track1->mediatype - track2->mediatype;
       break;
-  default:
-      g_warning ("Programming error: tm_data_compare_func: no sort method for tm_item %d\n", tm_item);
+  case TM_NUM_COLUMNS:
       break;
   }
 
@@ -1590,7 +1660,7 @@ gboolean tm_search_equal_func (GtkTreeModel *model,
   Track *track1;
   gboolean cmp = 0;
   gtk_tree_model_get (model, iter, READOUT_COL, &track1, -1);
-  switch (column)
+  switch ((TM_item)column)
   {
   case TM_COLUMN_TITLE:
   case TM_COLUMN_ALBUM:
@@ -1613,10 +1683,17 @@ gboolean tm_search_equal_func (GtkTreeModel *model,
   case TM_COLUMN_TV_SHOW:
   case TM_COLUMN_TV_EPISODE:
   case TM_COLUMN_TV_NETWORK:
-    cmp = (compare_string_start_case_insensitive (
-	  track_get_item (track1, TM_to_T (column)),
-	  key) != 0);
-    break;
+  case TM_COLUMN_ALBUMARTIST:
+  case TM_COLUMN_SORT_ARTIST:
+  case TM_COLUMN_SORT_TITLE:
+  case TM_COLUMN_SORT_ALBUM:
+  case TM_COLUMN_SORT_ALBUMARTIST:
+  case TM_COLUMN_SORT_COMPOSER:
+  case TM_COLUMN_SORT_TVSHOW:
+      cmp = (compare_string_start_case_insensitive (
+		 track_get_item (track1, TM_to_T (column)),
+		 key) != 0);
+      break;
   case TM_COLUMN_TRACK_NR:
   case TM_COLUMN_IPOD_ID:
   case TM_COLUMN_TRANSFERRED:
@@ -1638,7 +1715,6 @@ gboolean tm_search_equal_func (GtkTreeModel *model,
   case TM_COLUMN_SEASON_NR:
   case TM_COLUMN_EPISODE_NR:
   case TM_NUM_COLUMNS:
-      g_warning ("Programming error: tm_search_equal_func: no sort method for column %d\n", column);
       break;
   }
   return cmp;
@@ -1738,7 +1814,7 @@ static void tm_unsort (void)
 }
 
 
-static void tm_set_search_column (gint newcol)
+static void tm_set_search_column (TM_item newcol)
 {
 /*     printf ("track_treeview: %p, col: %d\n", track_treeview, newcol); */
     g_return_if_fail (track_treeview);
@@ -1768,6 +1844,13 @@ static void tm_set_search_column (gint newcol)
     case TM_COLUMN_TV_SHOW:
     case TM_COLUMN_TV_EPISODE:
     case TM_COLUMN_TV_NETWORK:
+    case TM_COLUMN_ALBUMARTIST:
+    case TM_COLUMN_SORT_ARTIST:
+    case TM_COLUMN_SORT_TITLE:
+    case TM_COLUMN_SORT_ALBUM:
+    case TM_COLUMN_SORT_ALBUMARTIST:
+    case TM_COLUMN_SORT_COMPOSER:
+    case TM_COLUMN_SORT_TVSHOW:
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (track_treeview), TRUE);
 	break;
     case TM_COLUMN_TRACK_NR:
@@ -2262,8 +2345,6 @@ void tm_create_treeview (void)
 		    G_CALLBACK (tm_sort_column_changed),
 		    (gpointer)0);
 
-  /* initialize sorting */
-  tm_sort (prefs_get_int("tm_sortcol"), prefs_get_int("tm_sort"));
   /* set correct column for typeahead */
   if (prefs_get_int_value (TM_PREFS_SEARCH_COLUMN, &col))
   {
