@@ -23,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file.c 1596 2007-06-26 14:33:01Z dforsi $
+|  $Id: file.c 1824 2007-12-13 10:51:04Z dforsi $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -280,7 +280,7 @@ add_playlist_by_filename (iTunesDB *itdb, gchar *plfile,
     /* attempt to open playlist file */
     if (!(fp = fopen (plfile, "r")))
     {
-	gtkpod_warning (_("Cannot open '%s' for reading.\n\n"));
+	gtkpod_warning (_("Could not open '%s' for reading.\n"), plfile);
 	g_free (plname);
 	return FALSE;  /* definitely not! */
     }
@@ -856,6 +856,7 @@ static gboolean copy_new_info (Track *from, Track *to)
 	case T_TRANSFERRED:
 	case T_PLAYCOUNT:
 	case T_VOLUME:
+	case T_GAPLESS_TRACK_FLAG:
 	    /* not applicable */
 	    break;
 	case T_ITEM_NUM:
@@ -1761,6 +1762,9 @@ void update_track_from_file (iTunesDB *itdb, Track *track)
 	    g_free (name_on_ipod);
 	}
 
+	/* Set this flag to true to ensure artwork is reread from file */
+	netr->tartwork_changed = TRUE;
+	
 	/* notify display model */
 	if (netr->tchanged)
 	{
@@ -1768,6 +1772,14 @@ void update_track_from_file (iTunesDB *itdb, Track *track)
 	    data_changed (itdb);
 	    netr->tchanged = FALSE;
 	}
+	else
+	{
+		/* Rather than depend on the track data being changed, only the artwork may have changed
+		 * hence the reason for the user updating from file
+		 */
+		coverart_track_changed (track, COVERART_CHANGE_SIGNAL);
+	}
+	
 	display_updated (track, NULL);
         g_free (oldhash);
     }
