@@ -1,4 +1,4 @@
-/* Time-stamp: <2006-09-01 22:00:03 jcs>
+/* Time-stamp: <2007-05-06 20:20:31 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file.h,v 1.55 2006/09/01 13:18:42 jcsjcs Exp $
+|  $Id: file.h 1065 2007-05-06 14:16:17Z jcsjcs $
 */
 
 #ifndef __FILE_H__
@@ -53,23 +53,29 @@ typedef enum
     FILE_TYPE_MOV,
     FILE_TYPE_MPG,
     FILE_TYPE_M3U,
+    FILE_TYPE_OGG,
+    FILE_TYPE_FLAC,
     FILE_TYPE_PLS,
     FILE_TYPE_IMAGE,
     FILE_TYPE_DIRECTORY
 } FileType;
 
+
+/* Don't change the order of this enum -- when exporting playlists the
+   file requester depends on having these in order because the toggle
+   buttons are arranged that way */
 typedef enum
 {
     SOURCE_PREFER_LOCAL = 0,
-    SOURCE_PREFER_IPOD,
     SOURCE_LOCAL,
-    SOURCE_IPOD
+    SOURCE_IPOD,
+    SOURCE_PREFER_IPOD
 } FileSource;
 
 
 typedef void (*AddTrackFunc)(Playlist *plitem, Track *track, gpointer data);
 
-FileType determine_file_type(gchar *path);
+FileType determine_file_type (const gchar *path);
 gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
 				Playlist *plitem, gboolean descend,
 				AddTrackFunc addtrackfunc, gpointer data);
@@ -91,7 +97,6 @@ iTunesDB *gp_import_itdb (iTunesDB *old_itdb, const gint type,
 			  const gchar *mp, const gchar *name_off,
 			  const gchar *name_loc);
 void gp_load_ipods (void);
-iTunesDB *gp_merge_itdb (iTunesDB *old_itdb);
 iTunesDB *gp_load_ipod (iTunesDB *itdb);
 gboolean gp_eject_ipod(iTunesDB *itdb);
 gboolean gp_save_itdb (iTunesDB *itdb);
@@ -102,7 +107,6 @@ gboolean files_are_saved (void);
 gchar *get_file_name_from_source (Track *track, FileSource source);
 gchar* get_preferred_track_name_format(Track *s);
 void mark_track_for_deletion (iTunesDB *itdb, Track *track);
-void unmark_track_for_deletion (iTunesDB *itdb, Track *track);
 void gp_info_deleted_tracks (iTunesDB *itdb,
 			     gdouble *size, guint32 *num);
 void update_charset_info (Track *track);
