@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-06-25 11:14:46 jcs>
+/* Time-stamp: <2007-02-24 13:22:58 jcs>
 |
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
@@ -25,26 +25,27 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file_export.c,v 1.50 2005/06/25 02:52:25 jcsjcs Exp $
+|  $Id: file_export.c 1824 2007-12-13 10:51:04Z dforsi $
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include "misc.h"
-#include "prefs.h"
-#include "file.h"
 #include "charset.h"
+#include "file.h"
 #include "info.h"
-#include "md5.h"
-#include <limits.h>
-#include <stdio.h>
+#include "sha1.h"
+#include "misc.h"
+#include "misc_track.h"
+#include "prefs.h"
 #include <errno.h>
+#include <limits.h>
+#include <math.h>
+#include <stdio.h>
+#include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <string.h>
-#include <math.h>
-
+#include <unistd.h>
 
 /* Structure to keep all necessary information */
 struct fcd
@@ -91,37 +92,14 @@ static gboolean mutex_data = FALSE;
 
 
 /**
- * Recursively make directories in the given filename.
- * @return FALSE is this is not possible.
- */
-gboolean
-mkdirhier(char* filename)
-{
-	char* p = filename;
-	if (*p == G_DIR_SEPARATOR) p++;
-	while ((p = index(p, G_DIR_SEPARATOR)) != NULL) {
-		*p = '\0';
-		if (!g_file_test(filename, G_FILE_TEST_EXISTS)) {
-			if (mkdir(filename, 0755) == -1) {
-				gtkpod_warning (_("Error creating %s: %s\n"), filename, g_strerror(errno));
-				return FALSE;
-			}
-		}
-		*p = G_DIR_SEPARATOR;
-		p++;
-	}
-	return TRUE;
-}
-
-/**
- * copy_file_from_fd_to_fd - given two open file descriptors, read from one
- * 	and write the data to the other
+ * copy_file_fd_sync - given two open file descriptors, read from one
+ * 	and write the data to the other, fsync() before returning.
  * @from - the file descriptor we're reading from
  * @to - the file descriptor we're writing to
  * Returns TRUE on write success, FALSE on write failure
  */
 static gboolean
-copy_file_fd(FILE *from, FILE *to)
+copy_file_fd_sync (FILE *from, FILE *to)
 {
     gboolean result = FALSE;
     gchar data[READ_WRITE_BLOCKSIZE];
@@ -162,6 +140,8 @@ copy_file_fd(FILE *from, FILE *to)
 	    
 	} while(!(feof(from)));
     }
+    if (!result)
+	result = fsync (fileno (to));
     return(result);
 }
 
@@ -216,7 +196,7 @@ copy_file(gchar *file, gchar *dest)
     {
 	if((to = fopen(dest, "w")))
 	{
-	    result = copy_file_fd(from, to);
+	    result = copy_file_fd_sync (from, to);
 	    fclose(to);
 	}
 	else
@@ -234,7 +214,7 @@ copy_file(gchar *file, gchar *dest)
     }
     else
     {
-	gtkpod_warning (_("Unable to open '%s' for reading\n"), file);
+	gtkpod_warning (_("Could not open '%s' for reading.\n"), file);
     }
     return(result);
 }
@@ -268,7 +248,7 @@ track_get_export_filename (Track *track)
     g_return_val_if_fail (track, NULL);
 
     prefs_get_string_value (EXPORT_FILES_TPL, &template);
-    res_utf8 = get_string_from_template (track, template, TRUE);
+    res_utf8 = get_string_from_full_template (track, template, TRUE);
     C_FREE (template);
 
     prefs_get_int_value (EXPORT_FILES_SPECIAL_CHARSET, &special_charset);
@@ -306,11 +286,13 @@ static gboolean write_track (struct fcd *fcd)
 	gchar *from_file = NULL;
 	if (fcd->track->itdb->usertype & GP_ITDB_TYPE_IPOD)
 	{
-	    from_file = get_file_name_on_ipod (fcd->track);
+	    from_file = get_file_name_from_source (fcd->track,
+						   SOURCE_IPOD);
 	}
 	else if (fcd->track->itdb->usertype & GP_ITDB_TYPE_LOCAL)
 	{
-	    from_file = get_file_name_on_harddisk (fcd->track);
+	    from_file = get_file_name_from_source (fcd->track,
+						   SOURCE_LOCAL);
 	}
 	else
 	{
@@ -322,7 +304,7 @@ static gboolean write_track (struct fcd *fcd)
 	    prefs_get_string_value (EXPORT_FILES_PATH, &dest_dir);
 	    filename = g_build_filename (dest_dir, dest_file, NULL);
 
-	    if (mkdirhier(filename))
+	    if (mkdirhierfile(filename))
 	    {
 		if(copy_file(from_file, filename))
 		{
@@ -341,8 +323,10 @@ static gboolean write_track (struct fcd *fcd)
 	}
 	else
 	{
+	    gchar *buf = get_track_info (fcd->track, FALSE);
 	    gtkpod_warning (_("Could find file for '%s' on the iPod\n"),
-			    get_track_info (fcd->track, FALSE));
+			    buf);
+	    g_free (buf);
 	}
 	g_free(dest_file);
     }
@@ -362,7 +346,7 @@ static gpointer th_write_track (gpointer fcd)
     g_cond_signal (cond);
     g_mutex_unlock (mutex);
 
-    return (gpointer)result;
+    return GINT_TO_POINTER(result);
 }
 #endif
 
@@ -452,8 +436,8 @@ static void export_files_write (struct fcd *fcd)
 	start = time(NULL);
 	for(l = fcd->tracks; l && !abort; l = l->next)
 	{
+	    gboolean resultWrite = TRUE;
 	    Track *tr = (Track*)l->data;
-	    gchar *buf;
 
 	    fcd->track = tr;
 	    copied += tr->size;
@@ -473,32 +457,38 @@ static void export_files_write (struct fcd *fcd)
 		    g_cond_timed_wait (cond, mutex, &gtime);
 		} while(!mutex_data);
 		g_mutex_unlock (mutex);
-		result &= (gboolean)g_thread_join (thread);
+		resultWrite = (gboolean)GPOINTER_TO_INT(g_thread_join (thread));
+		result &= resultWrite;
 	    }
 	    else {
 		g_warning ("Thread creation failed, falling back to default.\n");
-		result &= write_track (fcd);
+		resultWrite = write_track (fcd);
+		result &= resultWrite;
 	    }
 #else
-	    result &= write_track (fcd);
+	    resultWrite = write_track (fcd);
+	    result &= resultWrite;
 	    while (widgets_blocked && gtk_events_pending ())
 		gtk_main_iteration ();
 #endif
-	    if (!result)
+	    if (!resultWrite)
 	    {
 		gtkpod_warning (_("Failed to write '%s-%s'\n"), tr->artist, tr->title);	
 	    }
 
 	    ++count;
-	    if (count == 1)  /* we need longer timeout */
-		prefs_set_statusbar_timeout (3*STATUSBAR_TIMEOUT);
+	    if (count == 1) /* we need longer timeout */
+	    {
+		gtkpod_statusbar_timeout (3*STATUSBAR_TIMEOUT);
+	    }
+	        
 	    if (count == n)  /* we need to reset timeout */
-		prefs_set_statusbar_timeout (0);
-	    buf = g_strdup_printf (ngettext ("Copied %d of %d track.",
-					     "Copied %d of %d tracks.", n),
-				   count, n);
-	    gtkpod_statusbar_message(buf);
-	    g_free (buf);
+	    {
+		gtkpod_statusbar_timeout (0);
+	    }
+	    gtkpod_statusbar_message (ngettext ("Copied %d of %d track.",
+						"Copied %d of %d tracks.", n),
+				      count, n);
 
 	    fraction = copied/total;
 
@@ -564,14 +554,22 @@ void export_files_init (GList *tracks, GList **filenames,
     struct fcd *fcd;
     GtkWidget *fc;
     GladeXML *export_files_xml;
+    iTunesDB *itdb = NULL;
 
-    /* no export possible if in offline mode */
-    if (tracks && prefs_get_offline ())
+    if (tracks)
     {
 	Track *tr = tracks->data;
 	g_return_if_fail (tr);
-	g_return_if_fail (tr->itdb);
-	if (tr->itdb->usertype & GP_ITDB_TYPE_IPOD)
+	itdb = tr->itdb;
+	g_return_if_fail (itdb);
+    }
+
+    /* no export possible if in offline mode */
+    if (tracks && get_offline (itdb))
+    {
+	Track *tr = tracks->data;
+	g_return_if_fail (tr);
+	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
 	{
 	    GtkWidget *dialog = gtk_message_dialog_new (
 		GTK_WINDOW (gtkpod_window),
@@ -594,9 +592,9 @@ void export_files_init (GList *tracks, GList **filenames,
 	NULL);
 
     export_files_xml = glade_xml_new (xml_file, "export_files_options", NULL);
-    win = glade_xml_get_widget (export_files_xml, "export_files_options");
-    options = glade_xml_get_widget (export_files_xml, "options_frame");
-    message_box = glade_xml_get_widget (export_files_xml, "message_box");
+    win = gtkpod_xml_get_widget (export_files_xml, "export_files_options");
+    options = gtkpod_xml_get_widget (export_files_xml, "options_frame");
+    message_box = gtkpod_xml_get_widget (export_files_xml, "message_box");
 
     /* Information needed to clean up later */
     fcd = g_malloc0 (sizeof (struct fcd));
@@ -620,9 +618,9 @@ void export_files_init (GList *tracks, GList **filenames,
     if (display)
     {
 	GList *gl;
-	GtkWidget *label = glade_xml_get_widget (export_files_xml,
+	GtkWidget *label = gtkpod_xml_get_widget (export_files_xml,
 						 "message");
-	GtkWidget *tv = glade_xml_get_widget (export_files_xml,
+	GtkWidget *tv = gtkpod_xml_get_widget (export_files_xml,
 					      "textview");
 	GtkTextBuffer *tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW(tv));
 	if (message)  gtk_label_set_text (GTK_LABEL (label), message);
@@ -718,8 +716,8 @@ GList *export_trackglist_when_necessary (iTunesDB *itdb_s,
     g_return_val_if_fail (itdb_d, NULL);
     g_return_val_if_fail (gtkpod_window, NULL);
 
-    if (!(itdb_s->usertype & GP_ITDB_TYPE_IPOD) ||
-	!(itdb_d->usertype & GP_ITDB_TYPE_LOCAL))
+    if (!((itdb_s->usertype & GP_ITDB_TYPE_IPOD) &&
+	  (itdb_d->usertype & GP_ITDB_TYPE_LOCAL)))
     {   /* drag is not from iPod to local database -> return copy of
 	 * @tracks */
 	return g_list_copy (tracks);
@@ -728,7 +726,7 @@ GList *export_trackglist_when_necessary (iTunesDB *itdb_s,
     /* drag is from iPod to local database */
 
     /* no drag possible if in offline mode */
-    if (prefs_get_offline ())
+    if (get_offline (itdb_s))
     {
 	GtkWidget *dialog = gtk_message_dialog_new (
 	    GTK_WINDOW (gtkpod_window),
@@ -749,7 +747,7 @@ GList *export_trackglist_when_necessary (iTunesDB *itdb_s,
 	Track *tr = gl->data;
 	g_return_val_if_fail (tr, NULL);
 
-	otr = md5_track_exists (itdb_d, tr);
+	otr = sha1_track_exists (itdb_d, tr);
 
 	if (otr)
 	{
@@ -945,9 +943,9 @@ static void export_playlist_file_write (struct fcd *fcd)
 
     num = g_list_length (fcd->tracks);
 
-    prefs_get_int_value (EXPORT_PLAYLIST_FILE_TYPE, &type);
-    prefs_get_int_value (EXPORT_PLAYLIST_FILE_SOURCE, &source);
-    prefs_get_string_value (EXPORT_PLAYLIST_FILE_TPL, &template);
+    type = prefs_get_int (EXPORT_PLAYLIST_FILE_TYPE);
+    source = prefs_get_int (EXPORT_PLAYLIST_FILE_SOURCE);
+    template = prefs_get_string (EXPORT_PLAYLIST_FILE_TPL);
     if (!template)
 	template = g_strdup (EXPORT_PLAYLIST_FILE_TPL_DFLT);
 
@@ -958,7 +956,6 @@ static void export_playlist_file_write (struct fcd *fcd)
     if (file)
     {
 	guint i,n;
-	gchar *buf;
 
 	switch (type)
 	{
@@ -973,9 +970,8 @@ static void export_playlist_file_write (struct fcd *fcd)
 	for (n=0,i=0; i<num; ++i)
 	{
 	    Track *track = g_list_nth_data (fcd->tracks, i);
-	    gchar *infotext_utf8 = get_string_from_template (track,
-							     template,
-							     FALSE);
+	    gchar *infotext_utf8 = get_string_from_full_template (
+		track, template, FALSE);
 	    gchar *filename = get_file_name_from_source (track,
 							 source);
 	    gchar *infotext;
@@ -1026,11 +1022,9 @@ static void export_playlist_file_write (struct fcd *fcd)
 	    break;
 	}
 	fclose (file);
-	buf = g_strdup_printf (
+	gtkpod_statusbar_message  (
 	    ngettext ("Created playlist with one track.",
 		      "Created playlist with %d tracks.", n), n);
-	gtkpod_statusbar_message (buf);
-	g_free (buf);
     }
     else
     {
@@ -1098,9 +1092,11 @@ void export_playlist_file_init (GList *tracks)
     GladeXML *export_playlist_xml;
     
     export_playlist_xml = glade_xml_new (xml_file, "export_playlist_file_options", NULL);
-    win = glade_xml_get_widget (export_playlist_xml, "export_playlist_file_options");
+    win = gtkpod_xml_get_widget (export_playlist_xml, "export_playlist_file_options");
 
-    options = glade_xml_get_widget (export_playlist_xml, "options_frame");
+    options = gtkpod_xml_get_widget (export_playlist_xml, "options_frame");
+
+    fcd->win_xml = export_playlist_xml;
 
     /* Information needed to clean up later */
     fcd->tracks = g_list_copy (tracks);
