@@ -1,6 +1,5 @@
-/* Time-stamp: <2005-11-25 23:52:12 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: info.c,v 1.34 2005/11/26 13:44:41 jcsjcs Exp $
+|  $Id: info.c 1550 2007-06-17 14:14:38Z jcsjcs $
 */
 
 /* This file provides functions for the info window as well as for the
@@ -50,14 +49,19 @@ static GtkWidget *gtkpod_statusbar = NULL;
 static GtkWidget *gtkpod_tracks_statusbar = NULL;
 static GtkWidget *gtkpod_space_statusbar = NULL;
 static guint statusbar_timeout_id = 0;
+static guint statusbar_timeout = STATUSBAR_TIMEOUT;
 
+#define SPACE_TIMEOUT 1000
 /* lock for size related variables (used by child and parent) */
 static GMutex *space_mutex = NULL;
 static GThread *space_thread = NULL;
 static gboolean space_uptodate = FALSE;
-static gchar *space_mp = NULL;
-static gdouble space_ipod_free = 0;
-static gdouble space_ipod_used = 0;
+static gchar *space_mp = NULL;      /* thread save access through mutex */
+static iTunesDB *space_itdb = NULL; /* semi thread save access
+				     * (space_itdb will not be changed
+				     * while locked) */
+static gdouble space_ipod_free = 0; /* thread save access through mutex */
+static gdouble space_ipod_used = 0; /* thread save access through mutex */
 
 
 static gdouble get_ipod_free_space(void);
@@ -167,9 +171,10 @@ void info_open_window (void)
     if (info_window)
     {
 	gint defx, defy;
-	prefs_get_size_info (&defx, &defy);
+	defx = prefs_get_int("size_info.x");
+  defy = prefs_get_int("size_info.y");
 	gtk_window_set_default_size (GTK_WINDOW (info_window), defx, defy);
-	prefs_set_info_window (TRUE); /* notify prefs */
+	prefs_set_int("info_window", TRUE); /* notify prefs */
 	info_update ();
 	gtk_widget_show (info_window);
 	/* set the menu item for the info window correctly */
@@ -188,7 +193,7 @@ void info_close_window (void)
     win = info_window;
     info_window = NULL;
     gtk_widget_destroy (win);
-     prefs_set_info_window (FALSE); /* notify prefs */
+     prefs_set_int("info_window", FALSE); /* notify prefs */
     /* set the menu item for the info window correctly */
     display_set_info_window_menu ();
 }
@@ -200,7 +205,8 @@ void info_update_default_sizes (void)
     {
 	gint defx, defy;
 	gtk_window_get_size (GTK_WINDOW (info_window), &defx, &defy);
-	prefs_set_size_info (defx, defy);
+	prefs_set_int("size_info.x", defx);
+	prefs_set_int("size_info.y", defy);
     }
 }
 
@@ -220,7 +226,7 @@ static void info_update_track_view_displayed (void)
     GList *displayed;
 
     g_return_if_fail (info_window);
-    displayed = display_get_selected_members (prefs_get_sort_tab_num()-1);
+    displayed = display_get_selected_members (prefs_get_int("sort_tab_num")-1);
     fill_in_info (displayed, &tracks, &playtime, &filesize);
     fill_label_uint ("tracks_displayed", tracks);
     fill_label_time ("playtime_displayed", playtime);
@@ -234,7 +240,7 @@ void info_update_track_view_selected (void)
     GList *selected;
 
     if (!info_window) return; /* not open */
-    selected = display_get_selection (prefs_get_sort_tab_num ());
+    selected = display_get_selection (prefs_get_int("sort_tab_num"));
     fill_in_info (selected, &tracks, &playtime, &filesize);
     g_list_free (selected);
     fill_label_uint ("tracks_selected", tracks);
@@ -364,7 +370,7 @@ void info_update_totals_view_space (void)
 	gp_info_deleted_tracks (itdb, &del_filesize, &del_tracks);
 	fill_label_uint ("deleted_tracks", del_tracks);
 	fill_label_size ("deleted_filesize", del_filesize);
-	if (!prefs_get_offline ())
+	if (!get_offline (itdb))
 	{
 	    if (ipod_connected ())
 	    {
@@ -395,6 +401,7 @@ void
 gtkpod_statusbar_init(void)
 {
     gtkpod_statusbar = gtkpod_xml_get_widget (main_window_xml, "gtkpod_status");
+    statusbar_timeout = STATUSBAR_TIMEOUT;
 }
 
 static gint
@@ -410,28 +417,46 @@ gtkpod_statusbar_clear(gpointer data)
 }
 
 
-void
+static void
 gtkpod_statusbar_reset_timeout (void)
 {
     if (statusbar_timeout_id != 0) /* remove last timeout, if still present */
 	gtk_timeout_remove (statusbar_timeout_id);
-    statusbar_timeout_id = gtk_timeout_add(prefs_get_statusbar_timeout (),
-					   (GtkFunction) gtkpod_statusbar_clear,
-					   NULL);
+    statusbar_timeout_id = gtk_timeout_add (statusbar_timeout,
+					    (GtkFunction) gtkpod_statusbar_clear,
+					    NULL);
+}
+
+void
+gtkpod_statusbar_timeout (guint timeout)
+{
+    if (timeout == 0)
+	statusbar_timeout = STATUSBAR_TIMEOUT;
+    else
+	statusbar_timeout = timeout;
+
+    gtkpod_statusbar_reset_timeout ();
 }
 
 
 void
-gtkpod_statusbar_message(const gchar *message)
+gtkpod_statusbar_message(const gchar *message, ...)
 {
     if(gtkpod_statusbar)
     {
-	gchar buf[PATH_MAX];
+	va_list arg;
+	gchar *text;
 	guint context = 1;
 
-	snprintf(buf, PATH_MAX, "  %s", message);
+	va_start (arg, message);
+	text = g_strdup_vprintf (message, arg);
+	va_end (arg);
+
 	gtk_statusbar_pop(GTK_STATUSBAR(gtkpod_statusbar), context);
-	gtk_statusbar_push(GTK_STATUSBAR(gtkpod_statusbar), context,  buf);
+	gtk_statusbar_push(GTK_STATUSBAR(gtkpod_statusbar), context,  text);
+
+	g_free (text);
+
 	gtkpod_statusbar_reset_timeout ();
     }
 }
@@ -450,26 +475,26 @@ gtkpod_tracks_statusbar_update(void)
     if(gtkpod_tracks_statusbar)
     {
 	gchar *buf;
-	iTunesDB *itdb;
 	Playlist *pl;
 	pl = pm_get_selected_playlist ();
 	/* select of which iTunesDB data should be displayed */
 	if (pl)
 	{
-	    itdb = pl->itdb;
+	    iTunesDB *itdb = pl->itdb;
+	    g_return_if_fail (itdb);
+
+	    buf = g_strdup_printf (_(" P:%d T:%d/%d"),
+				   itdb_playlists_number (itdb) - 1,
+				   tm_get_nr_of_tracks (),
+				   itdb_tracks_number (itdb));
 	}
 	else
 	{
-	    itdb = get_itdb_ipod ();
+	    buf = g_strdup ("");
 	}
 	/* gets called before itdbs are setup up -> fail silently */
 /*	g_return_if_fail (itdb);*/
-	if (!itdb) return;
 	
-	buf = g_strdup_printf (_(" P:%d T:%d/%d"),
-			       itdb_playlists_number (itdb) - 1,
-			       tm_get_nr_of_tracks (),
-			       itdb_tracks_number (itdb));
 	gtk_statusbar_pop(GTK_STATUSBAR(gtkpod_tracks_statusbar), 1);
 	gtk_statusbar_push(GTK_STATUSBAR(gtkpod_tracks_statusbar), 1,  buf);
 	g_free (buf);
@@ -489,21 +514,46 @@ gtkpod_tracks_statusbar_update(void)
    be accessed securely by using a locking mechanism. Therefore we
    keep a copy of the mount point here. Access must only be done
    after locking. */
-void space_set_ipod_mount (const gchar *mp)
+void space_set_ipod_itdb (iTunesDB *itdb)
 {
-    g_return_if_fail (mp);
+    const gchar *mp = NULL;
+
+    if (itdb)
+    {
+	ExtraiTunesDBData *eitdb = itdb->userdata;
+	g_return_if_fail (eitdb);
+
+	if (!eitdb->ipod_ejected)
+	{
+	    mp = itdb_get_mountpoint (itdb);
+	}
+    }
+
     if (space_mutex)  g_mutex_lock (space_mutex);
 
-    /* update the free space data if mount point changed */
-    if (!space_mp || (strcmp (space_mp, mp) != 0))
-	space_data_update ();
+    space_itdb = itdb;
 
-    g_free (space_mp);
-    space_mp = g_strdup (mp);
+    /* update the free space data if mount point changed */
+    if (!space_mp || !mp || (strcmp (space_mp, mp) != 0))
+    {
+	g_free (space_mp);
+	space_mp = g_strdup (mp);
+
+	space_data_update ();
+    }
 
     if (space_mutex)   g_mutex_unlock (space_mutex);
+
 }
 
+/* retrieve the currently set ipod itdb -- needed in case the itdb is
+   deleted */
+iTunesDB *space_get_ipod_itdb (void)
+{
+    return space_itdb;
+}
+
+
 
 /* iPod space has to be reread */
 void space_data_update (void)
@@ -529,7 +579,7 @@ gboolean ipod_connected (void)
 
 /* we'll use statvfs to determine free space on the iPod where
    available, df otherwise */
-#ifdef HAVE_statvfs
+#ifdef HAVE_STATVFS
 #include <sys/types.h>
 #include <sys/statvfs.h>
 /* update space_ipod_free and space_ipod_used */
@@ -539,15 +589,13 @@ static void th_space_update (void)
     struct statvfs stat;
     int	status;
 
-    /* don't read info when in offline mode */
-    if (!prefs_get_offline ())
-    {
-	g_mutex_lock (space_mutex);
-	mp = g_strdup (space_mp);
-	g_mutex_unlock (space_mutex);
-    }
-
     g_mutex_lock (space_mutex);
+
+    /* don't read info when in offline mode */
+    if (space_itdb && !get_offline (space_itdb))
+    {
+	mp = g_strdup (space_mp);
+    }
     if (mp)
     {
 	status = statvfs (mp, &stat);
@@ -755,22 +803,19 @@ get_filesize_as_string(gdouble size)
 static guint
 gtkpod_space_statusbar_update(void)
 {
-    if(gtkpod_space_statusbar)
+    if(space_itdb && gtkpod_space_statusbar)
     {
 	gchar *buf = NULL;
 	gchar *str = NULL;
 
-	if (!prefs_get_offline ())
+	if (!get_offline (space_itdb))
 	{
 	    if (ipod_connected ())
 	    {
 		gdouble left, pending, deleted;
-		iTunesDB *itdb = get_itdb_ipod ();
-		/* we may be called before any itdb is present */
-		if (!itdb) return TRUE;
 
-		gp_info_deleted_tracks (itdb, &deleted, NULL);
-		gp_info_nontransferred_tracks (itdb, &pending, NULL);
+		gp_info_deleted_tracks (space_itdb, &deleted, NULL);
+		gp_info_nontransferred_tracks (space_itdb, &pending, NULL);
 		left = get_ipod_free_space() + deleted;
 		if((left-pending) > 0)
 		{
@@ -798,7 +843,7 @@ gtkpod_space_statusbar_update(void)
 	g_free (str);
     }
     info_update_totals_view_space ();
-    return(TRUE);
+    return TRUE;
 }
 
 void
@@ -810,10 +855,47 @@ gtkpod_space_statusbar_init(void)
     {
 	space_mutex = g_mutex_new ();
 	if (!space_mp)
-	    space_mp = g_strdup (prefs_get_ipod_mount ());
-	th_space_update ();  /* make sure we have current data */
-	space_thread = g_thread_create (th_space_thread, NULL, FALSE, NULL);
+	{
+	    iTunesDB *itdb = gp_get_ipod_itdb ();
+
+	    if (itdb)
+	    {
+		space_mp = get_itdb_prefs_string (itdb, KEY_MOUNTPOINT);
+		th_space_update ();  /* make sure we have current data */
+	    }
+	}
+	space_thread = g_thread_create (th_space_thread,
+					    NULL, FALSE, NULL);
     }
     gtkpod_space_statusbar_update();
     gtk_timeout_add(1000, (GtkFunction) gtkpod_space_statusbar_update, NULL);
 }
+
+
+
+/*------------------------------------------------------------------*\
+ *                                                                  *
+ *              Frequently used error messages                      *
+ *                                                                  *
+\*------------------------------------------------------------------*/
+
+void message_sb_no_itdb_selected ()
+{
+    gtkpod_statusbar_message (_("No database or playlist selected"));
+}
+
+void message_sb_no_tracks_selected ()
+{
+    gtkpod_statusbar_message (_("No tracks selected"));
+}
+
+void message_sb_no_playlist_selected ()
+{
+    gtkpod_statusbar_message (_("No playlist selected"));
+}
+
+void message_sb_no_ipod_itdb_selected ()
+{
+    gtkpod_statusbar_message (_("No iPod or iPod playlist selected"));
+}
+
