@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-11-27 15:52:33 jcs>
+/* Time-stamp: <2006-06-05 21:34:12 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -30,7 +30,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb.h,v 1.18 2005/11/28 16:25:34 jcsjcs Exp $
+|  $Id: itdb.h 1750 2007-11-01 21:57:01Z phantom_sf $
 */
 
 #ifndef ITDB_H
@@ -47,4 +47,7 @@ typedef Itdb_Playlist Playlist;
 typedef Itdb_Track Track;
 typedef Itdb_Thumb Thumb;
 typedef Itdb_Artwork Artwork;
+typedef Itdb_IpodInfo IpodInfo;
+typedef Itdb_PhotoDB PhotoDB;
+typedef Itdb_PhotoAlbum PhotoAlbum;
 #endif
