@@ -1,7 +1,6 @@
 /* -*- coding: utf-8; -*-
-|  Time-stamp: <2006-06-28 01:21:08 jcs>
 |
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -25,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc.c,v 1.213 2006/06/27 16:30:02 jcsjcs Exp $
+|  $Id: misc.c 1822 2007-12-12 16:31:26Z dforsi $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -34,229 +33,152 @@
 
 #include <errno.h>
 #include <gtk/gtk.h>
-#include <stdlib.h>
+#include <glib/gstdio.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include "autodetection.h"
 #include "charset.h"
+#include "clientserver.h"
+#include "file_convert.h"
 #include "misc.h"
 #include "prefs.h"
 #include "misc_track.h"
-
+#include "display_photo.h"
+#include "stock_icons.h"
 
 #define DEBUG_MISC 0
 
+
+/* where to find the scripts */
+const gchar *SCRIPTDIR = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "scripts" G_DIR_SEPARATOR_S;
+
+
 /*------------------------------------------------------------------*\
  *                                                                  *
  *             About Window                                         *
  *                                                                  *
 \*------------------------------------------------------------------*/
 
-static GtkWidget *about_window = NULL;
-
 /* ATTENTION: directly used as callback in gtkpod.glade -- if you
    change the arguments of this function make sure you define a
    separate callback for gtkpod.glade */
 void open_about_window ()
 {
-  GtkLabel *about_label;
-  gchar *label_text;
-  GtkTextView *textview;
-  GtkTextIter ti;
-  GtkTextBuffer *tb;
-  GladeXML *about_xml;
+	const gchar *authors[] = {
+		_("© 2002 - 2007\n"
+		    "Jorg Schuler (jcsjcs at users dot sourceforge dot net)\n"
+		    "Corey Donohoe (atmos at atmos dot org)\n"
+		    ""),
+		_("Patches were supplied by the following people (list may be incomplete -- please contact me)\n"),
+		_("Ramesh Dharan: Multi-Edit (edit tags of several tracks in one run)"),
+		_("Hiroshi Kawashima: Japanese charset autodetection feature"),
+		_("Adrian Ulrich: porting of playlist code from mktunes.pl to itunesdb.c"),
+		_("Walter Bell: correct handling of DND URIs with escaped characters and/or cr/newlines at the end"),
+		_("Sam Clegg: user defined filenames when exporting tracks from the iPod"),
+		_("Chris Cutler: automatic creation of various playlist types"),
+		_("Graeme Wilford: reading and writing of the 'Composer' ID3 tags, progress dialogue during sync"),
+		_("Edward Matteucci: debugging, special playlist creation, most of the volume normalizing code"),
+		_("Jens Lautenbach: some optical improvements"),
+		_("Alex Tribble: iPod eject patch"),
+		_("Yaroslav Halchenko: Orphaned and dangling tracks handling"),
+		_("Andrew Huntwork: Filename case sensitivity fix and various other bugfixes"),
+		_("Ero Carrera: Filename validation and quick sync when copying tracks from the iPod"),
+		_("Jens Taprogge: Support for LAME's replay gain tag to normalize volume"),
+		_("Armando Atienza: Support with external playcounts"),
+		_("D.L. Sharp: Support for m4b files (bookmarkable AAC files)"),
+		_("Jim Hall: Decent INSTALL file"),
+		_("Juergen Helmers, Markus Gaugusch: Conversion scripts to sync calendar/contacts to the iPod"),    /* J"urgen! */
+		_("Flavio Stanchina: bugfixes"),
+		_("Chris Micacchi: when sorting ignore 'the' and similar at the beginning of the title"),
+		_("Steve Jay: use statvfs() instead of df (better portability, faster)"),
+		"",
+		_("Christoph Kunz: address compatibility issues when writing id3v2.4 type mp3 tags"),
+		"",
+		_("James Liggett:\n"
+		    "replacement of old GTK file selection dialogs with new GTK filechooser dialogs\n"
+		    "refactored user preferences system."),
+		"",
+		_("Daniel Kercher: sync scripts for abook and webcalendar"),
+		"",
+		_("Clinton Gormley: sync scripts for thunderbird"),
+		"",
+		_("Sebastien Beridot: sync script for ldif addressbook format"),
+		"",
+		_("Sebastian Scherer: sync script for kNotes"),
+		"",
+		_("Nick Piper: sync script for Palm, type-ahead search"),
+		"",
+		_("Uwe Hermann: help with support for iPod Video"),
+		"",
+		_("Iain Benson: support for compilation tag in mp3 files and separate display of compilations in the sort tab."),
+		_("Nicolas Chariot: icons of buttons\n"
+		    "\n"),
+		_("This program borrows code from the following projects:"),
+		_("gnutools: (mktunes.pl, ported to C) reading and writing of iTunesDB  (http://www.gnu.org/software/gnupod/)"),
+		_("iPod.cpp, iPod.h by Samuel Wood (sam dot wood at gmail dot com): some code for smart playlists is based on his C++-classes."),
+		_("mp3info: mp3 playlength detection (http://ibiblio.org/mp3info/)"),
+		_("xmms: dirbrowser, mp3 playlength detection (http://www.xmms.org)"),
+		"",
+		_("The GUI was created with the help of glade-2 (http://glade.gnome.org/)."),
+		NULL };
 
-  if (about_window != NULL) return;
-  /* about_window = create_gtkpod_about_window (); */
+	gchar  *translators[] = {
+		_("French: David Le Brun (david at dyn-ns dot net)"),
+		_("French: Éric Lassauge (rpmfarm at free dot fr)"),
+		_("German: Jorg Schuler (jcsjcs at users dot sourceforge dot net)"),
+		_("German: Kai-Ove"),
+		_("Hebrew: Assaf Gillat (gillata at gmail dot com)"),
+		_("Italian: Edward Matteucci (edward_matteucc at users dot sourceforge dot net)"),
+		_("Italian: Daniele Forsi (dforsi at gmail dot com)"),
+		_("Japanese: Ayako Sano"),
+		_("Japanese: Kentaro Fukuchi (fukuchi at users dot sourceforge dot net)"),
+		_("Romanian: Alex Eftimie (alexeftimie at gmail dot com)"),
+		_("Spanish: Alejandro Lamas Daviña (alejandro.lamas at ific dot uv dot es)"),
+		_("Swedish: Stefan Asserhall (stefan.asserhall at comhem dot se)"),
+		NULL
+	};
 
-  about_xml = glade_xml_new (xml_file, "gtkpod_about_window", NULL);
-  glade_xml_signal_autoconnect (about_xml);
-  about_window = gtkpod_xml_get_widget (about_xml, "gtkpod_about_window");
-  
-  
-  about_label = GTK_LABEL (gtkpod_xml_get_widget (about_xml, "about_label"));
-  label_text = g_strdup_printf (_("gtkpod Version %s: Cross-Platform Multi-Lingual Interface to Apple's iPod(tm)."), VERSION);
-  gtk_label_set_text (about_label, label_text);
-  g_free (label_text);
-  {
-      gchar *text[] = {_("\
-(C) 2002 - 2005\n\
-Jorg Schuler (jcsjcs at users dot sourceforge dot net)\n\
-Corey Donohoe (atmos at atmos dot org)\n\
-\n\
-\n"),
-		       _("\
-This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.\n\
-\n\
-This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\
-\n\
-You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.\n\
-\n\
-\n"),
-		       _("\
-Patches were supplied by the following people (list may be incomplete -- please contact me)\n\n"),
-		       _("\
-Ramesh Dharan: Multi-Edit (edit tags of several tracks in one run)\n"),
-		       _("\
-Hiroshi Kawashima: Japanese charset autodetecion feature\n"),
-		       _("\
-Adrian Ulrich: porting of playlist code from mktunes.pl to itunesdb.c\n"),
-		       _("\
-Walter Bell: correct handling of DND URIs with escaped characters and/or cr/newlines at the end\n"),
-		       _("\
-Sam Clegg: user defined filenames when exporting tracks from the iPod\n"),
-		       _("\
-Chris Cutler: automatic creation of various playlist types\n"),
-		       _("\
-Graeme Wilford: reading and writing of the 'Composer' ID3 tags, progress dialogue during sync\n"),
-		       _("\
-Edward Matteucci: debugging, special playlist creation, most of the volume normalizing code\n"),
-		       _("\
-Jens Lautenbach: some optical improvements\n"),
-		       _("\
-Alex Tribble: iPod eject patch\n"),
-		       _("\
-Yaroslav Halchenko: Orphaned and dangling tracks handling\n"),
-		       _("\
-Andrew Huntwork: Filename case sensitivity fix and various other bugfixes\n"),
-		       _("\
-Ero Carrera: Filename validation and quick sync when copying tracks from the iPod\n"),
-		       _("\
-Jens Taprogge: Support for LAME's replay gain tag to normalize volume\n"),
-		       _("\
-Armando Atienza: Support with external playcounts\n"),
-		       _("\
-D.L. Sharp: Support for m4b files (bookmarkable AAC files)\n"),
-		       _("\
-Jim Hall: Decent INSTALL file\n"),
-		       _("\
-Juergen Helmers, Markus Gaugusch: Conversion scripts to sync calendar/contacts to the iPod\n"),    /* J"urgen! */
-		       _("\
-Flavio Stanchina: bugfixes\n"),
-		       _("\
-Chris Micacchi: when sorting ignore 'the' and similar at the beginning of the title\n"),
-		       _("\
-Steve Jay: use statvfs() instead of df (better portability, faster)\n"),
-		       "\n",
-		       _("\
-Christoph Kunz: address compatibility issues when writing id3v2.4 type mp3 tags\n"),
-		       "\n",
-		       _("\
-James Liggett:\n\
-     replacement of old GTK file selection dialogs with new GTK filechooser dialogs\n\
-     refactored user preferences system.\n"),
-                       "\n",
-		       _("\
-Daniel Kercher: sync scripts for abook and webcalendar\n"),
-		       "\n",
-		       _("\
-Clinton Gormley: sync scripts for thunderbird\n"),
-		       "\n",
-		       _("\
-Sebastien Beridot: sync script for ldif addressbook format\n"),
-		       "\n",
-		       _("\
-Sebastian Scherer: sync script for kNotes\n"),
-		       "\n",
-		       _("\
-Nick Piper: sync script for Palm, type-ahead search\n"),
-		       "\n",
-		       _("\
-Uwe Hermann: help with support for iPod Video\n"),
-		       "\n",
-		       _("\
-Iain Benson: support for compilation tag in mp3 files and separate display of compilations in the sort tab.\n"),
-		       _("\
-Nicolas Chariot: icons of buttons\n\
-\n\
-\n"),
-		       _("\
-This program borrows code from the following projects:\n"),
-		       _("\
-    gnutools: (mktunes.pl, ported to C) reading and writing of iTunesDB (http://www.gnu.org/software/gnupod/)\n"),
-		       _("\
-    iPod.cpp, iPod.h by Samuel Wood (sam dot wood at gmail dot com): some code for smart playlists is based on his C++-classes.\n"),
-		       _("\
-    mp3info:  mp3 playlength detection (http://ibiblio.org/mp3info/)\n"),
-		       _("\
-    xmms:     dirbrowser, mp3 playlength detection (http://www.xmms.org)\n"),
-		       "\n",
-		       _("\
-The GUI was created with the help of glade-2 (http://glade.gnome.org/).\n"),
-		       NULL };
-      gchar **strp = text;
-      textview = GTK_TEXT_VIEW (gtkpod_xml_get_widget (about_xml, "credits_textview"));
-      tb = gtk_text_view_get_buffer (textview);
-      while (*strp)
-      {
-	  gtk_text_buffer_get_end_iter (tb, &ti);
-	  gtk_text_buffer_insert (tb, &ti, *strp, -1);
-	  ++strp;
-      }
-  }
+	gchar *license = _(
+		"This program is free software; you can redistribute it and/or\n"
+		"modify it under the terms of the GNU General Public License as\n"
+		"published by the Free Software Foundation; either version 2 of the\n"
+		"License, or (at your option) any later version.\n"
+		"\n"
+		"This program is distributed in the hope that it will be useful, but\n"
+		"WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See\n"
+		"the GNU General Public License for more details.\n"
+		"\n"
+		"You should have received a copy of the GNU General Public\n"
+		"License along with this program; if not, write to the Free Software\n"
+		"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA\n02111-1307, USA."
+	);
 
- {
-     gchar  *text[] = { _("\
-French:   David Le Brun (david at dyn-ns dot net)\n"),
-				     _("\
-German:   Jorg Schuler (jcsjcs at users dot sourceforge dot net)\n"),
-			             _("\
-Hebrew: Assaf Gillat (gillata at gmail dot com)\n"),
-				     _("\
-Italian:  Edward Matteucci (edward_matteucc at users dot sourceforge dot net)\n"),
-				     _("\
-Japanese: Ayako Sano\n"),
-				     _("\
-Japanese: Kentaro Fukuchi (fukuchi at users dot sourceforge dot net)\n"),
-				     _("\
-Swedish: Stefan Asserhall (stefan asserhall at comhem dot se)\n"),
-				     NULL };
-      gchar **strp = text;
-      textview = GTK_TEXT_VIEW (gtkpod_xml_get_widget (about_xml, "translators_textview"));
-      tb = gtk_text_view_get_buffer (textview);
-      while (*strp)
-      {
-	  gtk_text_buffer_get_end_iter (tb, &ti);
-	  gtk_text_buffer_insert (tb, &ti, *strp, -1);
-	  ++strp;
-      }
-  }
+	gchar *copyright = _("© 2002-2007\nJorg Schuler <jcsjcs@users.sourceforge.net>\nCorey Donohoe <atmos@atmos.org>");
+	gchar *translator_credits = g_strjoinv("\n", translators);
+	GdkPixbuf *icon = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "data" G_DIR_SEPARATOR_S "gtkpod-logo.png", NULL);
 
-  gtk_widget_show (about_window);
+	gtk_show_about_dialog(GTK_WINDOW(gtkpod_window),
+		"name", "gtkpod",
+		"version", VERSION,
+		"logo", icon,
+		"comments", _("Cross-platform multilingual interface to Apple's iPod™"),
+		"copyright", copyright,
+		"website", "http://gtkpod.org",
+		"license", license,
+		"authors", authors,
+		"translator_credits", translator_credits,
+		NULL);
+
+	g_free(translator_credits);
+
+	if(icon)
+		g_object_unref(icon);
 }
 
 
-/* callback for close button */
-void
-on_about_window_close_button           (GtkButton       *button,
-					gpointer         user_data)
-{
-  close_about_window (); /* in misc.c */
-}
-
-
-/* callback for window close button */
-gboolean
-on_about_window_close                  (GtkWidget       *widget,
-					GdkEvent        *event,
-					gpointer         user_data)
-{
-  close_about_window (); /* in misc.c */
-  return FALSE;
-}
-
-
-void close_about_window (void)
-{
-  g_return_if_fail (about_window != NULL);
-  gtk_widget_destroy (about_window);
-  about_window = NULL;
-}
-
-
-
-
 /*------------------------------------------------------------------*\
  *                                                                  *
  *             Miscellaneous                                        *
@@ -317,7 +239,34 @@ parse_tracks_from_string(gchar **s, Track **track)
     return FALSE;
 }
 
+gboolean
+parse_artwork_from_string(gchar **s, Artwork **artwork)
+{
+    g_return_val_if_fail (artwork, FALSE);
+    *artwork = NULL;
+    g_return_val_if_fail (s, FALSE);
 
+    if(*s)
+    {
+	gchar *str = *s;
+	gchar *strp = strchr (str, '\n');
+	int tokens;
+
+	if (strp == NULL)
+	{
+	    *artwork = NULL;
+	    *s = NULL;
+	    return FALSE;
+	}
+	tokens = sscanf (str, "%p", artwork);
+	++strp;
+	if (*strp) *s = strp;
+	else       *s = NULL;
+	if (tokens == 1) 	return TRUE;
+	else                    return FALSE;
+    }
+    return FALSE;
+}
 
 /* Duplicate a GList (shallow copy) */
 GList *glist_duplicate (GList *list)
@@ -338,6 +287,7 @@ GList *glist_duplicate (GList *list)
  *
  **************************************************************************/
 
+
 /* tries to call "/bin/sh @script" with command line options */
 static void do_script (const gchar *script, va_list args)
 {
@@ -417,6 +367,54 @@ void call_script (gchar *script, ...)
 
 
 
+/* Create a NULL-terminated array of strings given in the command
+   line. The last argument must be NULL.
+
+   As a special feature, the first argument is split up into
+   individual strings to allow the use of "convert-2mp3 -q <special
+   settings>". Set the first argument to NULL if you don't want this.
+
+   You must free the returned array with g_strfreev() after use. */
+gchar **build_argv_from_strings (const gchar *first_arg, ...)
+{
+    gchar **argv;
+    va_list args;
+    const gchar *str;
+    GPtrArray *ptra = g_ptr_array_sized_new (20);
+
+    if (first_arg)
+    {
+	gchar **strings = g_strsplit (first_arg, " ", 0);
+	gchar **strp = strings;
+	while (*strp)
+	{
+	    if (**strp)
+	    {   /* ignore empty strings */
+		g_ptr_array_add (ptra, g_strdup(*strp));
+	    }
+	    ++strp;
+	}
+	g_strfreev (strings);
+    }
+
+    va_start (args, first_arg);
+    do
+    {
+	str = va_arg (args, const gchar *);
+	g_ptr_array_add (ptra, g_strdup (str));
+    }
+    while (str);
+
+    va_end (args);
+
+    argv = (gchar **)g_ptr_array_free (ptra, FALSE);
+
+    return argv;
+}
+
+
+
+
 /* compare @str1 and @str2 case-sensitively or case-insensitively
  * depending on prefs settings */
 gint compare_string (const gchar *str1, const gchar *str2)
@@ -975,10 +973,14 @@ static gchar *select_template (Track *track, const gchar *p)
 
 /* Return a string for @track built according to @template.
 
-   @is_filename: if TRUE, remove potentially harmful characters.*/
+   @is_filename: if TRUE, remove potentially harmful characters.
+   @silent: don't print error messages (no gtk_*() calls -- thread
+   safe)
+*/
 gchar *get_string_from_template (Track *track,
 				 const gchar *template,
-				 gboolean is_filename)
+				 gboolean is_filename,
+				 gboolean silent)
 {
     GString *result;
     gchar *res_utf8;
@@ -1081,8 +1083,11 @@ gchar *get_string_from_template (Track *track,
 		tmp = "%";
 		break;
 	    default:
-		gtkpod_warning (_("Unknown token '%%%c' in template '%s'"),
-				*p, template);
+		if (!silent)
+		{
+		    gtkpod_warning (_("Unknown token '%%%c' in template '%s'"),
+				    *p, template);
+		}
 		break;
 	    }
 	    if (tmp)
@@ -1181,7 +1186,7 @@ gchar *get_string_from_full_template (Track *track,
 	}
     }
 
-    res_utf8 = get_string_from_template (track, template, is_filename);
+    res_utf8 = get_string_from_template (track, template, is_filename, FALSE);
 
     g_free (template);
 
@@ -1219,9 +1224,12 @@ gchar *which (const gchar *exe)
 
 /**
  * Recursively make directories.
+ *
+ * @silent: don't print error messages via gtk (->thread safe)
+ *
  * @return FALSE is this is not possible.
  */
-gboolean mkdirhier(const gchar *dirname)
+gboolean mkdirhier(const gchar *dirname, gboolean silent)
 {
     gchar *dn, *p;
 
@@ -1242,10 +1250,13 @@ gboolean mkdirhier(const gchar *dirname)
 
 	if (!g_file_test(dn, G_FILE_TEST_EXISTS))
 	{
-	    if (mkdir(dn, 0777) == -1)
+	    if (g_mkdir(dn, 0777) == -1)
 	    {
-		gtkpod_warning (_("Error creating %s: %s\n"),
-				dn, g_strerror(errno));
+		if (!silent)
+		{
+		    gtkpod_warning (_("Error creating %s: %s\n"),
+				    dn, g_strerror(errno));
+		}
 		g_free (dn);
 		return FALSE;
 	    }
@@ -1265,7 +1276,7 @@ gboolean mkdirhierfile(const gchar *filename)
 {
     gboolean result;
     gchar *dirname = g_path_get_dirname (filename);
-    result = mkdirhier (dirname);
+    result = mkdirhier (dirname, FALSE);
     g_free (dirname);
     return result;
 }
@@ -1290,6 +1301,51 @@ gchar *convert_filename (const gchar *filename)
 }
 
 
+/**
+ * get_size_of_directory
+ *
+ * Determine the total size in bytes of all files in @dir including
+ * subdirectories. This function ignores errors in the sense that if a
+ * directory or file cannot be accessed, a size of 0 is assumed.
+ */
+gint64 get_size_of_directory (const gchar *dir)
+{
+    GDir *gdir;
+    const gchar *fname;
+    gint64 tsize = 0;
+
+    g_return_val_if_fail (dir, 0);
+
+    gdir = g_dir_open (dir, 0, NULL);
+
+    /* Check for error */
+    if (!gdir)
+	return 0;
+
+    while ((fname = g_dir_read_name (gdir)))
+    {
+	gchar *fullname = g_build_filename (dir, fname, NULL);
+	if (g_file_test (fullname, G_FILE_TEST_IS_DIR))
+	{
+	    tsize += get_size_of_directory (fullname);
+	}
+	else if (g_file_test (fullname, G_FILE_TEST_IS_REGULAR))
+	{
+	    struct stat statbuf;
+	    if (g_stat (fullname, &statbuf) == 0)
+	    {   /* OK, add size */
+		tsize += statbuf.st_size;
+	    }
+	}
+	g_free (fullname);
+    }
+
+    g_dir_close (gdir);
+
+    return tsize;
+}
+
+
 
 /**
  * Wrapper for gtkpod_xml_get_widget() giving out a warning if widget
@@ -1308,7 +1364,6 @@ GtkWidget *gtkpod_xml_get_widget (GladeXML *xml, const gchar *name)
 }
 
 
-  gchar    *ipod_mount;     /* mount point of iPod */
 /* ------------------------------------------------------------
  *
  *        Helper functions for pref keys
@@ -1555,14 +1610,169 @@ void set_itdb_index_prefs_int (gint index,
     g_free (key);
 }
 
-/* Sorting gets disabled if either tmp disable sort or
- * block display are true */
-gboolean sorting_disabled()
+/**
+ * Helper function to remove all prefs strings for itdb @index and
+ * move all subsequent itdb strings forward. You can call this even
+ * after your have removed the itdb from itdbs_head.
+ * 
+ **/
+static void remove_itdb_index_prefs (gint index)
 {
-    return (prefs_get_int("block_display") || 
-	    prefs_get_int("tmp_disable_sort"));
+    struct itdbs_head *itdbs_head;
+    gchar *subkey;
+    gint i, n;
+
+    itdbs_head = gp_get_itdbs_head (gtkpod_window);
+    g_return_if_fail (itdbs_head);
+
+    n = g_list_length (itdbs_head->itdbs);
+    subkey = get_itdb_prefs_key (index, "");
+    prefs_flush_subkey (subkey);
+    g_free (subkey);
+    
+    for (i=index; i<=n; ++i)
+    {
+	gchar *from_key = get_itdb_prefs_key (i+1, "");
+	gchar *to_key = get_itdb_prefs_key (i, "");
+	prefs_rename_subkey (from_key, to_key);
+	g_free (from_key);
+	g_free (to_key);
+    }
 }
 
+
+/**
+ * Helper function to remove all prefs strings for @itdb and move all
+ * subsequent itdb strings forward. Call this before removing the itdb
+ * from itdbs_head.
+ * 
+ **/
+void remove_itdb_prefs (iTunesDB *itdb)
+{
+    g_return_if_fail (itdb);
+
+    remove_itdb_index_prefs (get_itdb_index (itdb));
+}
+
+/* Save all itdb_<index>_* keys to the iPod
+ * (<ControlDir>/gtkpod.prefs).
+ *
+ * Return value: TRUE on succes, FALSE on error
+ */
+gboolean save_ipod_index_prefs (gint index, const gchar *mountpoint)
+{
+    TempPrefs *temp_prefs;
+    gboolean result = FALSE;
+    gchar *subkey, *dir;
+
+    g_return_val_if_fail (mountpoint, FALSE);
+
+    /* isolate all 'itdb_<index>_*' keys */
+    subkey = get_itdb_prefs_key (index, "");
+    temp_prefs = prefs_create_subset (subkey);
+
+    /* rename to 'itdb_*' */
+    temp_prefs_rename_subkey (temp_prefs, subkey, "itdb_");
+
+    /* remove some keys */
+    temp_prefs_remove_key (temp_prefs, "itdb_mountpoint");
+    temp_prefs_remove_key (temp_prefs, "itdb_name");
+    temp_prefs_remove_key (temp_prefs, "itdb_type");
+
+    /* build filename path */
+    dir = itdb_get_itunes_dir (mountpoint);
+    if (dir)
+    {
+	GError *error = NULL;
+	gchar *path = g_build_filename (dir, "gtkpod.prefs", NULL);
+	result = temp_prefs_save (temp_prefs, path, &error);
+	if (result == FALSE)
+	{
+	    gtkpod_warning (_("Writing preferences file '%s' failed (%s).\n\n"),
+			    path,
+			    error? error->message:_("unspecified error"));
+	    g_error_free (error);
+	}
+	g_free (path);
+	g_free (dir);
+    }
+    else
+    {
+	gtkpod_warning (_("Writing preferences to the iPod (%s) failed: could not get path to Control Directory.\n\n"),
+			mountpoint);
+    }
+
+    temp_prefs_destroy (temp_prefs);
+    g_free (subkey);
+
+    return result;
+}
+
+
+
+/* Save all itdb_<index>_* keys to the iPod
+ * (<ControlDir>/gtkpod.prefs).
+ *
+ * Return value: TRUE on succes, FALSE on error
+ */
+gboolean save_ipod_prefs (iTunesDB *itdb, const gchar *mountpoint)
+{
+    g_return_val_if_fail (itdb && mountpoint, FALSE);
+    return save_ipod_index_prefs (get_itdb_index (itdb), mountpoint);
+}
+
+
+/* Load preferences file from the iPod and merge them into the general
+ * prefs system.
+ */
+static void load_ipod_index_prefs (gint index, const gchar *mountpoint)
+{
+    gchar *dir;
+
+    g_return_if_fail (mountpoint);
+
+    /* build filename path */
+    dir = itdb_get_itunes_dir (mountpoint);
+    if (dir)
+    {
+	TempPrefs *temp_prefs;
+	GError *error = NULL;
+	gchar *path = g_build_filename (dir, "gtkpod.prefs", NULL);
+	temp_prefs = temp_prefs_load (path, &error);
+	if (temp_prefs)
+	{
+	    gchar *subkey;
+	    subkey = get_itdb_prefs_key (index, "");
+	    /* rename 'itdb_*' to 'itdb_<index>_*' */
+	    temp_prefs_rename_subkey (temp_prefs, "itdb_", subkey);
+	    /* merge with real prefs */
+	    temp_prefs_apply (temp_prefs);
+	    /* destroy temp prefs */
+	    temp_prefs_destroy (temp_prefs);
+	}
+	else
+	{
+	    /* we ignore errors -- no need to be concerned about them */
+	    g_error_free (error);
+	}
+	g_free (dir);
+    }
+}
+
+
+
+/* Load preferences file from the iPod and merge them into the general
+ * prefs system.
+ */
+void load_ipod_prefs (iTunesDB *itdb, const gchar *mountpoint)
+{
+    g_return_if_fail (mountpoint);
+    load_ipod_index_prefs (get_itdb_index (itdb), mountpoint);
+}
+
+
+
+
 /* retrieve offline mode from itdb (convenience function) */
 gboolean get_offline (iTunesDB *itdb)
 {
@@ -1574,3 +1784,143 @@ gboolean get_offline (iTunesDB *itdb)
 
     return eitdb->offline;
 }
+
+/* ----------------------------------------------------------------
+ *
+ * Main program init and shutdown
+ *
+ * ---------------------------------------------------------------- */
+
+/**
+ * gtkpod_init
+ *
+ * initialize prefs and other services as well as display
+ */
+void gtkpod_init (int argc, char *argv[])
+{
+    gchar *progname;
+
+    /* initialize xml_file: if gtkpod is called in the build directory
+       (".../src/gtkpod") use the local gtkpod.glade (in the data
+       directory), otherwise use
+       "PACKAGE_DATA_DIR/PACKAGE/data/gtkpod.glade" */
+
+    progname = g_find_program_in_path (argv[0]);
+    if (progname)
+    {
+	static const gchar *SEPsrcSEPgtkpod = G_DIR_SEPARATOR_S "src" G_DIR_SEPARATOR_S "gtkpod";
+
+	if (!g_path_is_absolute (progname))
+	{
+	    gchar *cur_dir = g_get_current_dir ();
+	    gchar *prog_absolute;
+
+	    if (g_str_has_prefix (progname, "." G_DIR_SEPARATOR_S))
+		prog_absolute = g_build_filename (cur_dir,progname+2,NULL);
+	    else
+		prog_absolute = g_build_filename (cur_dir,progname,NULL);
+	    g_free (progname);
+	    g_free (cur_dir);
+	    progname = prog_absolute;
+	}
+
+	if (g_str_has_suffix (progname, SEPsrcSEPgtkpod))
+	{
+	    gchar *suffix = g_strrstr (progname, SEPsrcSEPgtkpod);
+	    if (suffix)
+	    {
+		*suffix = 0;
+		xml_file = g_build_filename (progname, "data", "gtkpod.glade", NULL);
+	    }
+	}
+	g_free (progname);
+	if (xml_file && !g_file_test (xml_file, G_FILE_TEST_EXISTS))
+	{
+	    g_free (xml_file);
+	    xml_file = NULL;
+	}
+    }
+    if (!xml_file)
+	xml_file = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, "data", "gtkpod.glade", NULL);
+    else
+    {
+	printf ("Using local gtkpod.glade file since program was started from source directory:\n%s\n", xml_file);
+    }
+
+    /* Initialisation of libxml */
+    LIBXML_TEST_VERSION;
+
+    main_window_xml = glade_xml_new (xml_file, "gtkpod", NULL);
+
+    glade_xml_signal_autoconnect (main_window_xml);
+  
+    gtkpod_window = gtkpod_xml_get_widget (main_window_xml, "gtkpod");
+
+    prefs_init (argc, argv); 
+
+    coverart_init (argv[0]);
+    stockid_init (argv[0]);
+
+    file_convert_init ();
+
+    display_create ();
+
+    gtk_widget_show (gtkpod_window);
+
+    init_data (gtkpod_window);   /* setup base data, importing all local
+				  * repositories */
+
+    /* stuff to be done before starting gtkpod */
+    call_script ("gtkpod.in", NULL);
+
+    autodetection_init ();
+	
+    server_setup ();   /* start server to accept playcount updates */
+}
+
+
+
+/**
+ * gtkpod_shutdown
+ *
+ * free memory, shutdown services and call gtk_main_quit ()
+ */
+void gtkpod_shutdown ()
+{
+    /* stop accepting requests for playcount updates */
+    server_shutdown ();
+
+		/* Change the windows back to track view to ensure the
+		 * sorttab state is saved correctly/
+		 */
+		gphoto_change_to_photo_window (FALSE);
+		
+    /* Sort column order needs to be stored */
+    tm_store_col_order();
+  
+    /* Update default sizes */
+    display_update_default_sizes();
+
+    /* shut down conversion infrastructure */
+    file_convert_shutdown ();
+
+    /* Save prefs */
+    prefs_save ();
+
+/* FIXME: release memory in a clean way */
+#if 0
+    remove_all_playlists ();  /* first remove playlists, then tracks!
+			       * (otherwise non-existing *tracks may
+			       * be accessed) */
+    remove_all_tracks ();
+#endif
+    display_cleanup ();
+
+    prefs_shutdown ();
+
+    xmlCleanupParser();
+    xmlMemoryDump();
+	
+    call_script ("gtkpod.out", NULL);
+    gtk_main_quit ();
+}
