@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-06-25 00:25:35 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc.h,v 1.119 2006/06/24 15:39:22 jcsjcs Exp $
+|  $Id: misc.h 1814 2007-12-11 12:53:22Z jcsjcs $
 */
 
 #ifndef __MISC_H__
@@ -38,14 +37,20 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <stdio.h>
+#include <libxml/xmlversion.h>
+#include <libxml/xmlmemory.h>
 #include "file.h"
 #include "display_itdb.h"
 #include "display.h"
 #include "confirmation.h"
 #include "time.h"
 
+
 #define C_FREE(a) {g_free(a); a=NULL;}
 
+/* where to find the scripts */
+extern const gchar *SCRIPTDIR;
+
 /* version of GTK_CHECK_VERSION that uses the runtime variables
    gtk_*_version... instead of the compile-time constants
    GTK_*_VERSION */
@@ -101,11 +106,14 @@ gchar *concat_dir_if_relative (G_CONST_RETURN gchar *base_dir,
 float get_ms_since (GTimeVal *old_time, gboolean update);
 gint get_sort_tab_number (gchar *text);
 void open_about_window (void);
-void close_about_window (void);
-gboolean parse_tracks_from_string(gchar **s, Track **track);
-gboolean gtkpod_main_quit(void);
+gboolean parse_tracks_from_string (gchar **s, Track **track);
+gboolean parse_artwork_from_string(gchar **s, Artwork **artwork);
+void gtkpod_init (int argc, char *argv[]);
+void gtkpod_shutdown (void);
 
+gchar *get_allowed_percent_char (void);
 T_item TM_to_T (TM_item sm);
+T_item char_to_T (char c);
 T_item ST_to_T (ST_CAT_item st);
 const gchar *get_tm_string (TM_item tm);
 const gchar *get_tm_tooltip (TM_item tm);
@@ -130,6 +138,8 @@ void update_blocked_widget (GtkWidget *w, gboolean sens);
   void unmount_ipod(void);*/
 void call_script (gchar *script, ...);
 
+gchar **build_argv_from_strings (const gchar *first_arg, ...);
+
 void delete_track_ok (struct DeleteData *dd);
 
 void gtkpod_warning (const gchar *format, ...);
@@ -179,8 +189,9 @@ gunichar2 *utf16_strdup (gunichar2 *utf16);
 
 void check_db (iTunesDB *db);
 
-gboolean mkdirhier(const gchar *dirname);
+gboolean mkdirhier(const gchar *dirname, gboolean silent);
 gboolean mkdirhierfile(const gchar *filename);
+gint64 get_size_of_directory (const gchar *dir);
 gchar *convert_filename (const gchar *filename);
 
 guint32 replaygain_to_soundcheck (gdouble gain);
@@ -218,7 +229,8 @@ gboolean option_get_toggle_button (GladeXML *win_xml,
 
 gchar *get_string_from_template (Track *track,
 				 const gchar *template,
-				 gboolean is_filename);
+				 gboolean is_filename,
+				 gboolean silent);
 gchar *get_string_from_full_template (Track *track,
 				      const gchar *full_template,
 				      gboolean is_filename);
@@ -247,6 +259,9 @@ void set_itdb_index_prefs_string (gint index,
 void set_itdb_prefs_int (iTunesDB *itdb, const gchar *subkey, gint value);
 void set_itdb_index_prefs_int (gint index,
 			       const gchar *subkey, gint value);
+void remove_itdb_prefs (iTunesDB *itdb);
+void load_ipod_prefs (iTunesDB *itdb, const gchar *mountpoint);
+gboolean save_ipod_prefs (iTunesDB *itdb, const gchar *mountpoint);
+
 gboolean get_offline (iTunesDB *itdb);
-gboolean sorting_disabled();
 #endif
