@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-09-24 20:49:39 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_confirm.c,v 1.44 2006/09/24 12:16:26 jcsjcs Exp $
+|  $Id: misc_confirm.c 1822 2007-12-12 16:31:26Z dforsi $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -34,11 +33,11 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include <sys/stat.h>
-#include "clientserver.h"
 #include "confirmation.h"
 #include "misc.h"
 #include "prefs.h"
 #include "info.h"
+#include "display_coverart.h"
 
 #ifdef HAVE_STATVFS
 #include <sys/types.h>
@@ -69,7 +68,7 @@ void gtkpod_warning (const gchar *format, ...)
     gtkpod_confirmation (CONF_ID_GTKPOD_WARNING,    /* gint id, */
 			 FALSE,                     /* gboolean modal, */
 			 _("Warning"),              /* title */
-			 _("The following has occured:"),
+			 _("The following has occurred:"),
 			 text,                /* text to be displayed */
 			 NULL, 0, NULL,       /* option 1 */
 			 NULL, 0, NULL,       /* option 2 */
@@ -125,7 +124,7 @@ void delete_populate_settings (struct DeleteData *dd,
 	    if (title)
 		*title = g_strdup (
 		    ngettext ("Delete Track Completely from iPod?",
-			      "Delete Tracks Completey from iPod?", n));
+			      "Delete Tracks Completely from iPod?", n));
 	    if (confirm_again)
 		*confirm_again = prefs_get_int("delete_ipod");
 	    if (confirm_again_key)
@@ -248,6 +247,9 @@ void delete_track_ok (struct DeleteData *dd)
     /* should never happen */
     if (!dd->tracks)	delete_track_cancel (dd);
 
+		/* Deafen the coverart display while deletion is occurring */
+		coverart_block_change (TRUE);
+		
     /* nr of tracks to be deleted */
     n = g_list_length (dd->tracks);
     if (dd->itdb->usertype & GP_ITDB_TYPE_IPOD)
@@ -308,6 +310,8 @@ void delete_track_ok (struct DeleteData *dd)
 	gp_playlist_remove_track (dd->pl, l->data, dd->deleteaction);
     }
 
+		/* Awaken coverart selection */
+		coverart_block_change (FALSE);
     g_list_free (dd->tracks);
     g_free (dd);
 
@@ -537,7 +541,7 @@ static void delete_playlist_ok (struct DeleteData *dd)
 		msg = g_strdup_printf (_("Removed all podcasts from the iPod"));
 		if (pm_get_selected_playlist () == dd->pl)
 		    st_redisplay (0);
-//		display_reset (0);
+/*		display_reset (0);*/
 	    }
 	    else
 	    {
@@ -701,8 +705,8 @@ void delete_playlist_head (DeleteAction deleteaction)
 		 * delete_populate_settings() further down */
 		dd->tracks = g_list_copy (pl->members);
 		label = g_strdup_printf (
-		    ngettext ("Are you sure you want to delete playlist '%s' and the following track completely from your ipod? The number of playlists this track is a member of is indicated in parentheses.",
-			      "Are you sure you want to delete playlist '%s' and the following tracks completely from your ipod? The number of playlists the tracks are member of is indicated in parentheses.", n), pl->name);
+		    ngettext ("Are you sure you want to delete playlist '%s' and the following track completely from your iPod? The number of playlists this track is a member of is indicated in parentheses.",
+			      "Are you sure you want to delete playlist '%s' and the following tracks completely from your iPod? The number of playlists the tracks are member of is indicated in parentheses.", n), pl->name);
 	    }
 	    break;
 	case DELETE_ACTION_PLAYLIST:
@@ -817,35 +821,13 @@ void delete_playlist_head (DeleteAction deleteaction)
 
 
 
-/* callback for quit menu entry */
-void
-on_quit1_activate                      (GtkMenuItem     *menuitem,
-					gpointer         user_data)
-{
-  if (!widgets_blocked) gtkpod_main_quit ();
-}
-
-
-/* callback for gtkpod window's close button */
-gboolean
-on_gtkpod_delete_event                 (GtkWidget       *widget,
-					GdkEvent        *event,
-					gpointer         user_data)
-{
-    if (!widgets_blocked)
-    {
-	return gtkpod_main_quit ();
-    }
-    return TRUE; /* don't quit -- would cause numerous error messages */
-}
-
 /**
- * gtkpod_main_quit
+ * gtkpod_shutdown
  *
- * return value: FALSE if it's OK to quit.
+ * return value: TRUE if it's OK to quit.
  */
-gboolean
-gtkpod_main_quit(void)
+static gboolean
+ok_to_close_gtkpod (void)
 {
     gint result = GTK_RESPONSE_YES;
 
@@ -863,22 +845,40 @@ gtkpod_main_quit(void)
 
     if (result == GTK_RESPONSE_YES)
     {
-	server_shutdown (); /* stop accepting requests for playcount updates */
+	return TRUE;
+    }
+    return FALSE;
+}
 
-	cleanup_prefs();
 
-/* FIXME: release memory in a clean way */
-#if 0
-	remove_all_playlists ();  /* first remove playlists, then
-				   * tracks! (otherwise non-existing
-				   *tracks may be accessed) */
-	remove_all_tracks ();
-#endif
-	display_cleanup ();
+/* callback for gtkpod window's close button */
+gboolean
+on_gtkpod_delete_event                 (GtkWidget       *widget,
+					GdkEvent        *event,
+					gpointer         user_data)
+{
+    if (!widgets_blocked)
+    {
+	if (ok_to_close_gtkpod ())
+	{
+	    gtkpod_shutdown ();
+	    return FALSE;
+	}
+    }
+    return TRUE; /* don't quit -- would cause numerous error messages */
+}
 
-	call_script ("gtkpod.out", NULL);
-	gtk_main_quit ();
-	return FALSE;
+
+/* callback for quit menu entry */
+void
+on_quit1_activate                      (GtkMenuItem     *menuitem,
+					gpointer         user_data)
+{
+    if (!widgets_blocked)
+    {
+	if (ok_to_close_gtkpod ())
+	{
+	    gtkpod_shutdown ();
+	}
     }
-    return TRUE;
 }
