@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-09-21 23:16:41 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_conversion.c,v 1.28 2006/09/21 15:03:13 jcsjcs Exp $
+|  $Id: misc_conversion.c 1655 2007-07-27 02:22:31Z tiffman $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -98,6 +97,20 @@ static const gchar *t_strings[] = {
     N_("Remember Playback Position"),
     N_("Skip when Shuffling"),
     N_("Artwork Path"),      /* 40 */
+    N_("Media Type"),
+    N_("TV Show"),
+    N_("TV Episode"),
+    N_("TV Network"),
+    N_("Season Nr"),         /* 45 */
+    N_("Episode Nr"),
+    N_("Album Artist"),
+    N_("Sort Artist"),
+    N_("Sort Title"),
+    N_("Sort Album"),        /* 50 */
+    N_("Sort Album Artist"),
+    N_("Sort Composer"),
+    N_("Sort TV Show"),
+    N_("Gapless Track Flag"),
     NULL };
 
 /* Tooltips for prefs window */
@@ -120,8 +133,7 @@ static const gchar *t_tooltips[] = {
     NULL,
     NULL,                                              /* 15 */
     NULL,
-    N_("Supposedly something that tells the iPod to "
-       "increase or decrease the playback speed"),
+    N_("Beats per minute"),
     N_("Number of times the track has been played"),
     N_("Star rating from 0 to 5"),
     N_("Date and time track has been added"),          /* 20 */
@@ -145,6 +157,20 @@ static const gchar *t_tooltips[] = {
     NULL,
     NULL,
     NULL,
+    NULL,   /* 40 */
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,   /* 45 */
+    NULL,
+    NULL,
+    N_("Used for sorting on the iPod"),
+    N_("Used for sorting on the iPod"),
+    N_("Used for sorting on the iPod"),  /* 50 */
+    N_("Used for sorting on the iPod"),
+    N_("Used for sorting on the iPod"),
+    N_("Used for sorting on the iPod"),
     NULL
  };
 
@@ -191,12 +217,72 @@ T_item TM_to_T (TM_item tm)
     case TM_COLUMN_SUBTITLE:      return T_SUBTITLE;
     case TM_COLUMN_TIME_RELEASED: return T_TIME_RELEASED;
     case TM_COLUMN_THUMB_PATH:    return T_THUMB_PATH;
+    case TM_COLUMN_MEDIA_TYPE:    return T_MEDIA_TYPE;
+    case TM_COLUMN_TV_SHOW:       return T_TV_SHOW;
+    case TM_COLUMN_TV_EPISODE:    return T_TV_EPISODE;
+    case TM_COLUMN_TV_NETWORK:    return T_TV_NETWORK;
+    case TM_COLUMN_SEASON_NR:     return T_SEASON_NR;
+    case TM_COLUMN_EPISODE_NR:    return T_EPISODE_NR;
+    case TM_COLUMN_ALBUMARTIST:   return T_ALBUMARTIST;
+    case TM_COLUMN_SORT_ARTIST:   return T_SORT_ARTIST;
+    case TM_COLUMN_SORT_TITLE:    return T_SORT_TITLE;
+    case TM_COLUMN_SORT_ALBUM:    return T_SORT_ALBUM;
+    case TM_COLUMN_SORT_ALBUMARTIST: return T_SORT_ALBUMARTIST;
+    case TM_COLUMN_SORT_COMPOSER: return T_SORT_COMPOSER;
+    case TM_COLUMN_SORT_TVSHOW:   return T_SORT_TVSHOW;
     case TM_NUM_COLUMNS:          g_return_val_if_reached (-1);
     }
     return -1;
 }
 
 
+/* See track_get_item() / track_get_item_pointer()  */
+gchar *get_allowed_percent_char(void )
+{
+    return g_strdup("latgcoysSny");
+}
+/* translates a char into a T_... (defined in display.h) */
+T_item char_to_T(gchar c){
+    switch (c) {
+        case 'l': return T_ALBUM;
+        case 'a': return T_ARTIST;
+        case 't': return T_TITLE;
+        case 'g': return T_GENRE;
+        case 'c': return T_COMMENT;
+        case 'o': return T_COMPOSER;
+        case 'f': return T_FILETYPE;
+        case 's': return T_PC_PATH;
+        case 'S': return T_IPOD_PATH;
+	    /* case 'i': return T_IPOD_ID; */
+        case 'n': return T_TRACK_NR;
+	    /* case 'f': return T_TRANSFERRED; */
+	    /* case 'z': return T_SIZE; */
+	    /* case 'L': return T_TRACKLEN; */
+	    /* case 'b': return T_BITRATE; */
+	    /* case 'r': return T_SAMPLERATE; */
+	    /* case 'b': return T_BPM; */
+	    /* case 'C': return T_PLAYCOUNT; */
+	    /* case 'i': return T_RATING; */
+	    /* case '': return T_TIME_ADDED; */
+	    /* case '': return T_TIME_PLAYED; */
+	    /* case '': return T_TIME_MODIFIED; */
+	    /* case '': return T_VOLUME; */
+	    /* case '': return T_SOUNDCHECK; */
+        case 'y': return T_YEAR;
+	    /* case 'd': return T_CD_NR; */
+	    /* case '': return T_GROUPING; */
+	    /* case '': return T_COMPILATION; */
+	    /* case '': return T_CATEGORY; */
+	    /* case '': return T_DESCRIPTION; */
+	    /* case '': return T_PODCASTURL; */
+	    /* case '': return T_PODCASTRSS; */
+	    /* case '': return T_SUBTITLE; */
+	    /* case '': return T_TIME_RELEASED; */
+	    /* case '': return T_CHECKED; */
+    }
+    return -1;
+}
+
 /* translates a ST_CAT_... (defined in display.h) into a
  * T_... (defined in display.h). Returns -1 in case a translation is not
  * possible */
@@ -417,7 +503,7 @@ time_t time_get_time (Track *track, T_item t_item)
 	mactime = 0;
 	break;
     }
-    return (itdb_time_mac_to_host (mactime));
+    return mactime;
 }
 
 
@@ -429,23 +515,23 @@ gchar *time_field_to_string (Track *track, T_item t_item)
 
 
 /* get the timestamp TM_COLUMN_TIME_CREATE/PLAYED/MODIFIED/RELEASED */
-void time_set_time (Track *track, time_t time, T_item t_item)
+void time_set_time (Track *track, time_t timet, T_item t_item)
 {
-    guint32 mactime = itdb_time_host_to_mac (time);
+    g_return_if_fail (track);
 
-    if (track) switch (t_item)
+    switch (t_item)
     {
     case T_TIME_ADDED:
-	track->time_added = mactime;
+	track->time_added = timet;
 	break;
     case T_TIME_PLAYED:
-	track->time_played = mactime;
+	track->time_played = timet;
 	break;
     case T_TIME_MODIFIED:
-	track->time_modified = mactime;
+	track->time_modified = timet;
 	break;
     case T_TIME_RELEASED:
-	track->time_released = mactime;
+	track->time_released = timet;
 	break;
     default:
 	break;
