@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-09-24 15:25:29 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_track.c,v 1.52 2006/09/24 06:42:52 jcsjcs Exp $
+|  $Id: misc_track.c 1686 2007-08-25 03:20:11Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +31,7 @@
 #endif
 
 #include "display_itdb.h"
-#include "md5.h"
+#include "sha1.h"
 #include "prefs.h"
 #include "misc.h"
 #include "misc_track.h"
@@ -45,35 +44,35 @@
 
 /* ------------------------------------------------------------ *\
 |                                                                |
-|         functions for md5 checksums                            |
+|         functions for sha1 checksums                            |
 |                                                                |
 \* ------------------------------------------------------------ */
 
 /**
- * Register all tracks in the md5 hash and remove duplicates (while
+ * Register all tracks in the sha1 hash and remove duplicates (while
  * preserving playlists)
  */
-void gp_md5_hash_tracks_itdb (iTunesDB *itdb)
+void gp_sha1_hash_tracks_itdb (iTunesDB *itdb)
 {
    gint ns, count;
    GList *gl;
 
    g_return_if_fail (itdb);
 
-   if (!prefs_get_int("md5")) return;
+   if (!prefs_get_int("sha1")) return;
    ns = itdb_tracks_number (itdb);   /* number of tracks */
    if (ns == 0)                 return;
 
    block_widgets (); /* block widgets -- this might take a while,
 			so we'll do refreshs */
-   md5_free (itdb);  /* release md5 hash */
+   sha1_free (itdb);  /* release sha1 hash */
    count = 0;
    /* populate the hash table */
    gl=itdb->tracks;
    while (gl)
    {
        Track *track=gl->data;
-       Track *oldtrack = md5_track_exists_insert (itdb, track);
+       Track *oldtrack = sha1_track_exists_insert (itdb, track);
 
        /* need to get next track now because it might be a duplicate and
 	  thus be removed when we call gp_duplicate_remove() */
@@ -103,7 +102,7 @@ void gp_md5_hash_tracks_itdb (iTunesDB *itdb)
  * Call gp_hash_tracks_itdb() for each itdb.
  *
  */
-void gp_md5_hash_tracks (void)
+void gp_sha1_hash_tracks (void)
 {
     GList *gl;
     struct itdbs_head *itdbs_head;
@@ -116,25 +115,25 @@ void gp_md5_hash_tracks (void)
     block_widgets ();
     for (gl=itdbs_head->itdbs; gl; gl=gl->next)
     {
-	gp_md5_hash_tracks_itdb (gl->data);
+	gp_sha1_hash_tracks_itdb (gl->data);
     }
     release_widgets ();
 }
 
 
 /**
- * Call md5_free() for each itdb and delete md5 checksums in all tracks.
+ * Call sha1_free() for each itdb and delete sha1 checksums in all tracks.
  *
  */
-void gp_md5_free_hash (void)
+void gp_sha1_free_hash (void)
 {
-    void rm_md5 (gpointer track, gpointer user_data)
+    void rm_sha1 (gpointer track, gpointer user_data)
 	{
 	    ExtraTrackData *etr;
 	    g_return_if_fail (track);
 	    etr = ((Track *)track)->userdata;
 	    g_return_if_fail (etr);
-	    C_FREE (etr->md5_hash);
+	    C_FREE (etr->sha1_hash);
 	}
     GList *gl;
     struct itdbs_head *itdbs_head;
@@ -148,8 +147,8 @@ void gp_md5_free_hash (void)
     {
 	iTunesDB *itdb = gl->data;
 	g_return_if_fail (itdb);
-	md5_free (itdb);
-	g_list_foreach (itdb->tracks, rm_md5, NULL);
+	sha1_free (itdb);
+	g_list_foreach (itdb->tracks, rm_sha1, NULL);
     }
 }
 
@@ -157,7 +156,7 @@ void gp_md5_free_hash (void)
 /* This function removes a duplicate track "track" from memory while
  * preserving the playlists.
  *
- * The md5 hash is not modified.
+ * The sha1 hash is not modified.
  *
  * The playcount/recent_playcount are modified to show the cumulative
  * playcounts for that track.
@@ -281,13 +280,18 @@ void gp_duplicate_remove (Track *oldtrack, Track *track)
        oldtrack->time_added =  MIN (oldtrack->time_added, track->time_added);
 
        /* Update filename if new track has filename set (should be
-	* always!?) */
+	  always!?) and old filename is not available or no longer
+	  valid */
        if (etr->pc_path_locale)
        {
-	   g_free (oldetr->pc_path_locale);
-	   g_free (oldetr->pc_path_utf8);
-	   oldetr->pc_path_locale = g_strdup (etr->pc_path_locale);
-	   oldetr->pc_path_utf8 = g_strdup (etr->pc_path_utf8);
+	   if (!oldetr->pc_path_locale ||
+	       !g_file_test (oldetr->pc_path_locale, G_FILE_TEST_IS_REGULAR))
+	   {
+	       g_free (oldetr->pc_path_locale);
+	       g_free (oldetr->pc_path_utf8);
+	       oldetr->pc_path_locale = g_strdup (etr->pc_path_locale);
+	       oldetr->pc_path_utf8 = g_strdup (etr->pc_path_utf8);
+	   }
        }
        if (itdb_playlist_contains_track (itdb_playlist_mpl (itdb),
 					 track))
@@ -301,7 +305,7 @@ void gp_duplicate_remove (Track *oldtrack, Track *track)
 	       g_return_if_fail (pl);
 	       /* if "track" is in playlist pl, we remove it and add
 		  the "oldtrack" instead (this way round we don't have
-		  to worry about changing md5 hash entries */
+		  to worry about changing sha1 hash entries */
 	       if (itdb_playlist_contains_track (pl, track))
 	       {
 		   gp_playlist_remove_track (pl, track,
@@ -331,41 +335,32 @@ void gp_duplicate_remove (Track *oldtrack, Track *track)
 
 
 /**
- * Register all tracks in the md5 hash and remove duplicates (while
+ * Register all tracks in the sha1 hash and remove duplicates (while
  * preserving playlists).
  * Call  gp_duplicate_remove (NULL, NULL); to show an info dialogue
  */
 void gp_itdb_hash (iTunesDB *itdb)
 {
-   gint ns, count, track_nr;
+   gint ns, track_nr;
    Track *track, *oldtrack;
 
    g_return_if_fail (itdb);
 
-   if (!prefs_get_int("md5")) return;
+
+   if (!prefs_get_int("sha1")) return;
 
    ns = itdb_tracks_number (itdb);
    if (ns == 0)                 return;
 
    block_widgets (); /* block widgets -- this might take a while,
 			so we'll do refreshs */
-   md5_free (itdb);  /* release md5 hash */
-   count = 0;
+   sha1_free (itdb);  /* release sha1 hash */
    track_nr = 0;
    /* populate the hash table */
    while ((track = g_list_nth_data (itdb->tracks, track_nr)))
    {
-       oldtrack = md5_track_exists_insert (itdb, track);
-       ++count;
+       oldtrack = sha1_track_exists_insert (itdb, track);
 /*        printf("%d:%d:%p:%p\n", count, track_nr, track, oldtrack); */
-       if (!prefs_get_int("block_display") &&
-	   (((count % 20) == 1) || (count == ns)))
-       { /* update for count == 1, 21, 41 ... and for count == n */
-	   gtkpod_statusbar_message (ngettext ("Hashed %d of %d track.",
-					       "Hashed %d of %d tracks.", ns),
-				     count, ns);
-	   while (widgets_blocked && gtk_events_pending ())  gtk_main_iteration ();
-       }
        if (oldtrack)
        {
 	   gp_duplicate_remove (oldtrack, track);
@@ -390,7 +385,7 @@ void gp_itdb_hash (iTunesDB *itdb)
 
 
 /* Returns the track with the filename @name or NULL, if none can be
- * found. This function also works if @name is on the iPod. */
+ * found. This function also works if @filename is on the iPod. */
 Track *gp_track_by_filename (iTunesDB *itdb, gchar *filename)
 {
     gchar *musicdir = NULL;
@@ -436,7 +431,7 @@ Track *gp_track_by_filename (iTunesDB *itdb, gchar *filename)
     else
     {   /* handle track on local filesystem */
 	GList *gl;
-	for (gl=itdb->tracks; gl; gl=gl->next)
+	for (gl=itdb->tracks; gl&&!result; gl=gl->next)
 	{
 	    Track *track = gl->data;
 	    ExtraTrackData *etr;
@@ -455,6 +450,337 @@ Track *gp_track_by_filename (iTunesDB *itdb, gchar *filename)
 }
 
 
+/* Find @track in repository @itdb by the following methods and return
+   the matches:
+
+   1) DND origin data
+   2) filename matches
+   3) SHA1 match 
+
+   If DND origin data is available and valid only one track is
+   returned. Otherwise all tracks matching the filename and the SHA1
+   are returned.
+
+   If DND origin data is found to be invalid it is deleted.
+
+   Return value: a GList with matching tracks. You must call
+   g_list_free() on the list when it is no longer needed.
+*/
+GList *gp_itdb_find_same_tracks (iTunesDB *itdb, Track *track)
+{
+    ExtraTrackData *etr;
+    Track *itr;
+    GList *tracks = NULL;
+
+    g_return_val_if_fail (itdb, NULL);
+    g_return_val_if_fail (track, NULL);
+
+    etr = track->userdata;
+    g_return_val_if_fail (etr, NULL);
+
+    if (itdb->id == etr->local_itdb_id)
+    {   /* we can probably find the original track from the DND data */
+	GList *gl;
+	for (gl=itdb->tracks; gl; gl=gl->next)
+	{
+	    itr = gl->data;
+	    g_return_val_if_fail (itr, NULL);
+	    if (itr->dbid == etr->local_track_dbid)
+	    {   /* found track */
+		tracks = g_list_prepend (tracks, itr);
+		return tracks;
+	    }
+	}
+	/* DND origin data is no longer valid */
+	etr->local_itdb_id = 0;
+	etr->local_track_dbid = 0;
+    }
+
+    /* No luck so far -- let's get filename matches */
+    tracks = gp_itdb_pc_path_hash_find_tracks (itdb, etr->pc_path_utf8);
+
+    /* And also try SHA1 match */
+    itr = sha1_sha1_exists (itdb, etr->sha1_hash);
+
+    if (itr)
+    {   /* insert into tracks list if not already present */
+	if (!g_list_find (tracks, itr))
+	{
+	    tracks = g_list_prepend (tracks, itr);
+	}
+    }
+
+    return tracks;
+}
+
+
+/* Find @track in all local repositories and return a list.
+
+   This function calls gp_itdb_find_same_tracks() for each local
+   repository and concatenates the results into one list.
+
+   Return value: a GList with matching tracks. You must call
+   g_list_free() on the list when it is no longer needed.
+*/
+GList *gp_itdb_find_same_tracks_in_local_itdbs (Track *track)
+{
+    GList *gl, *tracks=NULL;
+    struct itdbs_head *ih = gp_get_itdbs_head (gtkpod_window);
+
+    g_return_val_if_fail (ih, NULL);
+    g_return_val_if_fail (track, NULL);
+
+    for (gl=ih->itdbs; gl; gl=gl->next)
+    {
+	iTunesDB *itdb = gl->data;
+	g_return_val_if_fail (itdb, tracks);
+	if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	{
+	    GList *addtracks = gp_itdb_find_same_tracks (itdb, track);
+	    tracks = g_list_concat (tracks, addtracks);
+	}
+    }
+    return tracks;
+}
+
+
+
+/* Find @track in all repositories (local and iPod) and return a list.
+
+   This function calls gp_itdb_find_same_tracks() for each
+   repository and concatenates the results into one list.
+
+   Return value: a GList with matching tracks. You must call
+   g_list_free() on the list when it is no longer needed.
+*/
+GList *gp_itdb_find_same_tracks_in_itdbs (Track *track)
+{
+    GList *gl, *tracks=NULL;
+    struct itdbs_head *ih = gp_get_itdbs_head (gtkpod_window);
+
+    g_return_val_if_fail (ih, NULL);
+    g_return_val_if_fail (track, NULL);
+
+    for (gl=ih->itdbs; gl; gl=gl->next)
+    {
+	GList *addtracks;
+	iTunesDB *itdb = gl->data;
+	g_return_val_if_fail (itdb, tracks);
+	addtracks = gp_itdb_find_same_tracks (itdb, track);
+	tracks = g_list_concat (tracks, addtracks);
+    }
+    return tracks;
+}
+
+
+
+/* ------------------------------------------------------------ *\
+|                                                                |
+|         functions for local path hashtable                     |
+|                                                                |
+\* ------------------------------------------------------------ */
+
+/* set up hash table for local filenames */
+void gp_itdb_pc_path_hash_init (ExtraiTunesDBData *eitdb)
+{
+    g_return_if_fail (eitdb);
+
+    if (!eitdb->pc_path_hash)
+    {
+	eitdb->pc_path_hash = g_hash_table_new_full (
+	    g_str_hash, g_str_equal,
+	    g_free, NULL);
+    }
+}
+
+
+/* function for destroying hash value, used when destroying the hash
+ * table in gp_itdb_local_path_hash_destroy() below. */
+static void pc_path_hash_free_value (gpointer key,
+					gpointer value,
+					gpointer userdata)
+{
+    g_list_free (value);
+}
+
+
+/* free all memory associated with the local_path_hash */
+void gp_itdb_pc_path_hash_destroy (ExtraiTunesDBData *eitdb)
+{
+    g_return_if_fail (eitdb);
+
+    if (eitdb->pc_path_hash)
+    {
+	g_hash_table_foreach (eitdb->pc_path_hash,
+			      pc_path_hash_free_value,
+			      NULL);
+	g_hash_table_destroy (eitdb->pc_path_hash);
+	eitdb->pc_path_hash = NULL;
+    }
+}
+
+
+/* Add track to filehash. This function must only be called once for
+ * each track. */
+void gp_itdb_pc_path_hash_add_track (Track *track)
+{
+    iTunesDB *itdb;
+    ExtraTrackData *etr;
+    ExtraiTunesDBData *eitdb;
+
+    g_return_if_fail (track);
+    etr = track->userdata;
+    g_return_if_fail (etr);
+
+    itdb = track->itdb;
+    g_return_if_fail (itdb);
+
+    eitdb = itdb->userdata;
+    g_return_if_fail (eitdb);
+    g_return_if_fail (eitdb->pc_path_hash);
+
+    g_return_if_fail (track);
+    etr = track->userdata;
+    g_return_if_fail (etr);
+
+    /* This is only to detect programming errors */
+    g_return_if_fail (!etr->pc_path_hashed);
+
+    if (etr->pc_path_utf8 && *etr->pc_path_utf8)
+    {   /* add to hash table */
+	GList *tracks;
+	tracks = g_hash_table_lookup (eitdb->pc_path_hash, etr->pc_path_utf8);
+	tracks = g_list_prepend (tracks, track);
+	g_hash_table_replace (eitdb->pc_path_hash,
+			      g_strdup (etr->pc_path_utf8),
+			      tracks);
+	/* This is only to detect programming errors */
+	etr->pc_path_hashed = TRUE;
+    }
+}
+
+/* used in the next two functions */
+struct pc_path_hash_find_track_data
+{
+    Track *track;
+    gchar *key;
+};
+
+
+/* Used in the next function. Return TRUE and the current key if
+   td->track is contained in the list for key */
+static gboolean pc_path_hash_find_track (gpointer key,
+					 gpointer value,
+					 gpointer user_data)
+{
+    GList *tracks = value;
+    struct pc_path_hash_find_track_data *td = user_data;
+
+    if (g_list_find (tracks, td->track))
+    {
+	td->key = g_strdup (key);
+	return TRUE;
+    }
+    return FALSE;
+}
+
+
+
+/* remove track from filehash */
+void gp_itdb_pc_path_hash_remove_track (Track *track)
+{
+    struct pc_path_hash_find_track_data td;
+    ExtraTrackData *etr;
+    iTunesDB *itdb;
+    ExtraiTunesDBData *eitdb;
+    GList *tracks;
+
+    g_return_if_fail (track);
+    etr = track->userdata;
+    g_return_if_fail (etr);
+
+    itdb = track->itdb;
+    g_return_if_fail (itdb);
+
+    eitdb = itdb->userdata;
+    g_return_if_fail (eitdb);
+    g_return_if_fail (eitdb->pc_path_hash);
+
+    if (!etr->pc_path_hashed)	return;
+
+    if (etr->pc_path_utf8 && *etr->pc_path_utf8)
+    {   /* try lookup with filename */
+	GList *tracks;
+	tracks = g_hash_table_lookup (eitdb->pc_path_hash, etr->pc_path_utf8);
+	if (tracks)
+	{   /* filename exists */
+	    GList *link = g_list_find (tracks, track);
+	    if (link)
+	    {   /* track found */
+		tracks = g_list_remove_link (tracks, link);
+		if (tracks)
+		{   /* still tracks left under this filename */
+		    g_hash_table_replace (eitdb->pc_path_hash,
+					  g_strdup (etr->pc_path_utf8),
+					  tracks);
+		}
+		else
+		{   /* no more tracks left under this filename */
+		    g_hash_table_remove (eitdb->pc_path_hash,
+					 etr->pc_path_utf8);
+		}
+		etr->pc_path_hashed = FALSE;
+		return;
+	    }
+	}
+    }
+
+    /* We didn't find the track by filename, or now filename is
+     * available any more) -> search through the list */
+    td.track = track;
+    td.key = NULL;
+    tracks = g_hash_table_find (eitdb->pc_path_hash,
+				pc_path_hash_find_track,
+				&td);
+    if (tracks)
+    {
+	tracks = g_list_remove (tracks, track);
+	if (tracks)
+	{   /* still tracks left under this filename */
+	    g_hash_table_replace (eitdb->pc_path_hash,
+				  g_strdup (td.key),
+				  tracks);
+	}
+	else
+	{   /* no more tracks left under this filename */
+	    g_hash_table_remove (eitdb->pc_path_hash,
+				 td.key);
+	}
+	etr->pc_path_hashed = FALSE;
+	g_free (td.key);
+    }
+}
+
+
+/* Return all tracks with @filename (@filename in UTF8).
+   You must g_list_free the returned list when it is not longer needed. */
+GList *gp_itdb_pc_path_hash_find_tracks (iTunesDB *itdb, const gchar *filename)
+{
+    ExtraiTunesDBData *eitdb;
+    GList *tracks = NULL;
+
+    g_return_val_if_fail (itdb, NULL);
+    eitdb = itdb->userdata;
+    g_return_val_if_fail (eitdb, NULL);
+    g_return_val_if_fail (eitdb->pc_path_hash, NULL);
+
+    if (filename && *filename)
+	tracks = g_hash_table_lookup (eitdb->pc_path_hash, filename);
+
+    return g_list_copy (tracks);
+}
+
+
 
 /* ------------------------------------------------------------ *\
 |                                                                |
@@ -526,6 +852,36 @@ gchar **track_get_item_pointer (Track *track, T_item t_item)
     case T_THUMB_PATH:
 	result = &etr->thumb_path_utf8;
 	break;
+    case T_TV_SHOW:
+        result = &track->tvshow;
+        break;
+    case T_TV_EPISODE:
+        result = &track->tvepisode;
+        break;
+    case T_TV_NETWORK:
+        result = &track->tvnetwork;
+        break;
+    case T_ALBUMARTIST:
+	result = &track->albumartist;
+	break;
+    case T_SORT_ARTIST:
+	result = &track->sort_artist;
+	break;
+    case T_SORT_TITLE:
+	result = &track->sort_title;
+	break;
+    case T_SORT_ALBUM:
+	result = &track->sort_album;
+	break;
+    case T_SORT_ALBUMARTIST:
+	result = &track->sort_albumartist;
+	break;
+    case T_SORT_COMPOSER:
+	result = &track->sort_composer;
+	break;
+    case T_SORT_TVSHOW:
+	result = &track->sort_tvshow;
+	break;
     case T_ALL:
     case T_IPOD_ID:
     case T_TRACK_NR:
@@ -550,6 +906,10 @@ gchar **track_get_item_pointer (Track *track, T_item t_item)
     case T_REMEMBER_PLAYBACK_POSITION:
     case T_SKIP_WHEN_SHUFFLING:
     case T_CHECKED:
+    case T_MEDIA_TYPE:
+    case T_SEASON_NR:
+    case T_EPISODE_NR:
+    case T_GAPLESS_TRACK_FLAG:
     case T_ITEM_NUM:
 	g_return_val_if_reached (NULL);
     }
@@ -587,11 +947,17 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
     gboolean changed = FALSE;
     const gchar *fritem;
     gchar **toitem_ptr;
+    ExtraTrackData *efrtr, *etotr;
 
     g_return_val_if_fail (frtrack, FALSE);
     g_return_val_if_fail (totrack, FALSE);
     g_return_val_if_fail ((item > 0) && (item < T_ITEM_NUM), FALSE);
 
+    efrtr = frtrack->userdata;
+    etotr = totrack->userdata;
+    g_return_val_if_fail (efrtr, FALSE);
+    g_return_val_if_fail (etotr, FALSE);
+
     if (frtrack == totrack) return FALSE;
 
     switch (item)
@@ -613,6 +979,16 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
     case T_PODCASTRSS:
     case T_SUBTITLE:
     case T_THUMB_PATH:
+    case T_TV_SHOW:
+    case T_TV_EPISODE:
+    case T_TV_NETWORK:
+    case T_ALBUMARTIST:
+    case T_SORT_ARTIST:
+    case T_SORT_TITLE:
+    case T_SORT_ALBUM:
+    case T_SORT_ALBUMARTIST:
+    case T_SORT_COMPOSER:
+    case T_SORT_TVSHOW:
 	fritem = track_get_item (frtrack, item);
 	toitem_ptr = track_get_item_pointer (totrack, item);
 	g_return_val_if_fail (fritem, FALSE);
@@ -631,6 +1007,27 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
 		changed = TRUE;
 	    }
 	}
+	/* handle items that have two entries */
+	if (item == T_PC_PATH)
+	{
+	    if ((etotr->pc_path_locale == NULL) ||
+		(strcmp (efrtr->pc_path_locale, etotr->pc_path_locale) != 0))
+	    {
+		g_free (etotr->pc_path_locale);
+		etotr->pc_path_locale = g_strdup (efrtr->pc_path_locale);
+		changed = TRUE;
+	    }
+	}
+	if (item == T_THUMB_PATH)
+	{
+	    if ((etotr->thumb_path_locale == NULL) ||
+		(strcmp (efrtr->thumb_path_locale, etotr->thumb_path_locale) != 0))
+	    {
+		g_free (etotr->thumb_path_locale);
+		etotr->thumb_path_locale = g_strdup (efrtr->thumb_path_locale);
+		changed = TRUE;
+	    }
+	}
 	break;
     case T_IPOD_ID:
 	if (frtrack->id != totrack->id)
@@ -786,6 +1183,34 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
 	    changed = TRUE;
 	}
 	break;
+    case T_MEDIA_TYPE:
+	if (frtrack->mediatype != totrack->mediatype)
+	{
+	    totrack->mediatype = frtrack->mediatype;
+	    changed = TRUE;
+	}
+	break;
+    case T_SEASON_NR:
+	if (frtrack->season_nr != totrack->season_nr)
+	{
+	    totrack->season_nr = frtrack->season_nr;
+	    changed = TRUE;
+	}
+	break;
+    case T_EPISODE_NR:
+	if (frtrack->episode_nr != totrack->episode_nr)
+	{
+	    totrack->episode_nr = frtrack->episode_nr;
+	    changed = TRUE;
+	}
+	break;
+    case T_GAPLESS_TRACK_FLAG:
+	if (frtrack->gapless_track_flag != totrack->gapless_track_flag)
+	{
+	    totrack->gapless_track_flag = frtrack->gapless_track_flag;
+	    changed = TRUE;
+	}
+	break;
     case T_ITEM_NUM:
     case T_ALL:
 	g_return_val_if_reached (FALSE);
@@ -798,7 +1223,7 @@ gboolean track_copy_item (Track *frtrack, Track *totrack, T_item item)
 /* return a pointer to the specified timestamp. @t_item is one of (the
    applicable) T_* defined in track.h.  If the parameters are illegal,
    "0" is returned. */
-guint32 *track_get_timestamp_ptr (Track *track, T_item t_item)
+time_t *track_get_timestamp_ptr (Track *track, T_item t_item)
 {
     g_return_val_if_fail (track, NULL);
 
@@ -821,9 +1246,9 @@ guint32 *track_get_timestamp_ptr (Track *track, T_item t_item)
 /* return the specified timestamp. @t_item is one of
    (the * applicable) T_* defined in track.h. If the parameters are
    illegal, "0" is returned. */
-guint32 track_get_timestamp (Track *track, T_item t_item)
+time_t track_get_timestamp (Track *track, T_item t_item)
 {
-    guint32 *ptr;
+    time_t *ptr;
     g_return_val_if_fail (track, 0);
 
     ptr = track_get_timestamp_ptr (track, t_item);
@@ -870,6 +1295,16 @@ gchar *track_get_text (Track *track, T_item item)
     case T_PODCASTURL:
     case T_PODCASTRSS:
     case T_SUBTITLE:
+    case T_TV_SHOW:
+    case T_TV_EPISODE:
+    case T_TV_NETWORK:
+    case T_ALBUMARTIST:
+    case T_SORT_ARTIST:
+    case T_SORT_TITLE:
+    case T_SORT_ALBUM:
+    case T_SORT_ALBUMARTIST:
+    case T_SORT_COMPOSER:
+    case T_SORT_TVSHOW:
 	text = g_strdup (track_get_item (track, item));
 	break;
     case T_TRACK_NR:
@@ -965,6 +1400,15 @@ gchar *track_get_text (Track *track, T_item item)
     case T_SOUNDCHECK:
 	text = g_strdup_printf ("%0.2f", soundcheck_to_replaygain (track->soundcheck));
 	break;
+    case T_SEASON_NR:
+        text = g_strdup_printf ("%d", track->season_nr);
+	break;
+    case T_EPISODE_NR:
+        text = g_strdup_printf ("%d", track->episode_nr);
+	break;
+    case T_MEDIA_TYPE:
+        text = g_strdup_printf ("%#.8x", track->mediatype);
+	break;
     case T_TRANSFERRED:
     case T_COMPILATION:
     case T_REMEMBER_PLAYBACK_POSITION:
@@ -972,6 +1416,7 @@ gchar *track_get_text (Track *track, T_item item)
     case T_ALL:
     case T_CHECKED:
     case T_ITEM_NUM:
+    case T_GAPLESS_TRACK_FLAG:
 	break;
     }
     return text;
@@ -1038,6 +1483,16 @@ gboolean track_set_text (Track *track, const gchar *new_text, T_item item)
     case T_PODCASTURL:
     case T_PODCASTRSS:
     case T_SUBTITLE:
+    case T_TV_SHOW:
+    case T_TV_EPISODE:
+    case T_TV_NETWORK:
+    case T_ALBUMARTIST:
+    case T_SORT_ARTIST:
+    case T_SORT_TITLE:
+    case T_SORT_ALBUM:
+    case T_SORT_ALBUMARTIST:
+    case T_SORT_COMPOSER:
+    case T_SORT_TVSHOW:
         itemp_utf8 = track_get_item_pointer (track, item);
         if (g_utf8_collate (*itemp_utf8, new_text) != 0)
         {
@@ -1202,6 +1657,23 @@ gboolean track_set_text (Track *track, const gchar *new_text, T_item item)
 	    changed = TRUE;
 	}
 	break;
+    case T_SEASON_NR:
+        nr = atoi (new_text);
+        if ((nr >= 0) && (nr != track->season_nr))
+        {
+	    track->season_nr = nr;
+	    changed = TRUE;
+        }
+        break;
+    case T_EPISODE_NR:
+        nr = atoi (new_text);
+        if ((nr >= 0) && (nr != track->episode_nr))
+        {
+	    track->episode_nr = nr;
+	    changed = TRUE;
+        }
+        break;
+    case T_MEDIA_TYPE:
     case T_PC_PATH:
     case T_IPOD_PATH:
     case T_IPOD_ID:
@@ -1211,6 +1683,7 @@ gboolean track_set_text (Track *track, const gchar *new_text, T_item item)
     case T_SKIP_WHEN_SHUFFLING:
     case T_CHECKED:
     case T_ALL:
+    case T_GAPLESS_TRACK_FLAG:
     case T_ITEM_NUM:
     case T_THUMB_PATH: // TODO: this should in fact be settable
 	gtkpod_warning ("Programming error: track_set_text() called with illegal argument (item: %d)\n", item);
@@ -1245,7 +1718,7 @@ void gp_info_nontransferred_tracks (iTunesDB *itdb,
 	g_return_if_fail (etr);
 	if (!tr->transferred)
 	{
-	    if (size)  *size += tr->size - etr->oldsize;
+	    if (size)  *size += tr->size;
 	    if (num)   *num += 1;
 	}
     }
@@ -1282,12 +1755,13 @@ static void add_tracks_to_playlist (Playlist *pl,
 		/* set flags to 'podcast' if adding to podcast list */
 		if (itdb_playlist_is_podcasts (pl))
 		    gp_track_set_flags_podcast (track);
-
+#if 0 /* initially iTunes didn't add podcasts to the MPL */
 		if (!itdb_playlist_contains_track (to_mpl, track))
 		{   /* add to MPL if not already present (will happen
 		     * if dragged from the podcasts playlist */
 		    gp_playlist_add_track (to_mpl, track, TRUE);
 		}
+#endif
 		if (!itdb_playlist_is_mpl (pl))
 		{
 		    /* add to designated playlist -- unless adding
@@ -1309,8 +1783,49 @@ static void add_tracks_to_playlist (Playlist *pl,
 	    {   /* DND between different itdbs -- need to duplicate the
 		   track before inserting */
 		Track *duptr, *addtr;
+		ExtraTrackData *eduptr;
 		/* duplicate track */
 		duptr = itdb_track_duplicate (track);
+		eduptr = duptr->userdata;
+		g_return_if_fail (eduptr);
+
+		duptr->transferred = FALSE;
+
+		/* check if adding to iPod and track is on different iPod */
+		if ((from_itdb->usertype & GP_ITDB_TYPE_IPOD) &&
+		    (to_itdb->usertype & GP_ITDB_TYPE_IPOD))
+		{
+		    /* Check if track exists locally */
+		    if (!(eduptr->pc_path_locale &&
+			  g_file_test (eduptr->pc_path_locale, G_FILE_TEST_EXISTS)))
+		    {   /* No. Use iPod path as source */
+			g_free (eduptr->pc_path_locale);
+			g_free (eduptr->pc_path_utf8);
+			eduptr->pc_path_locale = itdb_filename_on_ipod (track);
+			eduptr->pc_path_utf8 = charset_to_utf8 (eduptr->pc_path_locale);
+		    }
+		    /* Remove old reference to iPod path */
+		    g_free (duptr->ipod_path);
+		    duptr->ipod_path = g_strdup ("");
+		}
+
+		if (!eduptr->pc_path_locale)
+		{
+		    gchar *buf;
+		    buf = get_track_info (track, FALSE);
+		    gtkpod_warning (_("Could not find source file for '%s'. Track not copied."));
+		    g_free (buf);
+		    itdb_track_free (duptr);
+		    return;
+		}
+
+		if ((from_itdb->usertype & GP_ITDB_TYPE_LOCAL) &&
+		    (to_itdb->usertype & GP_ITDB_TYPE_IPOD))
+		{   /* make sure the DND origin data is set correctly */
+		    eduptr->local_itdb_id = from_itdb->id;
+		    eduptr->local_track_dbid = track->dbid;
+		}
+
 		/* add to database -- if duplicate detection is on and the
 		   same track already exists in the database, the already
 		   existing track is returned and @duptr is freed */
@@ -1322,6 +1837,7 @@ static void add_tracks_to_playlist (Playlist *pl,
 
 		if (addtr == duptr)
 		{   /* no duplicate */
+#if 0 /* initially iTunes didn't add podcasts to the MPL */
 		    /* we need to add to the MPL if the track is no
 		       duplicate and will not be added to the podcasts
 		       playlist */
@@ -1330,7 +1846,13 @@ static void add_tracks_to_playlist (Playlist *pl,
 			   playlist */
 			gp_playlist_add_track (to_mpl, addtr, TRUE);
 		    }
+#else
+		    /* we need to add to the MPL if the track is no
+		       duplicate */
+		    gp_playlist_add_track (to_mpl, addtr, TRUE);
+#endif
 		}
+#if 0 /* initially iTunes didn't add podcasts to the MPL */
 		else
 		{   /* duplicate */
 		    /* we also need to add to the MPL if the track is a
@@ -1343,6 +1865,7 @@ static void add_tracks_to_playlist (Playlist *pl,
 			gp_playlist_add_track (to_mpl, addtr, TRUE);
 		    }
 		}
+#endif
 		/* add to designated playlist (if not mpl) -- unless
 		 * adding to podcasts list and track already * exists
 		 * there */
@@ -1495,6 +2018,8 @@ Playlist *add_text_plain_to_playlist (iTunesDB *itdb, Playlist *pl,
 		    case FILE_TYPE_MP4:
 		    case FILE_TYPE_MOV:
 		    case FILE_TYPE_MPG:
+                    case FILE_TYPE_OGG:
+                    case FILE_TYPE_FLAC:
 			if (!pl)
 			{  /* no playlist yet -- create new one */
 			    pl = add_new_pl_user_name (itdb, NULL,
