@@ -1,4 +1,4 @@
-/* Time-stamp: <2006-09-18 16:03:15 jcs>
+/* Time-stamp: <2007-06-23 02:49:08 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp4file.c,v 1.34 2006/09/18 07:10:44 jcsjcs Exp $
+|  $Id: mp4file.c 1740 2007-10-27 02:48:08Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -34,8 +34,10 @@
 #include "charset.h"
 #include "itdb.h"
 #include "misc.h"
-#include "mp4file.h"
 #include "prefs.h"
+#include "mp4file.h"
+
+
 
 /* ------------------------------------------------------------
 
@@ -73,6 +75,8 @@
    guint16 samplerate;        /+ e.g.: CD is 44100     +/
    guint32 peak_signal;	      /+ LAME Peak Signal * 0x800000         +/
    gboolean compilation;      /+ Track is part of a compilation CD   +/
+   gboolean lyrics_flag;
+   gint16 bpm;
 
    If prefs_get_int("readtags") returns FALSE you only should fill in
    tracklen, bitrate, samplerate, soundcheck and filetype
@@ -134,12 +138,20 @@
 #include <stdlib.h>
 #include <string.h>
 #include "mp4.h"
-#undef PACKAGE
-#undef VERSION
-/* need to include config.h again because mp4.h seems to #undef
-   PACKAGE */
-#include <config.h>
 
+#ifndef FREEFORM_ACCEPTS_EXTRA_ARG
+/* Version 1.6 of libmp4v2 introduces an index argument for MP4GetMetadataFreeForm. For C++ sources it defaults
+   to 0, but in C we have to specify it on our own. 
+*/
+#define MP4GetMetadataFreeForm(mp4File, name, pValue, pValueSize, owner)  MP4GetMetadataFreeForm(mp4File, name, pValue, pValueSize)
+#endif
+
+#ifndef COVERART_ACCEPTS_EXTRA_ARG
+/* Version 1.6 of libmp4v2 introduces an index argument for MP4GetMetadataCoverart. For C++ sources it defaults
+   to NULL, but in C we have to specify it on our own. 
+*/
+#define MP4GetMetadataCoverArt(hFile, coverArt, size, index) MP4GetMetadataCoverArt(hFile, coverArt, size)
+#endif
 
 static gboolean mp4_scan_soundcheck (MP4FileHandle mp4File, Track *track)
 {
@@ -151,7 +163,7 @@ static gboolean mp4_scan_soundcheck (MP4FileHandle mp4File, Track *track)
     g_return_val_if_fail (mp4File != MP4_INVALID_FILE_HANDLE, FALSE);
 
     if (MP4GetMetadataFreeForm(mp4File, "iTunNORM",
-			       &ppValue, &pValueSize))
+			       &ppValue, &pValueSize, NULL))
     {
 	gchar *str;
 	guint sc1=0, sc2=0;
@@ -173,7 +185,7 @@ static gboolean mp4_scan_soundcheck (MP4FileHandle mp4File, Track *track)
     }
 
     if (MP4GetMetadataFreeForm(mp4File, "replaygain_track_gain",
-			       &ppValue, &pValueSize))
+			       &ppValue, &pValueSize, NULL))
     {
 	gchar *str;
 	gdouble rg;
@@ -204,22 +216,32 @@ gboolean mp4_read_soundcheck (gchar *mp4FileName, Track *track)
     {
 	MP4TrackId trackId;
 	const char *trackType;
+	u_int32_t track_cur, tracks_num;
+	gboolean audio_or_video_found = FALSE;
 
-	trackId = MP4FindTrackId(mp4File, 0, NULL, 0);
-	trackType = MP4GetTrackType(mp4File, trackId);
+	tracks_num = MP4GetNumberOfTracks (mp4File, NULL,  0);
 
-	if (trackType &&
-	    ((strcmp(trackType, MP4_AUDIO_TRACK_TYPE) == 0) ||
-	     (strcmp(trackType, MP4_VIDEO_TRACK_TYPE) == 0) ||
-	     (strcmp(trackType, MP4_OD_TRACK_TYPE) == 0)))
+	for (track_cur=0; track_cur < tracks_num; ++track_cur)
 	{
-	    success = mp4_scan_soundcheck (mp4File, track);
+	    trackId = MP4FindTrackId(mp4File, track_cur, NULL, 0);
+	    trackType = MP4GetTrackType(mp4File, trackId);
+
+	    if (trackType &&
+		((strcmp(trackType, MP4_AUDIO_TRACK_TYPE) == 0) ||
+		 (strcmp(trackType, MP4_VIDEO_TRACK_TYPE) == 0) ||
+		 (strcmp(trackType, MP4_OD_TRACK_TYPE) == 0)))
+	    {
+		audio_or_video_found = TRUE;
+		success = mp4_scan_soundcheck (mp4File, track);
+	    }
+	    if (audio_or_video_found) break;
 	}
-	else
+	if (!audio_or_video_found)
 	{
 	    gchar *filename = charset_to_utf8 (mp4FileName);
-	    gtkpod_warning (_("'%s' does not appear to be a mp4 audio file.\n"),
-			    filename);
+	    gtkpod_warning (
+		_("'%s' does not appear to be a mp4 audio or video file.\n"),
+		filename);
 	    g_free (filename);
 	}
 	MP4Close(mp4File);
@@ -251,96 +273,136 @@ Track *mp4_get_file_info (gchar *mp4FileName)
     {
 	MP4TrackId trackId;
 	const char *trackType;
+	u_int32_t track_cur, tracks_num;
+	gboolean audio_or_video_found = FALSE;
+/*	gboolean artwork_found = FALSE; not used yet */
 
-	trackId = MP4FindTrackId(mp4File, 0, NULL, 0);
-	trackType = MP4GetTrackType(mp4File, trackId);
-	if (trackType &&
-	    ((strcmp(trackType, MP4_AUDIO_TRACK_TYPE) == 0) ||
-	     (strcmp(trackType, MP4_VIDEO_TRACK_TYPE) == 0) ||
-	     (strcmp(trackType, MP4_OD_TRACK_TYPE) == 0)))
+	tracks_num = MP4GetNumberOfTracks (mp4File, NULL,  0);
+
+	for (track_cur=0; track_cur < tracks_num; ++track_cur)
 	{
-	    gchar *value;
-	    guint16 numvalue, numvalue2;
-	    MP4Duration trackDuration = MP4GetTrackDuration(mp4File, trackId);
-	    double msDuration = 
-		(double)MP4ConvertFromTrackDuration(mp4File, trackId,
-						    trackDuration,
-						    MP4_MSECS_TIME_SCALE);
-	    guint32 avgBitRate = MP4GetTrackBitRate(mp4File, trackId);
-	    guint32 samplerate = MP4GetTrackTimeScale(mp4File, trackId);
-
-	    track = gp_track_new ();
-
-	    track->tracklen = msDuration;
-	    track->bitrate = avgBitRate/1000;
-	    track->samplerate = samplerate;
-	    value = strrchr (mp4FileName, '.');
-	    if (value)
+	    trackId = MP4FindTrackId(mp4File, track_cur, NULL, 0);
+	    trackType = MP4GetTrackType(mp4File, trackId);
+	    if (trackType &&
+		(audio_or_video_found == FALSE) &&
+		((strcmp(trackType, MP4_AUDIO_TRACK_TYPE) == 0) ||
+		 (strcmp(trackType, MP4_VIDEO_TRACK_TYPE) == 0) ||
+		 (strcmp(trackType, MP4_OD_TRACK_TYPE) == 0)))
 	    {
-		if (g_strcasecmp (value, ".m4a") == 0)
-		    track->filetype = g_strdup ("AAC audio file");
-		if (g_strcasecmp (value, ".m4p") == 0)
-		    track->filetype = g_strdup ("Protected AAC audio file");
-		if (g_strcasecmp (value, ".m4b") == 0)
-		    track->filetype = g_strdup ("AAC audio book file");
-		if (g_strcasecmp (value, ".mp4") == 0)
-		    track->filetype = g_strdup ("MP4 video file");
-	    }
-	    if (prefs_get_int("readtags"))
-	    {
-		if (MP4GetMetadataName(mp4File, &value) && value != NULL)
+		gchar *value;
+		guint16 numvalue, numvalue2;
+		MP4Duration trackDuration = MP4GetTrackDuration(mp4File, trackId);
+		double msDuration = 
+		    (double)MP4ConvertFromTrackDuration(mp4File, trackId,
+							trackDuration,
+							MP4_MSECS_TIME_SCALE);
+		guint32 avgBitRate = MP4GetTrackBitRate(mp4File, trackId);
+		guint32 samplerate = MP4GetTrackTimeScale(mp4File, trackId);
+		
+		track = gp_track_new ();
+		
+		track->tracklen = msDuration;
+		track->bitrate = avgBitRate/1000;
+		track->samplerate = samplerate;
+		value = strrchr (mp4FileName, '.');
+		if (value)
 		{
-		    track->title = charset_to_utf8 (value);
-		    g_free(value);
+		    if (g_strcasecmp (value, ".m4a") == 0)
+			track->filetype = g_strdup ("AAC audio file");
+		    if (g_strcasecmp (value, ".m4p") == 0)
+			track->filetype = g_strdup ("Protected AAC audio file");
+		    if (g_strcasecmp (value, ".m4b") == 0)
+			track->filetype = g_strdup ("AAC audio book file");
+		    if (g_strcasecmp (value, ".mp4") == 0)
+			track->filetype = g_strdup ("MP4 video file");
 		}
-		if (MP4GetMetadataArtist(mp4File, &value) && value != NULL)
+		if (prefs_get_int("readtags"))
 		{
-		    track->artist = charset_to_utf8 (value);
-		    g_free(value);
+		    if (MP4GetMetadataName(mp4File, &value) && value != NULL)
+		    {
+			track->title = charset_to_utf8 (value);
+			g_free(value);
+		    }
+		    if (MP4GetMetadataArtist(mp4File, &value) && value != NULL)
+		    {
+			track->artist = charset_to_utf8 (value);
+			g_free(value);
+		    }
+		    if (MP4GetMetadataWriter(mp4File, &value) && value != NULL)
+		    {
+			track->composer = charset_to_utf8 (value);
+			g_free(value);
+		    }
+		    if (MP4GetMetadataComment(mp4File, &value) && value != NULL)
+		    {
+			track->comment = charset_to_utf8 (value);
+			g_free(value);
+		    }
+		    if (MP4GetMetadataYear(mp4File, &value) && value != NULL)
+		    {
+			track->year = atoi (value);
+			g_free(value);
+		    }
+		    if (MP4GetMetadataAlbum(mp4File, &value) && value != NULL)
+		    {
+			track->album = charset_to_utf8 (value);
+			g_free(value);
+		    }
+		    if (MP4GetMetadataTrack(mp4File, &numvalue, &numvalue2))
+		    {
+			track->track_nr = numvalue;
+			track->tracks = numvalue2;
+		    }
+		    if (MP4GetMetadataDisk(mp4File, &numvalue, &numvalue2))
+		    {
+			track->cd_nr = numvalue;
+			track->cds = numvalue2;
+		    }
+		    if (MP4GetMetadataGrouping(mp4File, &value) && value != NULL)
+		    {
+			track->grouping = charset_to_utf8 (value);
+			g_free (value);
+		    }
+		    if (MP4GetMetadataGenre(mp4File, &value) && value != NULL)
+		    {
+			track->genre = charset_to_utf8 (value);
+			g_free(value);
+		    }
+		    if (MP4GetMetadataTempo (mp4File, &numvalue))
+		    {
+			track->BPM = numvalue;
+		    }
 		}
-		if (MP4GetMetadataWriter(mp4File, &value) && value != NULL)
+		mp4_scan_soundcheck (mp4File, track);
+		audio_or_video_found = TRUE;
+
+		if (prefs_get_int("coverart_apic"))
 		{
-		    track->composer = charset_to_utf8 (value);
-		    g_free(value);
-		}
-		if (MP4GetMetadataComment(mp4File, &value) && value != NULL)
-		{
-		    track->comment = charset_to_utf8 (value);
-		    g_free(value);
-		}
-		if (MP4GetMetadataYear(mp4File, &value) && value != NULL)
-		{
-		    track->year = atoi (value);
-		    g_free(value);
-		}
-		if (MP4GetMetadataAlbum(mp4File, &value) && value != NULL)
-		{
-		    track->album = charset_to_utf8 (value);
-		    g_free(value);
-		}
-		if (MP4GetMetadataTrack(mp4File, &numvalue, &numvalue2))
-		{
-		    track->track_nr = numvalue;
-		    track->tracks = numvalue2;
-		}
-		if (MP4GetMetadataDisk(mp4File, &numvalue, &numvalue2))
-		{
-		    track->cd_nr = numvalue;
-		    track->cds = numvalue2;
-		}
-		if (MP4GetMetadataGenre(mp4File, &value) && value != NULL)
-		{
-		    track->genre = charset_to_utf8 (value);
-		    g_free(value);
+		    u_int8_t *image_data;
+		    u_int32_t image_data_len;
+		    if (MP4GetMetadataCoverArt (mp4File,
+						&image_data, &image_data_len, 0))
+		    {
+			if (image_data)
+			{
+/*			    FILE *file = fopen ("/tmp/tttt", "w");
+			    fwrite (image_data, 1, image_data_len, file);
+			    fclose (file);*/
+			    gp_track_set_thumbnails_from_data (track,
+							       image_data,
+							       image_data_len);
+			    g_free (image_data);
+			}
+		    }
 		}
 	    }
-	    mp4_scan_soundcheck (mp4File, track);
 	}
-	else
+	if (!audio_or_video_found)
 	{
 	    gchar *filename = charset_to_utf8 (mp4FileName);
-	    gtkpod_warning (_("'%s' does not appear to be a mp4 audio file.\n"),
-			    filename);
+	    gtkpod_warning (
+		_("'%s' does not appear to be a mp4 audio or video file.\n"),
+		filename);
 	    g_free (filename);
 	}
 	MP4Close(mp4File);
@@ -404,7 +466,7 @@ gboolean mp4_write_file_info (gchar *mp4FileName, Track *track)
 	    MP4GetMetadataAlbum (mp4File, &m_album);
 	    MP4GetMetadataGenre (mp4File, &m_genre);*/
 	    MP4GetMetadataTool (mp4File, &m_tool);
-	    MP4GetMetadataCoverArt (mp4File, &m_covert, &m_size);
+	    MP4GetMetadataCoverArt (mp4File, &m_covert, &m_size, 0);
 	    MP4MetadataDelete (mp4File);
 #endif
 	    value = charset_from_utf8 (track->title);
@@ -435,6 +497,12 @@ gboolean mp4_write_file_info (gchar *mp4FileName, Track *track)
 
 	    MP4SetMetadataDisk (mp4File, track->cd_nr, track->cds);
 
+	    MP4SetMetadataTempo (mp4File, track->BPM);
+
+	    value = charset_from_utf8 (track->grouping);
+	    MP4SetMetadataGrouping (mp4File, value);
+	    g_free (value);
+
 	    value = charset_from_utf8 (track->genre);
 	    MP4SetMetadataGenre (mp4File, value);
 	    g_free (value);
