@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-09-24 20:49:39 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Copyright (C) 2006 James Liggett <jrliggett at cox.net>
 |  Part of the gtkpod project.
 |
@@ -25,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: prefs.c,v 1.270 2006/09/24 12:16:26 jcsjcs Exp $
+|  $Id: prefs.c 1776 2007-11-12 22:38:00Z phantom_sf $
 */
 
 /* -------------------------------------------------------------------
@@ -51,6 +50,18 @@
  *
  * ---------------------------------------------------------------- */
 
+/* ----------------------------------------------------------------
+ *
+ * The prefs module should be thread safe. The hash table is locked
+ * before each read or write access.
+ *
+ * The temp_prefs module is not thread-safe. If necessary a locking
+ * mechanism can be implemented.
+ *
+ * ---------------------------------------------------------------- */
+
+
+
 
 /* This tells Alpha OSF/1 not to define a getopt prototype in <stdio.h>.
    Ditto for AIX 3.2 and <stdlib.h>.  */
@@ -62,13 +73,9 @@
 #  include <config.h>
 #endif
 
-#include <ctype.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/types.h>
+#include <glib/gstdio.h>
 #ifdef HAVE_GETOPT_LONG_ONLY
 #  include <getopt.h>
 #else
@@ -76,14 +83,9 @@
 #endif
 
 #include "clientserver.h"
-#include "display.h"
-#include "info.h"
 #include "misc.h"
-#include "misc_track.h"
 #include "prefs.h"
 
-/* New prefs backend. Will replace stuff above */
-
 /*
  * Data global to this module only
  */
@@ -91,6 +93,13 @@
 /* End-of-list marker for variable-length lists */
 #define LIST_END_MARKER "----++++----"
 
+struct temp_prefs_save
+{
+    GIOChannel *gio;
+    GError **error;
+    gboolean success;
+};
+
 struct sub_data
 {
     TempPrefs *temp_prefs;
@@ -100,8 +109,9 @@ struct sub_data
     gboolean exists;
 };
 
-/* Pointer to prefrences hash table */
+/* Pointer to preferences hash table */
 static GHashTable *prefs_table = NULL;
+static GMutex *prefs_table_mutex = NULL;
 
 /*
  * Functions used by this module only
@@ -128,14 +138,31 @@ enum {
   GP_HELP,
   GP_PLAYCOUNT,
   GP_MOUNT,
-  GP_AUTO,
+  GP_PRINT_HASH,
 };
 
-/* Set default prefrences */
+
+/* Lock the prefs table. If the table is already locked the calling
+ * thread will remain blocked until the lock is released by the other thread. */
+static void lock_prefs_table ()
+{
+    g_return_if_fail (prefs_table_mutex);
+    g_mutex_lock (prefs_table_mutex);
+}
+
+/* Unlock the prefs table again. */
+static void unlock_prefs_table ()
+{
+    g_return_if_fail (prefs_table_mutex);
+    g_mutex_unlock (prefs_table_mutex);
+}
+
+
+/* Set default preferences */
 static void set_default_preferences()
 {
     int i;
-    gchar *dir; /* Last directory browsed to */
+    gchar *str;
 
     prefs_set_int("update_existing", FALSE);
     prefs_set_int("id3_write", FALSE);
@@ -146,6 +173,7 @@ static void set_default_preferences()
     prefs_set_int("show_duplicates", TRUE);
     prefs_set_int("show_non_updated", TRUE);
     prefs_set_int("show_updated", TRUE);
+    prefs_set_int("photo_library_confirm_delete", TRUE);
     prefs_set_int("mserv_report_probs", TRUE);
     prefs_set_int("delete_ipod", TRUE);
     prefs_set_int("delete_file", TRUE);
@@ -155,7 +183,23 @@ static void set_default_preferences()
     prefs_set_string ("path_play_now", "xmms %s");
     prefs_set_string ("path_play_enqueue", "xmms -e %s");
     prefs_set_string ("path_mserv_trackinfo_root", "/var/lib/mserv/trackinfo/");
-  
+
+    str = g_build_filename (SCRIPTDIR, "convert-ogg2mp3.sh", NULL);
+    prefs_set_string ("path_conv_ogg", str);
+    g_free (str);
+    prefs_set_int ("convert_ogg", TRUE);
+
+    str = g_build_filename (SCRIPTDIR, "convert-flac2mp3.sh", NULL);
+    prefs_set_string ("path_conv_flac", str);
+    g_free (str);
+    prefs_set_int ("convert_flac", TRUE);
+
+    str = g_build_filename (SCRIPTDIR, "convert-wav2mp3.sh", NULL);
+    prefs_set_string ("path_conv_wav", str);
+    g_free (str);
+    prefs_set_int ("convert_wav", FALSE);
+
+
     /* Set sorting tab defaults */
     for (i = 0; i < SORT_TAB_MAX; i++)
     {
@@ -178,7 +222,7 @@ static void set_default_preferences()
     
     prefs_set_int("sort_tab_num", 2);
     
-    /* Set colum prefrences */
+    /* Set colum preferences */
     for (i = 0; i < TM_NUM_COLUMNS; i++)
     {
 	prefs_set_int_index("tm_col_width", i, 80);
@@ -220,7 +264,14 @@ static void set_default_preferences()
     prefs_set_int("size_info.x", 510);
     prefs_set_int("size_info.y", 300);
 
-    prefs_set_int("autoimport", FALSE);
+    /* size of file dialog if there is not a details textview */
+    prefs_set_int("size_file_dialog.x", 320);
+    prefs_set_int("size_file_dialog.y", 140);
+
+    /* size of file dialog if there is a details textview */
+    prefs_set_int("size_file_dialog_details.x", 320);
+    prefs_set_int("size_file_dialog_details.y", 140);
+
     prefs_set_int("readtags", TRUE);
     prefs_set_int("write_extended_info", TRUE);
     prefs_set_int("parsetags", FALSE);
@@ -235,7 +286,6 @@ static void set_default_preferences()
     prefs_set_int("add_recursively", TRUE);
     prefs_set_int("info_window", FALSE);
     prefs_set_int("last_prefs_page", 0);
-    prefs_set_int("tmp_disable_sort", TRUE);
     prefs_set_int("multi_edit_title", TRUE);
     prefs_set_int("multi_edit", FALSE);
     prefs_set_int("not_played_track", TRUE);
@@ -245,17 +295,17 @@ static void set_default_preferences()
     prefs_set_int("display_tooltips_prefs", TRUE);
     prefs_set_int("display_toolbar", TRUE);
     prefs_set_int("toolbar_style", GTK_TOOLBAR_BOTH);
-    prefs_set_int("block_display", FALSE);
-    prefs_set_int("md5", TRUE);
+    prefs_set_int("sha1", TRUE);
     prefs_set_string("export_template", "%o;%a - %t.mp3;%t.wav");
+    prefs_set_int("file_dialog_details_expanded", FALSE);
 
     /* Set last browsed directory */
-    dir = g_get_current_dir();
+    str = g_get_current_dir();
 
-    if (dir)
+    if (str)
     {
-	prefs_set_string("last_dir_browsed", dir);
-	g_free(dir);
+	prefs_set_string("last_dir_browsed", str);
+	g_free(str);
     }
     else
 	prefs_set_string("last_dir_browsed", g_get_home_dir());
@@ -266,7 +316,7 @@ static void set_default_preferences()
     prefs_set_int("st_sort", SORT_NONE);
     prefs_set_int("pm_sort", SORT_NONE);
     prefs_set_int("tm_sortcol", TM_COLUMN_TITLE);
-    prefs_set_int("tm_sort_", SORT_NONE);
+    prefs_set_int("tm_sort", SORT_NONE);
 }
 
 /* Initialize default variable-length list entries */
@@ -306,7 +356,8 @@ static void usage(FILE *fp)
 {
   locale_fprintf(fp, _("gtkpod version %s usage:\n"), VERSION);
   locale_fprintf(fp, _("  -h, --help:   display this message\n"));
-  locale_fprintf(fp, _("  -p <filename>:increment playcount for file by one\n"));
+  locale_fprintf(fp, _("  -p <file>:    increment playcount for file by one\n"));
+  locale_fprintf(fp, _("  --hash <file>:print gtkpod hash for file\n"));
   locale_fprintf(fp, _("  -m path:      define the mountpoint of your iPod\n"));
   locale_fprintf(fp, _("  --mountpoint: same as '-m'.\n"));
   locale_fprintf(fp, _("  -a:           import database automatically after start.\n"));
@@ -323,11 +374,10 @@ static void read_commandline(int argc, char *argv[])
 	{
 	    { "h",           no_argument,	NULL, GP_HELP },
 	    { "help",        no_argument,	NULL, GP_HELP },
-	    { "p",           required_argument,       NULL, GP_PLAYCOUNT },
+	    { "p",           required_argument, NULL, GP_PLAYCOUNT },
+	    { "hash",        required_argument, NULL, GP_PRINT_HASH },
 	    { "m",           required_argument,	NULL, GP_MOUNT },
 	    { "mountpoint",  required_argument,	NULL, GP_MOUNT },
-	    { "a",           no_argument,	NULL, GP_AUTO },
-	    { "auto",        no_argument,	NULL, GP_AUTO },
 	    { 0, 0, 0, 0 }
 	};
 	
@@ -344,12 +394,13 @@ static void read_commandline(int argc, char *argv[])
 	    client_playcount(optarg);
 	    exit(0);
 	    break;
+	case GP_PRINT_HASH:
+	    print_sha1_hash (optarg);
+	    exit(0);
+	    break;
 	case GP_MOUNT:
 	    prefs_set_string("initial_mountpoint", optarg);
 	    break;
-	case GP_AUTO:
-	    prefs_set_int("autoimport_commandline", TRUE);
-	    break;
 	default:
 	    locale_fprintf(stderr, "Unknown option: %s\n", argv[optind]);
 	    usage(stderr);
@@ -458,7 +509,7 @@ static gboolean check_subkey (gpointer key, gpointer value, gpointer user_data)
 
 /* Add key/value to temp_prefs if it matches subkey -- called by
  * prefs_create_subset() and temp_prefs_create_subset() */
-gboolean get_subset (gpointer key, gpointer value, gpointer user_data)
+static gboolean get_subset (gpointer key, gpointer value, gpointer user_data)
 {
     struct sub_data *sub_data = user_data;
 
@@ -505,9 +556,9 @@ gchar *prefs_get_cfgdir()
 	
     if (!g_file_test(folder, G_FILE_TEST_IS_DIR))
     {
-	if ((mkdir(folder, 0755)) == -1)
+	if ((g_mkdir(folder, 0777)) == -1)
 	{
-	    printf("Couldn't create ~/.gtkpod");
+	    printf(_("Couldn't create ~/.gtkpod\n"));
 	    return NULL;
 	}
     }
@@ -515,55 +566,65 @@ gchar *prefs_get_cfgdir()
     return folder;
 }
 
+
+/* get @key and @value from a string like "key=value" */
+static gboolean read_prefs_get_key_value (const gchar *buf,
+					  gchar **key, gchar **value)
+{
+    size_t len;  /* string length */
+    const gchar *buf_start; /* Pointer to where actual useful data starts in line */
+
+    g_return_val_if_fail (buf && key && value, FALSE);
+
+    /* Strip out any comments (lines that begin with ; or #) */
+    if ((buf[0] == ';') || (buf[0] == '#')) 
+	return FALSE;
+
+    /* Find the key and value, and look for malformed lines */
+    buf_start = strchr (buf, '=');
+
+    if ((!buf_start) || (buf_start == buf))
+    {
+	printf("Parse error reading prefs: %s", buf);
+	return FALSE;
+    }
+
+    /* Find the key name */
+    *key = g_strndup (buf, (buf_start - buf));
+
+    /* Strip whitespace */
+    g_strstrip (*key);
+
+    /* Find the value string */
+    *value = strdup (buf_start+1);
+
+    /* remove newline */
+    len = strlen (*value);
+    if ((len > 0) && ((*value)[len - 1] == 0x0a))
+	(*value)[len - 1] = 0;
+
+    /* Don't strip whitespace! If there is any, there's a reason for it. */
+    /* g_strstrip (*value); */
+
+    return TRUE;
+}
+
+
 /* Read preferences from a file */
 static void read_prefs_from_file(FILE *fp)
 {
     gchar buf[PATH_MAX];  /* Buffer that contains one line */
-    gchar *buf_start; /* Pointer to where actual useful data starts in line */
     gchar *key;  /* Pref value key */
     gchar *value; /* Pref value */
-    size_t len;  /* string length */
-	
-    if (fp)
+
+
+    g_return_if_fail (prefs_table && fp);
+
+    while (fgets(buf, PATH_MAX, fp))
     {
-	while (fgets(buf, PATH_MAX, fp))
+	if (read_prefs_get_key_value (buf, &key, &value))
 	{
-	    /* Strip out any comments (lines that begin with ; or #) */
-	    if ((buf[0] == ';') || (buf[0] == '#')) 
-		continue;
-			
-	    /* Find the key and value, and look for malformed lines */
-	    value = strchr(buf, '=');
-			
-	    if ((!value) || (value == buf))
-		printf("Parse error reading prefs: %s", buf);
-			
-	    /* Strip off whitespace */
-	    buf_start = buf;
-			
-	    while (g_ascii_isspace(*buf_start))
-		buf_start++;
-			
-	    /* Find the key name */
-	    key = g_strndup(buf, (value - buf_start));
-	    value++;
-			
-	    /* remove newline */
-	    len = strlen(value);
-			
-	    if ((len > 0) && (value[len - 1] == 0x0a))
-		value[len - 1] = 0;
-			
-	    /* Strip whitespace off the key value */
-	    while (g_ascii_isspace(*value))
-		value++;
-			
-	    /* Finally, load each key/value pair into the pref hash table */
-	    if (prefs_table)
-	    {
-		g_hash_table_insert(prefs_table, (gpointer)key, 
-				    (gpointer)g_strdup(value));
-	    }
+	    g_hash_table_insert (prefs_table, key, value);
 	}
     }
 }
@@ -571,8 +632,17 @@ static void read_prefs_from_file(FILE *fp)
 /* Write prefs to file */
 static void write_prefs_to_file(FILE *fp)
 {
-    if (prefs_table)
-	g_hash_table_foreach(prefs_table, write_key, (gpointer)fp);
+    lock_prefs_table ();
+
+    if (!prefs_table)
+    {
+	unlock_prefs_table ();
+	g_return_if_reached ();
+    }
+
+    g_hash_table_foreach(prefs_table, write_key, (gpointer)fp);
+
+    unlock_prefs_table ();
 }
 
 /* Load preferences, first loading the defaults, and then overwrite that with
@@ -615,8 +685,8 @@ static void load_prefs()
     set_default_list_entries();
 }
 
-/* Save preferences to user home folder (~/gtkpod/prefs) */
-static void save_prefs()
+/* Save preferences to user home folder (~/.gtkpod/prefs) */
+void prefs_save ()
 {
     gchar *filename;  /* Path of file to write to */
     gchar *config_dir;   /* Folder where prefs file is */
@@ -646,6 +716,98 @@ static void save_prefs()
     }
 }
 
+
+static gboolean temp_prefs_save_fe (gchar *key, gchar *value,
+				    struct temp_prefs_save *tps)
+{
+    gchar *buf;
+    GIOStatus status;
+
+    buf=g_strdup_printf ("%s=%s\n", key, value);
+    status = g_io_channel_write_chars (tps->gio, buf, -1, NULL, tps->error);
+    g_free (buf);
+    if (status != G_IO_STATUS_NORMAL)
+    {
+	tps->success = FALSE;
+	return TRUE;  /* stop traversal */
+    }
+    return FALSE;
+}
+
+
+/* Save @temp_prefs to @filename in the same manner as prefs_save is
+ * saving to ~/.gtkpod/prefs. @error: location where to store
+ * information about errors or NULL.
+ * 
+ * Return value: TRUE on success, FALSE if an error occured, in which
+ * case @error will be set accordingly.
+ */
+gboolean temp_prefs_save (TempPrefs *temp_prefs,
+			  const gchar *filename,
+			  GError **error)
+{
+    GIOChannel *gio;
+    struct temp_prefs_save tps;
+
+    g_return_val_if_fail (temp_prefs && filename, FALSE);
+
+    gio = g_io_channel_new_file (filename, "w", error);
+    tps.gio = gio;
+    tps.error = error;
+    tps.success = TRUE;
+    if (gio)
+    {
+	g_tree_foreach (temp_prefs->tree, (GTraverseFunc)temp_prefs_save_fe, &tps);
+	g_io_channel_unref (gio);
+    }
+
+    return tps.success;
+}
+
+
+TempPrefs *temp_prefs_load (const gchar *filename, GError **error)
+{
+    GIOChannel *gio;
+    TempPrefs *temp_prefs = NULL;
+
+    g_return_val_if_fail (filename, NULL);
+
+    gio = g_io_channel_new_file (filename, "r", error);
+    if (gio)
+    {
+	GIOStatus status;
+
+	temp_prefs = temp_prefs_create ();
+
+	do
+	{
+	    gchar *line;
+
+	    status = g_io_channel_read_line (gio, &line, NULL, NULL, error);
+	    if (status == G_IO_STATUS_NORMAL)
+	    {
+		gchar *key, *value;
+		if (read_prefs_get_key_value (line, &key, &value))
+		{
+		    temp_prefs_set_string (temp_prefs, key, value);
+		}
+		g_free (line);
+	    }
+	} while (status == G_IO_STATUS_NORMAL);
+
+	g_io_channel_unref (gio);
+
+	if (status != G_IO_STATUS_EOF)
+	{
+	    temp_prefs_destroy (temp_prefs);
+	    temp_prefs = NULL;
+	}
+    }
+
+    return temp_prefs;
+}
+
+
 /* Removes already existing list keys from the prefs table */
 static void wipe_list(const gchar *key)
 {
@@ -701,9 +863,24 @@ static void cleanup_keys()
 	prefs_set_string("coverart", NULL);
     }
   
+    /* rename tm_sort_ to tm_sort */
+    if (prefs_get_string_value("tm_sort_", &buf))
+    {
+	prefs_set_string("tm_sort", buf);
+	g_free(buf);
+	prefs_set_string("tm_sort_", NULL);
+    }
+
+    /* rename md5 to sha1 */
+    if (prefs_get_string_value("md5", &buf))
+    {
+	prefs_set_string("sha1", buf);
+	g_free(buf);
+	prefs_set_string("md5", NULL);
+    }
+
     /* Convert old path numbered keys to named ones */
   
-  
     /* Play Now */
     if (prefs_get_string_value_index("path", PATH_PLAY_NOW, &buf))
     {
@@ -854,7 +1031,15 @@ static void cleanup_keys()
     prefs_set_string("fix_path", NULL);
     prefs_set_string("write_gaintag", NULL);
     prefs_set_string("automount", NULL);
-
+    prefs_set_string("display_artcovers", NULL);
+    prefs_set_string("block_display", NULL);
+    prefs_set_string("tmp_disable_sort", NULL);
+    prefs_set_string("size_file_dialog.x", NULL);
+    prefs_set_string("size_file_dialog.y", NULL);
+    prefs_set_string("size_file_dialog_details.x", NULL);
+    prefs_set_string("size_file_dialog_details.y", NULL);
+    prefs_set_string("autoimport", NULL);
+    prefs_set_string("auto_import", NULL);
 
     /* sp_created_cond renamed to sp_added_cond */
     for (i = 0; i < SORT_TAB_MAX; i++)
@@ -926,13 +1111,6 @@ static void cleanup_keys()
 	prefs_set_string("size_info.y", NULL);
     }
     
-    /* auto_import reanmed to autoimport */
-    if (prefs_get_int_value("auto_import", &int_buf))
-    {
-	prefs_set_int("autoimport", int_buf);
-	prefs_set_string("auto_import", NULL);
-    }
-
     /* not_played_song renamed to not_played_track */
     if (prefs_get_int_value("not_played_song", &int_buf))
     {
@@ -964,7 +1142,7 @@ static void cleanup_keys()
     /* sm_sort_ renamed to tm_sort */
     if (prefs_get_int_value("sm_sort_", &int_buf))
     {
-	prefs_set_int("tm_sort_", int_buf);
+	prefs_set_int("tm_sort", int_buf);
 	prefs_set_string("sm_sort_", NULL);
     }
 
@@ -979,9 +1157,11 @@ static void cleanup_keys()
     /* This string was a wrong autoconvert--just ignore it */
     buf = prefs_get_string("export_template");
     
-    if (strcmp(buf, "%a - %a/%T - %T.mp3") == 0)
+    if (buf && strcmp(buf, "%a - %a/%T - %T.mp3") == 0)
 	prefs_set_string("export_template", NULL);
 
+    g_free (buf);
+
     /* We changed the meaning of the %x in export_template */
     if (version < 0.72)
     {
@@ -1013,27 +1193,66 @@ static void cleanup_keys()
     /* For versions < 0.91, remove all itdb keys */
     if (version < 0.91)
 	prefs_flush_subkey("itdb_");
-   
+
+    /* rename convert scripts */
+    if (version <= 0.99001)
+    {
+
+	const gchar **keyp;
+	const gchar *keys[] =
+	    {"path_conv_m4a", "path_conv_wav", "path_conv_mp3",
+	     "path_conv_ogg", "path_conv_flac", NULL};
+
+	for (keyp=keys; *keyp; ++keyp)
+	{
+	    buf = prefs_get_string (*keyp);
+	    if (buf)
+	    {
+		const gchar *needles[] =
+		    { "convert-flac2mp3.sh", "convert-m4a2mp3.sh",
+		      "convert-ogg2mp3.sh", "convert-wav2mp3.sh",
+		      "convert-flac2m4a.sh", "convert-m4a2m4a.sh",
+		      "convert-ogg2m4a.sh", "convert-wav2m4a.sh", NULL };
+		const gchar *replacements[] =
+		    { "convert-2mp3.sh    ", "convert-2mp3.sh   ",
+		      "convert-2mp3.sh   ", "convert-2mp3.sh   " ,
+		      "convert-2m4a.sh    ", "convert-2m4a.sh   ",
+		      "convert-2m4a.sh   ", "convert-2m4a.sh   " };
+		const gchar **needlep;
+		const gchar **replp;
+		replp = replacements;
+		for (needlep=needles; *needlep; ++needlep, ++replp)
+		{
+		    gchar *bufp = strstr (buf, *needlep);
+		    if (bufp)
+		    {
+			g_memmove (bufp, *replp, strlen (*replp));
+			prefs_set_string (*keyp, buf);
+			break;
+		    }
+		}
+		g_free (buf);
+	    }
+	}
+    }
+
     prefs_set_string ("version", VERSION);
 }
 
-/* Do things that need to be done before saving prefs */
-static void finalize_prefs()
-{
-  /* Sort column order needs to be stored */
-  tm_store_col_order();
-  
-  /* Update default sizes */
-  display_update_default_sizes();
-}
-
 /* Initialize the prefs table and read configuration */
-void init_prefs(int argc, char *argv[])
+void prefs_init (int argc, char *argv[])
 {
+    if (!prefs_table_mutex)
+	prefs_table_mutex = g_mutex_new ();
+
+    lock_prefs_table ();
+
     /* Create the prefs hash table */
     prefs_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free,
 					g_free);
-	
+
+    unlock_prefs_table ();
+
     /* Load preferences */
     load_prefs();
 
@@ -1047,28 +1266,31 @@ void init_prefs(int argc, char *argv[])
     read_commandline(argc, argv);  
 }
 
-/* Save prefs data to a file, and then delete the hash table */
-void cleanup_prefs()
+/* Delete the hash table */
+void prefs_shutdown ()
 {
-    if (prefs_table)
+    lock_prefs_table ();
+
+    if (!prefs_table)
     {
-	/* Let prefs do some things before saving */
-	finalize_prefs();
-    
-	/* Save prefs */
-	save_prefs();
-		
-	/* Delete the prefs hash table */
-	g_hash_table_destroy(prefs_table);
-	prefs_table = NULL;
+	unlock_prefs_table ();
+	g_return_if_reached ();
     }
+
+    /* Delete the prefs hash table */
+    g_hash_table_destroy(prefs_table);
+    prefs_table = NULL;
+
+    unlock_prefs_table ();
+
+    /* We can't free the prefs_table_mutex in a thread-safe way */
 }
 
 /* Create the temp prefs tree */
 /* Free the returned structure with delete_temp_prefs() */
-TempPrefs *temp_prefs_create()
+TempPrefs *temp_prefs_create ()
 {
-    TempPrefs *temp_prefs;  /* Retunred temp prefs structure */
+    TempPrefs *temp_prefs;  /* Returned temp prefs structure */
 
     temp_prefs = (TempPrefs*)g_malloc(sizeof(TempPrefs));
 
@@ -1079,7 +1301,7 @@ TempPrefs *temp_prefs_create()
 }
 
 /* Delete temp prefs */
-void temp_prefs_destroy(TempPrefs *temp_prefs)
+void temp_prefs_destroy (TempPrefs *temp_prefs)
 {
     g_return_if_fail (temp_prefs);
     g_return_if_fail (temp_prefs->tree);
@@ -1089,7 +1311,7 @@ void temp_prefs_destroy(TempPrefs *temp_prefs)
 }
 
 /* Copy key data from the temp prefs tree to the hash table */
-static gboolean copy_key(gpointer key, gpointer value, gpointer user_data)
+static gboolean copy_key (gpointer key, gpointer value, gpointer user_data)
 {
     prefs_set_string(key, value);
 	
@@ -1097,7 +1319,7 @@ static gboolean copy_key(gpointer key, gpointer value, gpointer user_data)
 }
 
 /* Copy the data from the temp prefs tree to the permanent prefs table */
-void temp_prefs_apply(TempPrefs *temp_prefs)
+void temp_prefs_apply (TempPrefs *temp_prefs)
 {
     g_return_if_fail (temp_prefs);
     g_return_if_fail (temp_prefs->tree);
@@ -1108,7 +1330,7 @@ void temp_prefs_apply(TempPrefs *temp_prefs)
 
 /* Create a temp_prefs tree containing a subset of keys in the
    permanent prefs table (those starting with @subkey */
-TempPrefs *prefs_create_subset (const gchar *subkey)
+static TempPrefs *prefs_create_subset_unlocked (const gchar *subkey)
 {
     struct sub_data sub_data;
 
@@ -1122,6 +1344,21 @@ TempPrefs *prefs_create_subset (const gchar *subkey)
     return sub_data.temp_prefs;
 }
 
+/* Create a temp_prefs tree containing a subset of keys in the
+   permanent prefs table (those starting with @subkey */
+TempPrefs *prefs_create_subset (const gchar *subkey)
+{
+    TempPrefs *temp_prefs;
+
+    lock_prefs_table ();
+
+    temp_prefs = prefs_create_subset_unlocked (subkey);
+
+    unlock_prefs_table ();
+
+    return temp_prefs;
+}
+
 
 /* Create a temp_prefs tree containing a subset of keys in the
    permanent prefs table (those starting with @subkey */
@@ -1144,12 +1381,16 @@ TempPrefs *temp_prefs_create_subset (TempPrefs *temp_prefs,
 
 /* Remove all keys in the temp prefs tree from the permanent prefs
    table */
-void temp_prefs_flush(TempPrefs *temp_prefs)
+void temp_prefs_flush (TempPrefs *temp_prefs)
 {
     g_return_if_fail (temp_prefs);
     g_return_if_fail (temp_prefs->tree);
 
+    lock_prefs_table ();
+
     g_tree_foreach (temp_prefs->tree, flush_key, NULL);
+
+    unlock_prefs_table ();
 }
 
 /* Return the number of keys stored in @temp_prefs */
@@ -1185,9 +1426,17 @@ gboolean temp_prefs_subkey_exists (TempPrefs *temp_prefs,
 /* Remove all keys that start with @subkey */
 void prefs_flush_subkey (const gchar *subkey)
 {
-    g_return_if_fail (prefs_table);
+    lock_prefs_table ();
+
+    if (!prefs_table)
+    {
+	unlock_prefs_table ();
+	g_return_if_reached ();
+    }
 
     g_hash_table_foreach_remove (prefs_table, match_subkey, (gchar *)subkey);
+
+    unlock_prefs_table ();
 }
 
 
@@ -1196,11 +1445,18 @@ void prefs_flush_subkey (const gchar *subkey)
 void prefs_rename_subkey (const gchar *subkey_old, const gchar *subkey_new){
     struct sub_data sub_data;
 
-    g_return_if_fail (prefs_table);
     g_return_if_fail (subkey_old);
     g_return_if_fail (subkey_new);
 
-    sub_data.temp_prefs = prefs_create_subset (subkey_old);
+    lock_prefs_table ();
+
+    if (!prefs_table)
+    {
+	unlock_prefs_table ();
+	g_return_if_reached ();
+    }
+
+    sub_data.temp_prefs = prefs_create_subset_unlocked (subkey_old);
     sub_data.temp_prefs_orig = NULL;
 
     if (temp_prefs_size (sub_data.temp_prefs) > 0)
@@ -1211,6 +1467,8 @@ void prefs_rename_subkey (const gchar *subkey_old, const gchar *subkey_new){
     }
 
     temp_prefs_destroy (sub_data.temp_prefs);
+
+    unlock_prefs_table ();
 }
 
 
@@ -1248,14 +1506,21 @@ void prefs_set_string(const gchar *key, const gchar *value)
 {
     g_return_if_fail (key);
 
-    if (prefs_table)
+    lock_prefs_table ();
+
+    if (!prefs_table)
     {
-	if (value)
-	    g_hash_table_insert (prefs_table,
-				 g_strdup(key), g_strdup(value));
-	else
-	    g_hash_table_remove (prefs_table, key);
+	unlock_prefs_table ();
+	g_return_if_reached ();
     }
+
+    if (value)
+	g_hash_table_insert (prefs_table,
+			     g_strdup(key), g_strdup(value));
+    else
+	g_hash_table_remove (prefs_table, key);
+
+    unlock_prefs_table ();
 }
 
 /* Set a key value to a given integer */
@@ -1263,11 +1528,18 @@ void prefs_set_int(const gchar *key, const gint value)
 {
     gchar *strvalue; /* String value converted from integer */
 
-    if (prefs_table)
+    lock_prefs_table ();
+
+    if (!prefs_table)
     {
-	strvalue = g_strdup_printf("%i", value);
-	g_hash_table_insert(prefs_table, g_strdup(key), strvalue);
+	unlock_prefs_table ();
+	g_return_if_reached ();
     }
+
+    strvalue = g_strdup_printf("%i", value);
+    g_hash_table_insert(prefs_table, g_strdup(key), strvalue);
+
+    unlock_prefs_table ();
 }
 
 /* Set a key to an int64 value */
@@ -1275,66 +1547,95 @@ void prefs_set_int64(const gchar *key, const gint64 value)
 {
     gchar *strvalue; /* String value converted from int64 */
 	
-    if (prefs_table)
+    lock_prefs_table ();
+
+    if (!prefs_table)
     {
-	strvalue = g_strdup_printf("%llu", value);
-	g_hash_table_insert(prefs_table, g_strdup(key), strvalue);	
+	unlock_prefs_table ();
+	g_return_if_reached ();
     }
+
+    strvalue = g_strdup_printf("%" G_GINT64_FORMAT, value);
+    g_hash_table_insert(prefs_table, g_strdup(key), strvalue);	
+
+    unlock_prefs_table ();
 }
 
 void prefs_set_double(const gchar *key, gdouble value)
 {
     gchar *strvalue; /* String value converted from integer */
 
-    if (prefs_table)
+    lock_prefs_table ();
+
+    if (!prefs_table)
     {
-	strvalue = g_strdup_printf("%f", value);
-	g_hash_table_insert(prefs_table, g_strdup(key), strvalue);
+	unlock_prefs_table ();
+	g_return_if_reached ();
     }
+
+    strvalue = g_strdup_printf("%f", value);
+    g_hash_table_insert(prefs_table, g_strdup(key), strvalue);
+
+    unlock_prefs_table ();
 }
 
 /* Get a string value associated with a key. Free returned string. */
 gchar *prefs_get_string(const gchar *key)
-{	
-    if (prefs_table)
-	return g_strdup(g_hash_table_lookup(prefs_table, key));
-    else
-	return NULL;
+{
+    gchar *string = NULL;
+
+    lock_prefs_table ();
+
+    g_return_val_if_fail (prefs_table, (unlock_prefs_table(), NULL));
+
+    string = g_strdup(g_hash_table_lookup(prefs_table, key));
+
+    unlock_prefs_table ();
+
+    return string;
 }
 
 /* Use this if you need to know if the given key actually exists */
 /* The value parameter can be NULL if you don't need the value itself. */
 gboolean prefs_get_string_value(const gchar *key, gchar **value)
 {
-    gchar *string;  /* String value from prefs table */
-	
-    if (prefs_table)
-    {
-	string = g_hash_table_lookup(prefs_table, key);
+    const gchar *string;  /* String value from prefs table */
+    gboolean valid = FALSE;
+
+    lock_prefs_table ();
+
+    g_return_val_if_fail (prefs_table, (unlock_prefs_table(), FALSE));
+
+    string = g_hash_table_lookup(prefs_table, key);
 		
-	if (value)
-	    *value = g_strdup (string);
-	if (string)
-	    return TRUE;
-    }
-    return FALSE;
+    if (value)
+	*value = g_strdup (string);
+    if (string)
+	valid = TRUE;
+
+    unlock_prefs_table ();
+
+    return valid;
 }
 
 /* Get an integer value from a key */
 gint prefs_get_int(const gchar *key)
 {
     gchar *string; /* Hash value string */
-    gint value;  /* Retunred value */
+    gint value;  /* Returned value */
 	
     value = 0;
 	
-    if (prefs_table)
-    {
-	string = g_hash_table_lookup(prefs_table, key);
+    lock_prefs_table ();
+
+    g_return_val_if_fail (prefs_table, (unlock_prefs_table(), value));
+
+    string = g_hash_table_lookup(prefs_table, key);
 		
-	if (string)
-	    value = atoi(string);
-    }
+    if (string)
+	value = atoi(string);
+
+    unlock_prefs_table ();
 
     return value;
 }
@@ -1344,40 +1645,48 @@ gint prefs_get_int(const gchar *key)
 gboolean prefs_get_int_value(const gchar *key, gint *value)
 {
     gchar *string;  /* String value from prefs table */
-	
-    if (prefs_table)
+    gboolean valid = FALSE;
+
+    lock_prefs_table ();
+
+    g_return_val_if_fail (prefs_table, (unlock_prefs_table(), FALSE));
+
+    string = g_hash_table_lookup(prefs_table, key);
+
+    if (value)
     {
-	string = g_hash_table_lookup(prefs_table, key);
-
-	if (value)
-	{
-	    if (string)
-		*value = atoi(string);
-	    else
-		*value = 0;
-	}
-
 	if (string)
-	    return TRUE;
+	    *value = atoi(string);
+	else
+	    *value = 0;
     }
-    return FALSE;
+
+    if (string)
+	valid = TRUE;
+
+    unlock_prefs_table ();
+
+    return valid;
 }
 
 /* Get a 64 bit integer value from a key */
 gint64 prefs_get_int64(const gchar *key)
 {
     gchar *string;  /* Key value string */
-    gint64 value;  /* Retunred value */
+    gint64 value;  /* Returned value */
 	
     value = 0;
 
-    if (prefs_table)
-    {
-	string = g_hash_table_lookup(prefs_table, key);
+    lock_prefs_table ();
 
-	if (string)
-	    value = g_ascii_strtoull(string, NULL, 10);
-    }
+    g_return_val_if_fail (prefs_table, (unlock_prefs_table(), value));
+
+    string = g_hash_table_lookup(prefs_table, key);
+
+    if (string)
+	value = g_ascii_strtoull(string, NULL, 10);
+
+    unlock_prefs_table ();
 	
     return value;
 }
@@ -1388,36 +1697,47 @@ gint64 prefs_get_int64(const gchar *key)
 gboolean prefs_get_int64_value(const gchar *key, gint64 *value)
 {
     gchar *string;  /* String value from prefs table */
-	
-    if (prefs_table)
-    {
-	string = g_hash_table_lookup(prefs_table, key);
+    gboolean valid = FALSE;
+
+    lock_prefs_table ();
+
+    g_return_val_if_fail (prefs_table, (unlock_prefs_table(), FALSE));
+
+    string = g_hash_table_lookup(prefs_table, key);
 		
+    if (value)
+    {
 	if (string)
-	{
-	    if (value)
-		*value = g_ascii_strtoull(string, NULL, 10);
-			
-	    return TRUE;
-	}
+	    *value = g_ascii_strtoull(string, NULL, 10);
+	else
+	    *value = 0;
     }
-    return FALSE;
+
+    if (string)
+	valid = TRUE;
+
+    unlock_prefs_table ();
+
+    return valid;
 }
 
 gdouble prefs_get_double(const gchar *key)
 {
     gchar *string;  /* Key value string */
-    gdouble value;  /* Retunred value */
+    gdouble value;  /* Returned value */
 	
     value = 0;
 
-    if (prefs_table)
-    {
-	string = g_hash_table_lookup(prefs_table, key);
+    lock_prefs_table ();
 
-	if (string)
-	    value = g_ascii_strtod(string, NULL);
-    }
+    g_return_val_if_fail (prefs_table, (unlock_prefs_table(), value));
+
+    string = g_hash_table_lookup(prefs_table, key);
+
+    if (string)
+	value = g_ascii_strtod(string, NULL);
+
+    unlock_prefs_table ();
 	
     return value;
 }
@@ -1425,20 +1745,28 @@ gdouble prefs_get_double(const gchar *key)
 gboolean prefs_get_double_value(const gchar *key, gdouble *value)
 {
     gchar *string;  /* String value from prefs table */
-	
-    if (prefs_table)
+    gboolean valid = FALSE;
+
+    lock_prefs_table ();
+
+    g_return_val_if_fail (prefs_table, (unlock_prefs_table(), FALSE));
+
+    string = g_hash_table_lookup(prefs_table, key);
+
+    if (value)
     {
-	string = g_hash_table_lookup(prefs_table, key);
-		
 	if (string)
-	{
-	    if (value)
-		*value = g_ascii_strtod(string, NULL);
-			
-	    return TRUE;
-	}
+	    *value = g_ascii_strtod(string, NULL);
+	else
+	    *value = 0;
     }
-    return FALSE;
+
+    if (string)
+	valid = TRUE;
+
+    unlock_prefs_table ();
+
+    return valid;
 }
 
 /* Functions for numbered pref keys */
@@ -1604,7 +1932,7 @@ gboolean prefs_get_double_value_index(const gchar *key, guint index,
  * temp_prefs_remove_key() to remove key from the temp prefs. Setting
  * it to NULL will not remove the key. It will instead remove the key
  * in the main prefs table when you call temp_prefs_apply(). */
-void temp_prefs_set_string(TempPrefs *temp_prefs, const gchar *key, 
+void temp_prefs_set_string (TempPrefs *temp_prefs, const gchar *key, 
 			   const gchar *value)
 {
     g_return_if_fail (temp_prefs && temp_prefs->tree);
@@ -1645,7 +1973,7 @@ void temp_prefs_set_int64(TempPrefs *temp_prefs, const gchar *key,
     g_return_if_fail (temp_prefs && temp_prefs->tree);
     g_return_if_fail (key);
 
-    strvalue = g_strdup_printf("%llu", value);
+    strvalue = g_strdup_printf("%" G_GINT64_FORMAT, value);
     g_tree_insert(temp_prefs->tree, g_strdup(key), strvalue);
 }
 
@@ -1695,7 +2023,7 @@ gboolean temp_prefs_get_string_value(TempPrefs *temp_prefs,
 gint temp_prefs_get_int(TempPrefs *temp_prefs, const gchar *key)
 {
     gchar *string; /* Hash value string */
-    gint value;  /* Retunred value */
+    gint value;  /* Returned value */
 	
     g_return_val_if_fail (temp_prefs && temp_prefs->tree, 0);
     g_return_val_if_fail (key, 0);
@@ -1740,7 +2068,7 @@ gdouble temp_prefs_get_double(TempPrefs *temp_prefs,
 			      const gchar *key)
 {
     gchar *string; /* Hash value string */
-    gdouble value;  /* Retunred value */
+    gdouble value;  /* Returned value */
 	
     g_return_val_if_fail (temp_prefs && temp_prefs->tree, 0);
     g_return_val_if_fail (key, 0);
@@ -1911,7 +2239,7 @@ GList *prefs_get_list(const gchar *key)
 {
     guint end_marker_hash;  /* Hash value of the list end marker */
     guint item_hash;  /* Hash value of current list string */
-    gchar *item_string;  /* List iterm string */
+    gchar *item_string;  /* List item string */
     guint i;  /* Counter */
     GList *list;  /* List that contains items */
 	
