@@ -1,6 +1,5 @@
-/* Time-stamp: <2005-07-16 01:08:24 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 |
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: tools.c,v 1.42 2005/10/18 14:29:55 jcsjcs Exp $
+|  $Id: tools.c 1684 2007-08-18 06:38:49Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -32,16 +31,20 @@
 #endif
 
 
-#include <sys/wait.h>
-#include <string.h>
-#include <unistd.h>
-#include <errno.h>
 #include "info.h"
 #include "misc.h"
+#include "misc_track.h"
 #include "mp3file.h"
-#include "tools.h"
 #include "prefs.h"
+#include "tools.h"
+#include <errno.h>
+#include <fcntl.h>
 #include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <unistd.h>
 
 
 /*pipe's definition*/
@@ -67,21 +70,229 @@ static GCond  *cond = NULL;
 static gboolean mutex_data = FALSE;
 #endif
 
-/* will get the volume either from mp3gain or from LAME's ReplayGain */
-static gint32 nm_get_soundcheck (Track *track)
+
+/* Run @command on @track_path.
+ *
+ * Command may include options, like "mp3gain -q -k %s"
+ *
+ * %s is replaced by @track_path if present, otherwise @track_path is
+ * added at the end.
+ *
+ * Return value: TRUE if the command ran successfully, FALSE if any
+ * error occurred.
+ */
+static gboolean run_exec_on_track (const gchar *commandline,
+				   const gchar *track_path)
 {
-    ExtraTrackData *etr;
-    gint32 sc = TRACKVOLERROR;
+    gchar *command_full_path = NULL;
+    gchar *command = NULL;
+    gchar *command_base = NULL;
+    const gchar *nextarg;
+    gboolean success = FALSE;
+    gboolean percs = FALSE;
+    GPtrArray *args;
 
-    g_return_val_if_fail (track, sc);
-    etr = track->userdata;
-    g_return_val_if_fail (etr, sc);
+    int status, fdnull, ret;
+    pid_t tpid;
 
-    if (!etr->radio_gain_set)
-	get_gain (track);
-    if (etr->radio_gain_set) 
-	sc = replaygain_to_soundcheck (etr->radio_gain);
-    return sc;
+    g_return_val_if_fail (commandline, FALSE);
+    g_return_val_if_fail (track_path, FALSE);
+
+
+    /* skip whitespace */
+    while (g_ascii_isspace (*commandline))  ++commandline;
+
+    /* find the command itself -- separated by ' ' */
+    nextarg = strchr (commandline, ' ');
+    if (!nextarg)
+    {
+	nextarg = commandline + strlen (commandline);
+    }
+
+    command = g_strndup (commandline, nextarg-commandline);
+
+    command_full_path = g_find_program_in_path (command);
+
+    if (!command_full_path)
+    {
+	gtkpod_warning (_("Could not find '%s'.\nPlease specifiy the exact path in the Tools section of the preference dialog or install the program if it is not installed on your system.\n\n"), command);
+	goto cleanup;
+    }
+
+    command_base = g_path_get_basename (command_full_path);
+
+    /* Create the command line to be used with execv(). */
+    args =  g_ptr_array_sized_new (strlen (commandline));
+    /* add the full path */
+    g_ptr_array_add (args, command_full_path);
+    /* add the basename */
+    g_ptr_array_add (args, command_base);
+    /* add the command line arguments */
+
+    commandline = nextarg;
+
+    /* skip whitespace */
+    while (g_ascii_isspace (*commandline))  ++commandline;
+
+    while (*commandline != 0)
+    {
+	const gchar *next;
+
+	next = strchr (commandline, ' ');
+	/* next argument is everything to the end */
+	if (!next)
+	    next = commandline + strlen (commandline);
+
+	if (strncmp (commandline, "%s", 2) == 0)
+	{   /* substitute %s with @track_path */
+	    g_ptr_array_add (args, g_strdup (track_path));
+	    percs = TRUE;
+	}
+	else
+	{
+	    g_ptr_array_add (args,
+			     g_strndup (commandline, next-commandline));
+	}
+
+	/* skip to next argument */
+	commandline = next;
+
+	/* skip whitespace */
+	while (g_ascii_isspace (*commandline))  ++commandline;
+    } 
+
+    /* Add @track_path if "%s" was not present */
+    if (!percs)
+	g_ptr_array_add (args, g_strdup (track_path));
+
+    /* need NULL pointer */
+    g_ptr_array_add (args, NULL);
+
+    tpid = fork ();
+
+    switch (tpid)
+    {
+    case 0: /* we are the child */
+    {
+	gchar **argv = (gchar **)args->pdata;
+#if 0
+	gchar **bufp = argv;
+	while (*bufp)	{ puts (*bufp); ++bufp;	}
+#endif
+	/* redirect output to /dev/null */
+	if ((fdnull = open("/dev/null", O_WRONLY | O_NDELAY)) != -1)
+	{
+	    dup2(fdnull, fileno(stdout));
+	}
+	execv(argv[0], &argv[1]);
+	exit(0);
+	break;
+    }
+    case -1: /* we are the parent, fork() failed  */
+	g_ptr_array_free (args, TRUE);
+	break;
+    default: /* we are the parent, everything's fine */
+	tpid = waitpid (tpid, &status, 0);
+	g_ptr_array_free (args, TRUE);
+	if (WIFEXITED(status))
+	    ret = WEXITSTATUS(status);
+	else
+	    ret = 2;
+	if (ret > 1)
+	{
+	    gtkpod_warning (_("Execution of '%s' failed.\n\n"),
+			    command_full_path);
+	}
+	else
+	{
+	    success = TRUE;
+	}
+	break;
+    }
+
+
+  cleanup:
+    g_free (command_full_path);
+    g_free (command);
+    g_free (command_base);
+ 
+    return success;
+}
+
+
+
+
+/* reread the soundcheck value from the file */
+static gboolean nm_get_soundcheck (Track *track)
+{
+    gboolean success = FALSE;
+    gchar *path;
+    gchar *commandline = NULL;
+
+    g_return_val_if_fail (track, FALSE);
+
+    if (read_soundcheck (track))
+	return TRUE;
+
+    path = get_file_name_from_source (track, SOURCE_PREFER_LOCAL);
+
+    if (path)
+    {
+	switch (determine_file_type (path))
+	{
+	case FILE_TYPE_MP3: 
+	    commandline = prefs_get_string ("path_mp3gain");
+	    if (!commandline)
+	    {
+		gtkpod_warning (
+		    _("Did not normalize '%s'. Set mp3gain path in the Tools section of the preferences.\n"), path);
+	    }
+	    break;
+	case FILE_TYPE_M4A:
+	case FILE_TYPE_M4P:
+	case FILE_TYPE_M4B:
+	    commandline = prefs_get_string ("path_aacgain");
+	    if (!commandline)
+	    {
+		gtkpod_warning (
+		    _("Did not normalize '%s'. Set aacgain path in the Tools section of the preferences.\n"), path);
+	    }
+	    break;
+	case FILE_TYPE_WAV: /* FIXME */
+	case FILE_TYPE_OGG: /* FIXME */
+	case FILE_TYPE_FLAC: /* FIXME */
+	case FILE_TYPE_M4V:
+	case FILE_TYPE_MP4:
+	case FILE_TYPE_MOV:
+	case FILE_TYPE_MPG:
+	case FILE_TYPE_UNKNOWN:
+	    gtkpod_warning (
+		_("Normalization failed: file type not supported (%s).\n\n"),
+		path);
+	    break;
+	case FILE_TYPE_M3U: 
+	case FILE_TYPE_PLS: 
+	case FILE_TYPE_IMAGE: 
+	case FILE_TYPE_DIRECTORY:
+	    break;
+	}
+	if (commandline)
+	    success = run_exec_on_track (commandline, path);
+	g_free (path);
+    }
+    else
+    {
+	gchar *buf = get_track_info (track, FALSE);
+	gtkpod_warning (
+	    _("Normalization failed: file not available (%s).\n\n"),
+	    buf);
+	g_free (buf);
+    }
+
+    if (success)
+	return read_soundcheck (track);
+    else
+	return FALSE;
 }
 
 
@@ -90,12 +301,12 @@ static gint32 nm_get_soundcheck (Track *track)
 /* Threaded getTrackGain*/
 static gpointer th_nm_get_soundcheck (gpointer track)
 {
-   gint32 gain=nm_get_soundcheck ((Track *)track);
+   gboolean success = nm_get_soundcheck ((Track *)track);
    g_mutex_lock (mutex);
    mutex_data = TRUE; /* signal that thread will end */
    g_cond_signal (cond);
    g_mutex_unlock (mutex);
-   return GUINT_TO_POINTER(gain);
+   return GUINT_TO_POINTER(success);
 }
 #endif
 
@@ -128,12 +339,13 @@ static void normalization_abort(gboolean *abort)
 void nm_tracks_list (GList *list)
 {
   gint count, succ_count, n, nrs;
-  gint32 new_soundcheck = 0;
+  guint32 old_soundcheck;
+  gboolean success;
   static gboolean abort;
   GtkWidget *dialog, *progress_bar, *label, *track_label;
   GtkWidget *image, *hbox;
   time_t diff, start, fullsecs, hrs, mins, secs;
-  gchar *buf, *progtext = NULL;
+  gchar *progtext = NULL;
 
 #ifdef G_THREADS_ENABLED
   GThread *thread = NULL;
@@ -211,23 +423,25 @@ void nm_tracks_list (GList *list)
   else
   {
       /* we need ***much*** longer timeout */
-      prefs_set_statusbar_timeout (30*STATUSBAR_TIMEOUT);
+      gtkpod_statusbar_timeout (30*STATUSBAR_TIMEOUT);
   }
   while (!abort &&  (list!=NULL))
   {
      Track  *track = list->data;
      gchar *label_buf = g_strdup_printf ("%d/%d", count, n);
-     gchar *sb_buf = g_strdup_printf (_("%s - %s"),
-				      track->artist, track->title);
 
      gtk_label_set_text (GTK_LABEL (track_label), label_buf);
-     gtkpod_statusbar_message(sb_buf);
+
+     gtkpod_statusbar_message (_("%s - %s"),
+			       track->artist, track->title);
      C_FREE (label_buf);
-     C_FREE (sb_buf);
 
      while (widgets_blocked && gtk_events_pending ())
 	 gtk_main_iteration ();
 
+     /* need to know so we can update the display when necessary */
+     old_soundcheck = track->soundcheck;
+
 #ifdef G_THREADS_ENABLED
      mutex_data = FALSE;
      thread = g_thread_create (th_nm_get_soundcheck, track, TRUE, NULL);
@@ -257,35 +471,32 @@ void nm_tracks_list (GList *list)
 	     g_cond_timed_wait (cond, mutex, &gtime);
 	 }
 	 while(!mutex_data);
-	 new_soundcheck = GPOINTER_TO_UINT(g_thread_join (thread));
+	 success = GPOINTER_TO_UINT(g_thread_join (thread));
 	 g_mutex_unlock (mutex);
      }
      else
      {
 	 g_warning ("Thread creation failed, falling back to default.\n");
-	 new_soundcheck = nm_get_soundcheck (track);
+	 success = nm_get_soundcheck (track);
      }
 #else
-     new_soundcheck = nm_get_soundcheck (track);
+     success = nm_get_soundcheck (track);
 #endif
 
      /*normalization part*/
-     if(new_soundcheck == TRACKVOLERROR)
+     if(!success)
      {
-	 gchar *path = get_file_name_verified (track);
-	 gchar *buf = g_strdup_printf (
+	 gchar *path = get_file_name_from_source (track, SOURCE_PREFER_LOCAL);
+	 gtkpod_warning (
 	     _("'%s-%s' (%s) could not be normalized.\n\n"),
 	     track->artist, track->title, path? path:"");
-	 gtkpod_warning (buf);
 	 g_free (path);
-	 g_free (buf);
      }
      else
      {
 	 ++succ_count;
-	 if(new_soundcheck != track->soundcheck)
+	 if(old_soundcheck != track->soundcheck)
 	 {
-	     track->soundcheck = new_soundcheck;
 	     pm_track_changed (track);
 	     data_changed (track->itdb);
 	 }
@@ -314,13 +525,11 @@ void nm_tracks_list (GList *list)
      list=g_list_next(list);
   } /*end while*/
 
-  prefs_set_statusbar_timeout (0);
+  gtkpod_statusbar_timeout (0);
 
-  buf = g_strdup_printf (ngettext ("Normalized %d of %d tracks.",
-				   "Normalized %d of %d tracks.", n),
-			 count, n);
-  gtkpod_statusbar_message(buf);
-  g_free (buf);
+  gtkpod_statusbar_message (ngettext ("Normalized %d of %d tracks.",
+				      "Normalized %d of %d tracks.", n),
+			    count, n);
 
   gtk_widget_destroy (dialog);
   release_widgets();
@@ -342,13 +551,22 @@ typedef enum
 } SyncType;
 
 
+/* FIXME: tools need to be defined for each itdb separately */
 /* replace %i in all strings of argv with prefs_get_ipod_mount() */
-static void tools_sync_replace_percent_i (gchar **argv)
+static void tools_sync_replace_percent_i (iTunesDB *itdb, gchar **argv)
 {
-    const gchar *ipod_mount = prefs_get_ipod_mount ();
-    gint ipod_mount_len = strlen (ipod_mount);
+    gchar *ipod_mount;
+    gint ipod_mount_len;
     gint offset = 0;
 
+    if (!itdb) return;
+
+    ipod_mount = get_itdb_prefs_string (itdb, KEY_MOUNTPOINT);
+
+    if (!ipod_mount) ipod_mount = g_strdup ("");
+
+    ipod_mount_len = strlen (ipod_mount);
+
     while (argv && *argv)
     {
 	gchar *str = *argv;
@@ -374,12 +592,14 @@ static void tools_sync_replace_percent_i (gchar **argv)
 	    ++argv;
 	}
     }
+
+    g_free (ipod_mount);
 }
 
 
 /* execute the specified script, giving out error/status messages on
    the way */
-static gboolean tools_sync_script (SyncType type)
+static gboolean tools_sync_script (iTunesDB *itdb, SyncType type)
 {
     gchar *script=NULL;
     gchar *script_path, *buf;
@@ -392,13 +612,13 @@ static gboolean tools_sync_script (SyncType type)
     switch (type)
     {
     case SYNC_CONTACTS:
-	script = g_strdup (prefs_get_path (PATH_SYNC_CONTACTS));
+	script = get_itdb_prefs_string (itdb, "path_sync_contacts");
 	break;
     case SYNC_CALENDAR:
-	script = g_strdup (prefs_get_path (PATH_SYNC_CALENDAR));
+	script = get_itdb_prefs_string (itdb, "path_sync_calendar");
 	break;
     case SYNC_NOTES:
-	script = g_strdup (prefs_get_path (PATH_SYNC_NOTES));
+	script = get_itdb_prefs_string (itdb, "path_sync_notes");
 	break;
     default:
 	fprintf (stderr, "Programming error: tools_sync_script () called with %d\n", type);
@@ -417,7 +637,7 @@ static gboolean tools_sync_script (SyncType type)
 
     argv = g_strsplit (script, " ", -1);
 
-    tools_sync_replace_percent_i (argv);
+    tools_sync_replace_percent_i (itdb, argv);
 
     script_path = g_find_program_in_path (argv[0]);
     if (!script_path)
@@ -437,7 +657,7 @@ static gboolean tools_sync_script (SyncType type)
 
     /*create the pipe*/
     pipe(fdpipe);
-    /*than fork*/
+    /*then fork*/
     pid=fork();
 
     /*and cast mp3gain*/
@@ -448,6 +668,7 @@ static gboolean tools_sync_script (SyncType type)
     case 0: /*child*/
 	close(fdpipe[READ]);
 	dup2(fdpipe[WRITE],fileno(stdout));
+	dup2(fdpipe[WRITE],fileno(stderr));
 	close(fdpipe[WRITE]);
 	execv(script_path, argv);
 	break;
@@ -477,33 +698,31 @@ static gboolean tools_sync_script (SyncType type)
     return TRUE;
 }
 
-gboolean tools_sync_all (void)
+gboolean tools_sync_all (iTunesDB *itdb)
 {
-    gboolean x;
-    x = tools_sync_script (SYNC_CALENDAR);
-    if (!x) {
-	return x;
-    }
-    tools_sync_script (SYNC_CONTACTS);
-    if (!x) {
-	return x;
-    }
-    return tools_sync_script (SYNC_NOTES);
+    gboolean success;
+
+    success = tools_sync_script (itdb, SYNC_CALENDAR);
+    if (success)
+	success = tools_sync_script (itdb, SYNC_CONTACTS);
+    if (success)
+	tools_sync_script (itdb, SYNC_NOTES);
+    return success;
 }
 
-gboolean tools_sync_contacts (void)
+gboolean tools_sync_contacts (iTunesDB *itdb)
 {
-    return tools_sync_script (SYNC_CONTACTS);
+    return tools_sync_script (itdb, SYNC_CONTACTS);
 }
 
-gboolean tools_sync_calendar (void)
+gboolean tools_sync_calendar (iTunesDB *itdb)
 {
-    return tools_sync_script (SYNC_CALENDAR);
+    return tools_sync_script (itdb, SYNC_CALENDAR);
 }
 
-gboolean tools_sync_notes (void)
+gboolean tools_sync_notes (iTunesDB *itdb)
 {
-    return tools_sync_script (SYNC_NOTES);
+    return tools_sync_script (itdb, SYNC_NOTES);
 }
 
 /* ------------------------------------------------------------
@@ -532,12 +751,12 @@ do_command_on_entries (const gchar *command, const gchar *what,
 
     if ((!command) || (strlen (command) == 0))
     {
-	gchar *buf = g_strdup_printf (_("No command set for '%s'"), what);
-	gtkpod_statusbar_message (buf);
-	C_FREE (buf);
+	gtkpod_statusbar_message  (_("No command set for '%s'"), what);
 	return;
     }
 
+    while (g_ascii_isspace (*command))  ++command;
+
     /* find the command itself -- separated by ' ' */
     next = strchr (command, ' ');
     if (!next)
@@ -548,16 +767,13 @@ do_command_on_entries (const gchar *command, const gchar *what,
     {
 	str = g_strndup (command, next-command);
     }
-    while (g_ascii_isspace (*command))  ++command;
     /* get the full path */
     commandc = g_find_program_in_path (str);
     if (!commandc)
     {
-	gchar *buf = g_strdup_printf (_("Could not find command '%s' specified for '%s'"),
-				      str, what);
-	gtkpod_statusbar_message (buf);
-	C_FREE (buf);
-	C_FREE (str);
+	gtkpod_statusbar_message  (_("Could not find command '%s' specified for '%s'"),
+				   str, what);
+	g_free (str);
 	return;
     }
     C_FREE (str);
@@ -587,7 +803,10 @@ do_command_on_entries (const gchar *command, const gchar *what,
 	{
 	    for(l = selected_tracks; l; l = l->next)
 	    {
-		if((str = get_file_name_verified((Track*)l->data)))
+		Track *tr = l->data;
+		g_return_if_fail (tr);
+		str = get_file_name_from_source(tr, SOURCE_PREFER_LOCAL);
+		if(str)
 		{
 		    g_ptr_array_add (args, str);
 		}
@@ -630,9 +849,11 @@ do_command_on_entries (const gchar *command, const gchar *what,
  */
 void tools_play_tracks (GList *selected_tracks)
 {
-    do_command_on_entries (prefs_get_path (PATH_PLAY_NOW),
+    gchar *path = prefs_get_string ("path_play_now");
+    do_command_on_entries (path,
 			   _("Play Now"),
 			   selected_tracks);
+    g_free (path);
 }
 
 /*
@@ -641,9 +862,11 @@ void tools_play_tracks (GList *selected_tracks)
  */
 void tools_enqueue_tracks (GList *selected_tracks)
 {
-    do_command_on_entries (prefs_get_path (PATH_PLAY_ENQUEUE),
+    gchar *path = prefs_get_string ("path_play_enqueue");
+    do_command_on_entries (path,
 			   _("Enqueue"),
 			   selected_tracks);
+    g_free (path);
 }
 
 
